// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package azcertificates

// BackupCertificateOptions contains the optional parameters for the Client.BackupCertificate method.
type BackupCertificateOptions struct {
	// placeholder for future optional parameters
}

// CreateCertificateOptions contains the optional parameters for the Client.CreateCertificate method.
type CreateCertificateOptions struct {
	// placeholder for future optional parameters
}

// DeleteCertificateOperationOptions contains the optional parameters for the Client.DeleteCertificateOperation method.
type DeleteCertificateOperationOptions struct {
	// placeholder for future optional parameters
}

// DeleteCertificateOptions contains the optional parameters for the Client.DeleteCertificate method.
type DeleteCertificateOptions struct {
	// placeholder for future optional parameters
}

// DeleteContactsOptions contains the optional parameters for the Client.DeleteContacts method.
type DeleteContactsOptions struct {
	// placeholder for future optional parameters
}

// DeleteIssuerOptions contains the optional parameters for the Client.DeleteIssuer method.
type DeleteIssuerOptions struct {
	// placeholder for future optional parameters
}

// GetCertificateOperationOptions contains the optional parameters for the Client.GetCertificateOperation method.
type GetCertificateOperationOptions struct {
	// placeholder for future optional parameters
}

// GetCertificateOptions contains the optional parameters for the Client.GetCertificate method.
type GetCertificateOptions struct {
	// placeholder for future optional parameters
}

// GetCertificatePolicyOptions contains the optional parameters for the Client.GetCertificatePolicy method.
type GetCertificatePolicyOptions struct {
	// placeholder for future optional parameters
}

// GetContactsOptions contains the optional parameters for the Client.GetContacts method.
type GetContactsOptions struct {
	// placeholder for future optional parameters
}

// GetDeletedCertificateOptions contains the optional parameters for the Client.GetDeletedCertificate method.
type GetDeletedCertificateOptions struct {
	// placeholder for future optional parameters
}

// GetIssuerOptions contains the optional parameters for the Client.GetIssuer method.
type GetIssuerOptions struct {
	// placeholder for future optional parameters
}

// ImportCertificateOptions contains the optional parameters for the Client.ImportCertificate method.
type ImportCertificateOptions struct {
	// placeholder for future optional parameters
}

// ListCertificatePropertiesOptions contains the optional parameters for the Client.NewListCertificatePropertiesPager method.
type ListCertificatePropertiesOptions struct {
	// Specifies whether to include certificates which are not completely provisioned.
	IncludePending *bool
}

// ListCertificatePropertiesVersionsOptions contains the optional parameters for the Client.NewListCertificatePropertiesVersionsPager
// method.
type ListCertificatePropertiesVersionsOptions struct {
	// placeholder for future optional parameters
}

// ListDeletedCertificatePropertiesOptions contains the optional parameters for the Client.NewListDeletedCertificatePropertiesPager
// method.
type ListDeletedCertificatePropertiesOptions struct {
	// Specifies whether to include certificates which are not completely provisioned.
	IncludePending *bool
}

// ListIssuerPropertiesOptions contains the optional parameters for the Client.NewListIssuerPropertiesPager method.
type ListIssuerPropertiesOptions struct {
	// placeholder for future optional parameters
}

// MergeCertificateOptions contains the optional parameters for the Client.MergeCertificate method.
type MergeCertificateOptions struct {
	// placeholder for future optional parameters
}

// PurgeDeletedCertificateOptions contains the optional parameters for the Client.PurgeDeletedCertificate method.
type PurgeDeletedCertificateOptions struct {
	// placeholder for future optional parameters
}

// RecoverDeletedCertificateOptions contains the optional parameters for the Client.RecoverDeletedCertificate method.
type RecoverDeletedCertificateOptions struct {
	// placeholder for future optional parameters
}

// RestoreCertificateOptions contains the optional parameters for the Client.RestoreCertificate method.
type RestoreCertificateOptions struct {
	// placeholder for future optional parameters
}

// SetContactsOptions contains the optional parameters for the Client.SetContacts method.
type SetContactsOptions struct {
	// placeholder for future optional parameters
}

// SetIssuerOptions contains the optional parameters for the Client.SetIssuer method.
type SetIssuerOptions struct {
	// placeholder for future optional parameters
}

// UpdateCertificateOperationOptions contains the optional parameters for the Client.UpdateCertificateOperation method.
type UpdateCertificateOperationOptions struct {
	// placeholder for future optional parameters
}

// UpdateCertificateOptions contains the optional parameters for the Client.UpdateCertificate method.
type UpdateCertificateOptions struct {
	// placeholder for future optional parameters
}

// UpdateCertificatePolicyOptions contains the optional parameters for the Client.UpdateCertificatePolicy method.
type UpdateCertificatePolicyOptions struct {
	// placeholder for future optional parameters
}

// UpdateIssuerOptions contains the optional parameters for the Client.UpdateIssuer method.
type UpdateIssuerOptions struct {
	// placeholder for future optional parameters
}
