// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: inspect/inspect.proto

package inspect

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	emptypb "google.golang.org/protobuf/types/known/emptypb"

	common "github.com/siderolabs/talos/pkg/machinery/api/common"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type DependencyEdgeType int32

const (
	DependencyEdgeType_OUTPUT_EXCLUSIVE    DependencyEdgeType = 0
	DependencyEdgeType_OUTPUT_SHARED       DependencyEdgeType = 3
	DependencyEdgeType_INPUT_STRONG        DependencyEdgeType = 1
	DependencyEdgeType_INPUT_WEAK          DependencyEdgeType = 2
	DependencyEdgeType_INPUT_DESTROY_READY DependencyEdgeType = 4
)

// Enum value maps for DependencyEdgeType.
var (
	DependencyEdgeType_name = map[int32]string{
		0: "OUTPUT_EXCLUSIVE",
		3: "OUTPUT_SHARED",
		1: "INPUT_STRONG",
		2: "INPUT_WEAK",
		4: "INPUT_DESTROY_READY",
	}
	DependencyEdgeType_value = map[string]int32{
		"OUTPUT_EXCLUSIVE":    0,
		"OUTPUT_SHARED":       3,
		"INPUT_STRONG":        1,
		"INPUT_WEAK":          2,
		"INPUT_DESTROY_READY": 4,
	}
)

func (x DependencyEdgeType) Enum() *DependencyEdgeType {
	p := new(DependencyEdgeType)
	*p = x
	return p
}

func (x DependencyEdgeType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (DependencyEdgeType) Descriptor() protoreflect.EnumDescriptor {
	return file_inspect_inspect_proto_enumTypes[0].Descriptor()
}

func (DependencyEdgeType) Type() protoreflect.EnumType {
	return &file_inspect_inspect_proto_enumTypes[0]
}

func (x DependencyEdgeType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use DependencyEdgeType.Descriptor instead.
func (DependencyEdgeType) EnumDescriptor() ([]byte, []int) {
	return file_inspect_inspect_proto_rawDescGZIP(), []int{0}
}

// The ControllerRuntimeDependency message contains the graph of controller-resource dependencies.
type ControllerRuntimeDependency struct {
	state         protoimpl.MessageState      `protogen:"open.v1"`
	Metadata      *common.Metadata            `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Edges         []*ControllerDependencyEdge `protobuf:"bytes,2,rep,name=edges,proto3" json:"edges,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ControllerRuntimeDependency) Reset() {
	*x = ControllerRuntimeDependency{}
	mi := &file_inspect_inspect_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ControllerRuntimeDependency) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ControllerRuntimeDependency) ProtoMessage() {}

func (x *ControllerRuntimeDependency) ProtoReflect() protoreflect.Message {
	mi := &file_inspect_inspect_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ControllerRuntimeDependency.ProtoReflect.Descriptor instead.
func (*ControllerRuntimeDependency) Descriptor() ([]byte, []int) {
	return file_inspect_inspect_proto_rawDescGZIP(), []int{0}
}

func (x *ControllerRuntimeDependency) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ControllerRuntimeDependency) GetEdges() []*ControllerDependencyEdge {
	if x != nil {
		return x.Edges
	}
	return nil
}

type ControllerRuntimeDependenciesResponse struct {
	state         protoimpl.MessageState         `protogen:"open.v1"`
	Messages      []*ControllerRuntimeDependency `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ControllerRuntimeDependenciesResponse) Reset() {
	*x = ControllerRuntimeDependenciesResponse{}
	mi := &file_inspect_inspect_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ControllerRuntimeDependenciesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ControllerRuntimeDependenciesResponse) ProtoMessage() {}

func (x *ControllerRuntimeDependenciesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_inspect_inspect_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ControllerRuntimeDependenciesResponse.ProtoReflect.Descriptor instead.
func (*ControllerRuntimeDependenciesResponse) Descriptor() ([]byte, []int) {
	return file_inspect_inspect_proto_rawDescGZIP(), []int{1}
}

func (x *ControllerRuntimeDependenciesResponse) GetMessages() []*ControllerRuntimeDependency {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ControllerDependencyEdge struct {
	state             protoimpl.MessageState `protogen:"open.v1"`
	ControllerName    string                 `protobuf:"bytes,1,opt,name=controller_name,json=controllerName,proto3" json:"controller_name,omitempty"`
	EdgeType          DependencyEdgeType     `protobuf:"varint,2,opt,name=edge_type,json=edgeType,proto3,enum=inspect.DependencyEdgeType" json:"edge_type,omitempty"`
	ResourceNamespace string                 `protobuf:"bytes,3,opt,name=resource_namespace,json=resourceNamespace,proto3" json:"resource_namespace,omitempty"`
	ResourceType      string                 `protobuf:"bytes,4,opt,name=resource_type,json=resourceType,proto3" json:"resource_type,omitempty"`
	ResourceId        string                 `protobuf:"bytes,5,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *ControllerDependencyEdge) Reset() {
	*x = ControllerDependencyEdge{}
	mi := &file_inspect_inspect_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ControllerDependencyEdge) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ControllerDependencyEdge) ProtoMessage() {}

func (x *ControllerDependencyEdge) ProtoReflect() protoreflect.Message {
	mi := &file_inspect_inspect_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ControllerDependencyEdge.ProtoReflect.Descriptor instead.
func (*ControllerDependencyEdge) Descriptor() ([]byte, []int) {
	return file_inspect_inspect_proto_rawDescGZIP(), []int{2}
}

func (x *ControllerDependencyEdge) GetControllerName() string {
	if x != nil {
		return x.ControllerName
	}
	return ""
}

func (x *ControllerDependencyEdge) GetEdgeType() DependencyEdgeType {
	if x != nil {
		return x.EdgeType
	}
	return DependencyEdgeType_OUTPUT_EXCLUSIVE
}

func (x *ControllerDependencyEdge) GetResourceNamespace() string {
	if x != nil {
		return x.ResourceNamespace
	}
	return ""
}

func (x *ControllerDependencyEdge) GetResourceType() string {
	if x != nil {
		return x.ResourceType
	}
	return ""
}

func (x *ControllerDependencyEdge) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

var File_inspect_inspect_proto protoreflect.FileDescriptor

const file_inspect_inspect_proto_rawDesc = "" +
	"\n" +
	"\x15inspect/inspect.proto\x12\ainspect\x1a\x13common/common.proto\x1a\x1bgoogle/protobuf/empty.proto\"\x84\x01\n" +
	"\x1bControllerRuntimeDependency\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x127\n" +
	"\x05edges\x18\x02 \x03(\v2!.inspect.ControllerDependencyEdgeR\x05edges\"i\n" +
	"%ControllerRuntimeDependenciesResponse\x12@\n" +
	"\bmessages\x18\x01 \x03(\v2$.inspect.ControllerRuntimeDependencyR\bmessages\"\xf2\x01\n" +
	"\x18ControllerDependencyEdge\x12'\n" +
	"\x0fcontroller_name\x18\x01 \x01(\tR\x0econtrollerName\x128\n" +
	"\tedge_type\x18\x02 \x01(\x0e2\x1b.inspect.DependencyEdgeTypeR\bedgeType\x12-\n" +
	"\x12resource_namespace\x18\x03 \x01(\tR\x11resourceNamespace\x12#\n" +
	"\rresource_type\x18\x04 \x01(\tR\fresourceType\x12\x1f\n" +
	"\vresource_id\x18\x05 \x01(\tR\n" +
	"resourceId*x\n" +
	"\x12DependencyEdgeType\x12\x14\n" +
	"\x10OUTPUT_EXCLUSIVE\x10\x00\x12\x11\n" +
	"\rOUTPUT_SHARED\x10\x03\x12\x10\n" +
	"\fINPUT_STRONG\x10\x01\x12\x0e\n" +
	"\n" +
	"INPUT_WEAK\x10\x02\x12\x17\n" +
	"\x13INPUT_DESTROY_READY\x10\x042y\n" +
	"\x0eInspectService\x12g\n" +
	"\x1dControllerRuntimeDependencies\x12\x16.google.protobuf.Empty\x1a..inspect.ControllerRuntimeDependenciesResponseBN\n" +
	"\x15dev.talos.api.inspectZ5github.com/siderolabs/talos/pkg/machinery/api/inspectb\x06proto3"

var (
	file_inspect_inspect_proto_rawDescOnce sync.Once
	file_inspect_inspect_proto_rawDescData []byte
)

func file_inspect_inspect_proto_rawDescGZIP() []byte {
	file_inspect_inspect_proto_rawDescOnce.Do(func() {
		file_inspect_inspect_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_inspect_inspect_proto_rawDesc), len(file_inspect_inspect_proto_rawDesc)))
	})
	return file_inspect_inspect_proto_rawDescData
}

var file_inspect_inspect_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_inspect_inspect_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_inspect_inspect_proto_goTypes = []any{
	(DependencyEdgeType)(0),                       // 0: inspect.DependencyEdgeType
	(*ControllerRuntimeDependency)(nil),           // 1: inspect.ControllerRuntimeDependency
	(*ControllerRuntimeDependenciesResponse)(nil), // 2: inspect.ControllerRuntimeDependenciesResponse
	(*ControllerDependencyEdge)(nil),              // 3: inspect.ControllerDependencyEdge
	(*common.Metadata)(nil),                       // 4: common.Metadata
	(*emptypb.Empty)(nil),                         // 5: google.protobuf.Empty
}
var file_inspect_inspect_proto_depIdxs = []int32{
	4, // 0: inspect.ControllerRuntimeDependency.metadata:type_name -> common.Metadata
	3, // 1: inspect.ControllerRuntimeDependency.edges:type_name -> inspect.ControllerDependencyEdge
	1, // 2: inspect.ControllerRuntimeDependenciesResponse.messages:type_name -> inspect.ControllerRuntimeDependency
	0, // 3: inspect.ControllerDependencyEdge.edge_type:type_name -> inspect.DependencyEdgeType
	5, // 4: inspect.InspectService.ControllerRuntimeDependencies:input_type -> google.protobuf.Empty
	2, // 5: inspect.InspectService.ControllerRuntimeDependencies:output_type -> inspect.ControllerRuntimeDependenciesResponse
	5, // [5:6] is the sub-list for method output_type
	4, // [4:5] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_inspect_inspect_proto_init() }
func file_inspect_inspect_proto_init() {
	if File_inspect_inspect_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_inspect_inspect_proto_rawDesc), len(file_inspect_inspect_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_inspect_inspect_proto_goTypes,
		DependencyIndexes: file_inspect_inspect_proto_depIdxs,
		EnumInfos:         file_inspect_inspect_proto_enumTypes,
		MessageInfos:      file_inspect_inspect_proto_msgTypes,
	}.Build()
	File_inspect_inspect_proto = out.File
	file_inspect_inspect_proto_goTypes = nil
	file_inspect_inspect_proto_depIdxs = nil
}
