// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: machine/image.proto

package machine

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"

	common "github.com/siderolabs/talos/pkg/machinery/api/common"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ImageServicePullLayerProgress_Status int32

const (
	// Keep this in sync with ImagePullLayerProgress.Status.
	ImageServicePullLayerProgress_DOWNLOADING       ImageServicePullLayerProgress_Status = 0
	ImageServicePullLayerProgress_DOWNLOAD_COMPLETE ImageServicePullLayerProgress_Status = 1
	ImageServicePullLayerProgress_EXTRACTING        ImageServicePullLayerProgress_Status = 2
	ImageServicePullLayerProgress_EXTRACT_COMPLETE  ImageServicePullLayerProgress_Status = 3
	ImageServicePullLayerProgress_ALREADY_EXISTS    ImageServicePullLayerProgress_Status = 4
)

// Enum value maps for ImageServicePullLayerProgress_Status.
var (
	ImageServicePullLayerProgress_Status_name = map[int32]string{
		0: "DOWNLOADING",
		1: "DOWNLOAD_COMPLETE",
		2: "EXTRACTING",
		3: "EXTRACT_COMPLETE",
		4: "ALREADY_EXISTS",
	}
	ImageServicePullLayerProgress_Status_value = map[string]int32{
		"DOWNLOADING":       0,
		"DOWNLOAD_COMPLETE": 1,
		"EXTRACTING":        2,
		"EXTRACT_COMPLETE":  3,
		"ALREADY_EXISTS":    4,
	}
)

func (x ImageServicePullLayerProgress_Status) Enum() *ImageServicePullLayerProgress_Status {
	p := new(ImageServicePullLayerProgress_Status)
	*p = x
	return p
}

func (x ImageServicePullLayerProgress_Status) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ImageServicePullLayerProgress_Status) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_image_proto_enumTypes[0].Descriptor()
}

func (ImageServicePullLayerProgress_Status) Type() protoreflect.EnumType {
	return &file_machine_image_proto_enumTypes[0]
}

func (x ImageServicePullLayerProgress_Status) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ImageServicePullLayerProgress_Status.Descriptor instead.
func (ImageServicePullLayerProgress_Status) EnumDescriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{6, 0}
}

type ImageServiceListRequest struct {
	state         protoimpl.MessageState     `protogen:"open.v1"`
	Containerd    *common.ContainerdInstance `protobuf:"bytes,1,opt,name=containerd,proto3" json:"containerd,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceListRequest) Reset() {
	*x = ImageServiceListRequest{}
	mi := &file_machine_image_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceListRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceListRequest) ProtoMessage() {}

func (x *ImageServiceListRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceListRequest.ProtoReflect.Descriptor instead.
func (*ImageServiceListRequest) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{0}
}

func (x *ImageServiceListRequest) GetContainerd() *common.ContainerdInstance {
	if x != nil {
		return x.Containerd
	}
	return nil
}

type ImageServiceListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Digest        string                 `protobuf:"bytes,2,opt,name=digest,proto3" json:"digest,omitempty"`
	Size          int64                  `protobuf:"varint,3,opt,name=size,proto3" json:"size,omitempty"`
	CreatedAt     *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=created_at,json=createdAt,proto3" json:"created_at,omitempty"`
	Labels        map[string]string      `protobuf:"bytes,5,rep,name=labels,proto3" json:"labels,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceListResponse) Reset() {
	*x = ImageServiceListResponse{}
	mi := &file_machine_image_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceListResponse) ProtoMessage() {}

func (x *ImageServiceListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceListResponse.ProtoReflect.Descriptor instead.
func (*ImageServiceListResponse) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{1}
}

func (x *ImageServiceListResponse) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ImageServiceListResponse) GetDigest() string {
	if x != nil {
		return x.Digest
	}
	return ""
}

func (x *ImageServiceListResponse) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *ImageServiceListResponse) GetCreatedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.CreatedAt
	}
	return nil
}

func (x *ImageServiceListResponse) GetLabels() map[string]string {
	if x != nil {
		return x.Labels
	}
	return nil
}

type ImageServicePullRequest struct {
	state      protoimpl.MessageState     `protogen:"open.v1"`
	Containerd *common.ContainerdInstance `protobuf:"bytes,1,opt,name=containerd,proto3" json:"containerd,omitempty"`
	// Image reference to pull.
	ImageRef      string `protobuf:"bytes,3,opt,name=image_ref,json=imageRef,proto3" json:"image_ref,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServicePullRequest) Reset() {
	*x = ImageServicePullRequest{}
	mi := &file_machine_image_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServicePullRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServicePullRequest) ProtoMessage() {}

func (x *ImageServicePullRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServicePullRequest.ProtoReflect.Descriptor instead.
func (*ImageServicePullRequest) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{2}
}

func (x *ImageServicePullRequest) GetContainerd() *common.ContainerdInstance {
	if x != nil {
		return x.Containerd
	}
	return nil
}

func (x *ImageServicePullRequest) GetImageRef() string {
	if x != nil {
		return x.ImageRef
	}
	return ""
}

type ImageServicePullResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Response:
	//
	//	*ImageServicePullResponse_Name
	//	*ImageServicePullResponse_PullProgress
	Response      isImageServicePullResponse_Response `protobuf_oneof:"response"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServicePullResponse) Reset() {
	*x = ImageServicePullResponse{}
	mi := &file_machine_image_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServicePullResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServicePullResponse) ProtoMessage() {}

func (x *ImageServicePullResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServicePullResponse.ProtoReflect.Descriptor instead.
func (*ImageServicePullResponse) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{3}
}

func (x *ImageServicePullResponse) GetResponse() isImageServicePullResponse_Response {
	if x != nil {
		return x.Response
	}
	return nil
}

func (x *ImageServicePullResponse) GetName() string {
	if x != nil {
		if x, ok := x.Response.(*ImageServicePullResponse_Name); ok {
			return x.Name
		}
	}
	return ""
}

func (x *ImageServicePullResponse) GetPullProgress() *ImageServicePullProgress {
	if x != nil {
		if x, ok := x.Response.(*ImageServicePullResponse_PullProgress); ok {
			return x.PullProgress
		}
	}
	return nil
}

type isImageServicePullResponse_Response interface {
	isImageServicePullResponse_Response()
}

type ImageServicePullResponse_Name struct {
	// Name of the pulled image (when done).
	Name string `protobuf:"bytes,1,opt,name=name,proto3,oneof"`
}

type ImageServicePullResponse_PullProgress struct {
	// Progress of the image pull (intermediate updates).
	PullProgress *ImageServicePullProgress `protobuf:"bytes,2,opt,name=pull_progress,json=pullProgress,proto3,oneof"`
}

func (*ImageServicePullResponse_Name) isImageServicePullResponse_Response() {}

func (*ImageServicePullResponse_PullProgress) isImageServicePullResponse_Response() {}

type ImageServiceImportRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Request:
	//
	//	*ImageServiceImportRequest_Containerd
	//	*ImageServiceImportRequest_ImageChunk
	Request       isImageServiceImportRequest_Request `protobuf_oneof:"request"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceImportRequest) Reset() {
	*x = ImageServiceImportRequest{}
	mi := &file_machine_image_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceImportRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceImportRequest) ProtoMessage() {}

func (x *ImageServiceImportRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceImportRequest.ProtoReflect.Descriptor instead.
func (*ImageServiceImportRequest) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{4}
}

func (x *ImageServiceImportRequest) GetRequest() isImageServiceImportRequest_Request {
	if x != nil {
		return x.Request
	}
	return nil
}

func (x *ImageServiceImportRequest) GetContainerd() *common.ContainerdInstance {
	if x != nil {
		if x, ok := x.Request.(*ImageServiceImportRequest_Containerd); ok {
			return x.Containerd
		}
	}
	return nil
}

func (x *ImageServiceImportRequest) GetImageChunk() *common.Data {
	if x != nil {
		if x, ok := x.Request.(*ImageServiceImportRequest_ImageChunk); ok {
			return x.ImageChunk
		}
	}
	return nil
}

type isImageServiceImportRequest_Request interface {
	isImageServiceImportRequest_Request()
}

type ImageServiceImportRequest_Containerd struct {
	// Containerd instance to use.
	Containerd *common.ContainerdInstance `protobuf:"bytes,1,opt,name=containerd,proto3,oneof"`
}

type ImageServiceImportRequest_ImageChunk struct {
	// Chunk of the image tarball.
	ImageChunk *common.Data `protobuf:"bytes,2,opt,name=image_chunk,json=imageChunk,proto3,oneof"`
}

func (*ImageServiceImportRequest_Containerd) isImageServiceImportRequest_Request() {}

func (*ImageServiceImportRequest_ImageChunk) isImageServiceImportRequest_Request() {}

type ImageServiceImportResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the imported image.
	Name          string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceImportResponse) Reset() {
	*x = ImageServiceImportResponse{}
	mi := &file_machine_image_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceImportResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceImportResponse) ProtoMessage() {}

func (x *ImageServiceImportResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceImportResponse.ProtoReflect.Descriptor instead.
func (*ImageServiceImportResponse) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{5}
}

func (x *ImageServiceImportResponse) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

type ImageServicePullLayerProgress struct {
	state         protoimpl.MessageState               `protogen:"open.v1"`
	Status        ImageServicePullLayerProgress_Status `protobuf:"varint,1,opt,name=status,proto3,enum=machine.ImageServicePullLayerProgress_Status" json:"status,omitempty"`
	Elapsed       *durationpb.Duration                 `protobuf:"bytes,2,opt,name=elapsed,proto3" json:"elapsed,omitempty"`
	Offset        int64                                `protobuf:"varint,3,opt,name=offset,proto3" json:"offset,omitempty"`
	Total         int64                                `protobuf:"varint,4,opt,name=total,proto3" json:"total,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServicePullLayerProgress) Reset() {
	*x = ImageServicePullLayerProgress{}
	mi := &file_machine_image_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServicePullLayerProgress) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServicePullLayerProgress) ProtoMessage() {}

func (x *ImageServicePullLayerProgress) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServicePullLayerProgress.ProtoReflect.Descriptor instead.
func (*ImageServicePullLayerProgress) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{6}
}

func (x *ImageServicePullLayerProgress) GetStatus() ImageServicePullLayerProgress_Status {
	if x != nil {
		return x.Status
	}
	return ImageServicePullLayerProgress_DOWNLOADING
}

func (x *ImageServicePullLayerProgress) GetElapsed() *durationpb.Duration {
	if x != nil {
		return x.Elapsed
	}
	return nil
}

func (x *ImageServicePullLayerProgress) GetOffset() int64 {
	if x != nil {
		return x.Offset
	}
	return 0
}

func (x *ImageServicePullLayerProgress) GetTotal() int64 {
	if x != nil {
		return x.Total
	}
	return 0
}

type ImageServicePullProgress struct {
	state         protoimpl.MessageState         `protogen:"open.v1"`
	LayerId       string                         `protobuf:"bytes,1,opt,name=layer_id,json=layerId,proto3" json:"layer_id,omitempty"`
	Progress      *ImageServicePullLayerProgress `protobuf:"bytes,2,opt,name=progress,proto3" json:"progress,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServicePullProgress) Reset() {
	*x = ImageServicePullProgress{}
	mi := &file_machine_image_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServicePullProgress) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServicePullProgress) ProtoMessage() {}

func (x *ImageServicePullProgress) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServicePullProgress.ProtoReflect.Descriptor instead.
func (*ImageServicePullProgress) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{7}
}

func (x *ImageServicePullProgress) GetLayerId() string {
	if x != nil {
		return x.LayerId
	}
	return ""
}

func (x *ImageServicePullProgress) GetProgress() *ImageServicePullLayerProgress {
	if x != nil {
		return x.Progress
	}
	return nil
}

type ImageServiceRemoveRequest struct {
	state      protoimpl.MessageState     `protogen:"open.v1"`
	Containerd *common.ContainerdInstance `protobuf:"bytes,1,opt,name=containerd,proto3" json:"containerd,omitempty"`
	// Image reference to remove.
	ImageRef      string `protobuf:"bytes,2,opt,name=image_ref,json=imageRef,proto3" json:"image_ref,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceRemoveRequest) Reset() {
	*x = ImageServiceRemoveRequest{}
	mi := &file_machine_image_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceRemoveRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceRemoveRequest) ProtoMessage() {}

func (x *ImageServiceRemoveRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceRemoveRequest.ProtoReflect.Descriptor instead.
func (*ImageServiceRemoveRequest) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{8}
}

func (x *ImageServiceRemoveRequest) GetContainerd() *common.ContainerdInstance {
	if x != nil {
		return x.Containerd
	}
	return nil
}

func (x *ImageServiceRemoveRequest) GetImageRef() string {
	if x != nil {
		return x.ImageRef
	}
	return ""
}

type ImageServiceVerifyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Image reference to verify.
	//
	// The image reference could be either in:
	//   - the digest form (e.g. "docker.io/library/nginx@sha256:abc123...") to ensure that the exact image is verified.
	//   - the tag form (e.g. "docker.io/library/nginx:latest") to verify the image currently pointed by the tag, and the resolved
	//     digested will be returned in the response.
	//
	// Any other format will cause the error.
	ImageRef string `protobuf:"bytes,1,opt,name=image_ref,json=imageRef,proto3" json:"image_ref,omitempty"`
	// Authentication credentials for the registry (if needed).
	//
	// By default Talos will use configured auth, but additional
	// image pull secret can be submitted here.
	Credentials   *ImageServiceCredentials `protobuf:"bytes,2,opt,name=credentials,proto3" json:"credentials,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceVerifyRequest) Reset() {
	*x = ImageServiceVerifyRequest{}
	mi := &file_machine_image_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceVerifyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceVerifyRequest) ProtoMessage() {}

func (x *ImageServiceVerifyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceVerifyRequest.ProtoReflect.Descriptor instead.
func (*ImageServiceVerifyRequest) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{9}
}

func (x *ImageServiceVerifyRequest) GetImageRef() string {
	if x != nil {
		return x.ImageRef
	}
	return ""
}

func (x *ImageServiceVerifyRequest) GetCredentials() *ImageServiceCredentials {
	if x != nil {
		return x.Credentials
	}
	return nil
}

type ImageServiceCredentials struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Host of the registry (e.g. "docker.io").
	Host string `protobuf:"bytes,1,opt,name=host,proto3" json:"host,omitempty"`
	// Username for the registry.
	Username string `protobuf:"bytes,2,opt,name=username,proto3" json:"username,omitempty"`
	// Password (token) for the registry.
	Password      string `protobuf:"bytes,3,opt,name=password,proto3" json:"password,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageServiceCredentials) Reset() {
	*x = ImageServiceCredentials{}
	mi := &file_machine_image_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceCredentials) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceCredentials) ProtoMessage() {}

func (x *ImageServiceCredentials) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceCredentials.ProtoReflect.Descriptor instead.
func (*ImageServiceCredentials) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{10}
}

func (x *ImageServiceCredentials) GetHost() string {
	if x != nil {
		return x.Host
	}
	return ""
}

func (x *ImageServiceCredentials) GetUsername() string {
	if x != nil {
		return x.Username
	}
	return ""
}

func (x *ImageServiceCredentials) GetPassword() string {
	if x != nil {
		return x.Password
	}
	return ""
}

type ImageServiceVerifyResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Was the image verified: if it didn't match any verify rule, false will be returned.
	// If the image matched the rule, but the verification failed, an error will be returned.
	Verified bool `protobuf:"varint,1,opt,name=verified,proto3" json:"verified,omitempty"`
	// Free-form verification result message, e.g. with details about the matched rule and how the image was verified.
	Message string `protobuf:"bytes,2,opt,name=message,proto3" json:"message,omitempty"`
	// The pinned image reference with resolved digest that was verified (e.g. "docker.io/library/nginx@sha256:abc123...").
	//
	// This is only set if verified=true.
	DigestedImageRef string `protobuf:"bytes,3,opt,name=digested_image_ref,json=digestedImageRef,proto3" json:"digested_image_ref,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *ImageServiceVerifyResponse) Reset() {
	*x = ImageServiceVerifyResponse{}
	mi := &file_machine_image_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageServiceVerifyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageServiceVerifyResponse) ProtoMessage() {}

func (x *ImageServiceVerifyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_image_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageServiceVerifyResponse.ProtoReflect.Descriptor instead.
func (*ImageServiceVerifyResponse) Descriptor() ([]byte, []int) {
	return file_machine_image_proto_rawDescGZIP(), []int{11}
}

func (x *ImageServiceVerifyResponse) GetVerified() bool {
	if x != nil {
		return x.Verified
	}
	return false
}

func (x *ImageServiceVerifyResponse) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

func (x *ImageServiceVerifyResponse) GetDigestedImageRef() string {
	if x != nil {
		return x.DigestedImageRef
	}
	return ""
}

var File_machine_image_proto protoreflect.FileDescriptor

const file_machine_image_proto_rawDesc = "" +
	"\n" +
	"\x13machine/image.proto\x12\amachine\x1a\x13common/common.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"U\n" +
	"\x17ImageServiceListRequest\x12:\n" +
	"\n" +
	"containerd\x18\x01 \x01(\v2\x1a.common.ContainerdInstanceR\n" +
	"containerd\"\x97\x02\n" +
	"\x18ImageServiceListResponse\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n" +
	"\x06digest\x18\x02 \x01(\tR\x06digest\x12\x12\n" +
	"\x04size\x18\x03 \x01(\x03R\x04size\x129\n" +
	"\n" +
	"created_at\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampR\tcreatedAt\x12E\n" +
	"\x06labels\x18\x05 \x03(\v2-.machine.ImageServiceListResponse.LabelsEntryR\x06labels\x1a9\n" +
	"\vLabelsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\"r\n" +
	"\x17ImageServicePullRequest\x12:\n" +
	"\n" +
	"containerd\x18\x01 \x01(\v2\x1a.common.ContainerdInstanceR\n" +
	"containerd\x12\x1b\n" +
	"\timage_ref\x18\x03 \x01(\tR\bimageRef\"\x86\x01\n" +
	"\x18ImageServicePullResponse\x12\x14\n" +
	"\x04name\x18\x01 \x01(\tH\x00R\x04name\x12H\n" +
	"\rpull_progress\x18\x02 \x01(\v2!.machine.ImageServicePullProgressH\x00R\fpullProgressB\n" +
	"\n" +
	"\bresponse\"\x95\x01\n" +
	"\x19ImageServiceImportRequest\x12<\n" +
	"\n" +
	"containerd\x18\x01 \x01(\v2\x1a.common.ContainerdInstanceH\x00R\n" +
	"containerd\x12/\n" +
	"\vimage_chunk\x18\x02 \x01(\v2\f.common.DataH\x00R\n" +
	"imageChunkB\t\n" +
	"\arequest\"0\n" +
	"\x1aImageServiceImportResponse\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\"\xb5\x02\n" +
	"\x1dImageServicePullLayerProgress\x12E\n" +
	"\x06status\x18\x01 \x01(\x0e2-.machine.ImageServicePullLayerProgress.StatusR\x06status\x123\n" +
	"\aelapsed\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\aelapsed\x12\x16\n" +
	"\x06offset\x18\x03 \x01(\x03R\x06offset\x12\x14\n" +
	"\x05total\x18\x04 \x01(\x03R\x05total\"j\n" +
	"\x06Status\x12\x0f\n" +
	"\vDOWNLOADING\x10\x00\x12\x15\n" +
	"\x11DOWNLOAD_COMPLETE\x10\x01\x12\x0e\n" +
	"\n" +
	"EXTRACTING\x10\x02\x12\x14\n" +
	"\x10EXTRACT_COMPLETE\x10\x03\x12\x12\n" +
	"\x0eALREADY_EXISTS\x10\x04\"y\n" +
	"\x18ImageServicePullProgress\x12\x19\n" +
	"\blayer_id\x18\x01 \x01(\tR\alayerId\x12B\n" +
	"\bprogress\x18\x02 \x01(\v2&.machine.ImageServicePullLayerProgressR\bprogress\"t\n" +
	"\x19ImageServiceRemoveRequest\x12:\n" +
	"\n" +
	"containerd\x18\x01 \x01(\v2\x1a.common.ContainerdInstanceR\n" +
	"containerd\x12\x1b\n" +
	"\timage_ref\x18\x02 \x01(\tR\bimageRef\"|\n" +
	"\x19ImageServiceVerifyRequest\x12\x1b\n" +
	"\timage_ref\x18\x01 \x01(\tR\bimageRef\x12B\n" +
	"\vcredentials\x18\x02 \x01(\v2 .machine.ImageServiceCredentialsR\vcredentials\"e\n" +
	"\x17ImageServiceCredentials\x12\x12\n" +
	"\x04host\x18\x01 \x01(\tR\x04host\x12\x1a\n" +
	"\busername\x18\x02 \x01(\tR\busername\x12\x1a\n" +
	"\bpassword\x18\x03 \x01(\tR\bpassword\"\x80\x01\n" +
	"\x1aImageServiceVerifyResponse\x12\x1a\n" +
	"\bverified\x18\x01 \x01(\bR\bverified\x12\x18\n" +
	"\amessage\x18\x02 \x01(\tR\amessage\x12,\n" +
	"\x12digested_image_ref\x18\x03 \x01(\tR\x10digestedImageRef2\x9a\x03\n" +
	"\fImageService\x12M\n" +
	"\x04List\x12 .machine.ImageServiceListRequest\x1a!.machine.ImageServiceListResponse0\x01\x12M\n" +
	"\x04Pull\x12 .machine.ImageServicePullRequest\x1a!.machine.ImageServicePullResponse0\x01\x12S\n" +
	"\x06Import\x12\".machine.ImageServiceImportRequest\x1a#.machine.ImageServiceImportResponse(\x01\x12D\n" +
	"\x06Remove\x12\".machine.ImageServiceRemoveRequest\x1a\x16.google.protobuf.Empty\x12Q\n" +
	"\x06Verify\x12\".machine.ImageServiceVerifyRequest\x1a#.machine.ImageServiceVerifyResponseBN\n" +
	"\x15dev.talos.api.machineZ5github.com/siderolabs/talos/pkg/machinery/api/machineb\x06proto3"

var (
	file_machine_image_proto_rawDescOnce sync.Once
	file_machine_image_proto_rawDescData []byte
)

func file_machine_image_proto_rawDescGZIP() []byte {
	file_machine_image_proto_rawDescOnce.Do(func() {
		file_machine_image_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_machine_image_proto_rawDesc), len(file_machine_image_proto_rawDesc)))
	})
	return file_machine_image_proto_rawDescData
}

var file_machine_image_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_machine_image_proto_msgTypes = make([]protoimpl.MessageInfo, 13)
var file_machine_image_proto_goTypes = []any{
	(ImageServicePullLayerProgress_Status)(0), // 0: machine.ImageServicePullLayerProgress.Status
	(*ImageServiceListRequest)(nil),           // 1: machine.ImageServiceListRequest
	(*ImageServiceListResponse)(nil),          // 2: machine.ImageServiceListResponse
	(*ImageServicePullRequest)(nil),           // 3: machine.ImageServicePullRequest
	(*ImageServicePullResponse)(nil),          // 4: machine.ImageServicePullResponse
	(*ImageServiceImportRequest)(nil),         // 5: machine.ImageServiceImportRequest
	(*ImageServiceImportResponse)(nil),        // 6: machine.ImageServiceImportResponse
	(*ImageServicePullLayerProgress)(nil),     // 7: machine.ImageServicePullLayerProgress
	(*ImageServicePullProgress)(nil),          // 8: machine.ImageServicePullProgress
	(*ImageServiceRemoveRequest)(nil),         // 9: machine.ImageServiceRemoveRequest
	(*ImageServiceVerifyRequest)(nil),         // 10: machine.ImageServiceVerifyRequest
	(*ImageServiceCredentials)(nil),           // 11: machine.ImageServiceCredentials
	(*ImageServiceVerifyResponse)(nil),        // 12: machine.ImageServiceVerifyResponse
	nil,                                       // 13: machine.ImageServiceListResponse.LabelsEntry
	(*common.ContainerdInstance)(nil),         // 14: common.ContainerdInstance
	(*timestamppb.Timestamp)(nil),             // 15: google.protobuf.Timestamp
	(*common.Data)(nil),                       // 16: common.Data
	(*durationpb.Duration)(nil),               // 17: google.protobuf.Duration
	(*emptypb.Empty)(nil),                     // 18: google.protobuf.Empty
}
var file_machine_image_proto_depIdxs = []int32{
	14, // 0: machine.ImageServiceListRequest.containerd:type_name -> common.ContainerdInstance
	15, // 1: machine.ImageServiceListResponse.created_at:type_name -> google.protobuf.Timestamp
	13, // 2: machine.ImageServiceListResponse.labels:type_name -> machine.ImageServiceListResponse.LabelsEntry
	14, // 3: machine.ImageServicePullRequest.containerd:type_name -> common.ContainerdInstance
	8,  // 4: machine.ImageServicePullResponse.pull_progress:type_name -> machine.ImageServicePullProgress
	14, // 5: machine.ImageServiceImportRequest.containerd:type_name -> common.ContainerdInstance
	16, // 6: machine.ImageServiceImportRequest.image_chunk:type_name -> common.Data
	0,  // 7: machine.ImageServicePullLayerProgress.status:type_name -> machine.ImageServicePullLayerProgress.Status
	17, // 8: machine.ImageServicePullLayerProgress.elapsed:type_name -> google.protobuf.Duration
	7,  // 9: machine.ImageServicePullProgress.progress:type_name -> machine.ImageServicePullLayerProgress
	14, // 10: machine.ImageServiceRemoveRequest.containerd:type_name -> common.ContainerdInstance
	11, // 11: machine.ImageServiceVerifyRequest.credentials:type_name -> machine.ImageServiceCredentials
	1,  // 12: machine.ImageService.List:input_type -> machine.ImageServiceListRequest
	3,  // 13: machine.ImageService.Pull:input_type -> machine.ImageServicePullRequest
	5,  // 14: machine.ImageService.Import:input_type -> machine.ImageServiceImportRequest
	9,  // 15: machine.ImageService.Remove:input_type -> machine.ImageServiceRemoveRequest
	10, // 16: machine.ImageService.Verify:input_type -> machine.ImageServiceVerifyRequest
	2,  // 17: machine.ImageService.List:output_type -> machine.ImageServiceListResponse
	4,  // 18: machine.ImageService.Pull:output_type -> machine.ImageServicePullResponse
	6,  // 19: machine.ImageService.Import:output_type -> machine.ImageServiceImportResponse
	18, // 20: machine.ImageService.Remove:output_type -> google.protobuf.Empty
	12, // 21: machine.ImageService.Verify:output_type -> machine.ImageServiceVerifyResponse
	17, // [17:22] is the sub-list for method output_type
	12, // [12:17] is the sub-list for method input_type
	12, // [12:12] is the sub-list for extension type_name
	12, // [12:12] is the sub-list for extension extendee
	0,  // [0:12] is the sub-list for field type_name
}

func init() { file_machine_image_proto_init() }
func file_machine_image_proto_init() {
	if File_machine_image_proto != nil {
		return
	}
	file_machine_image_proto_msgTypes[3].OneofWrappers = []any{
		(*ImageServicePullResponse_Name)(nil),
		(*ImageServicePullResponse_PullProgress)(nil),
	}
	file_machine_image_proto_msgTypes[4].OneofWrappers = []any{
		(*ImageServiceImportRequest_Containerd)(nil),
		(*ImageServiceImportRequest_ImageChunk)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_machine_image_proto_rawDesc), len(file_machine_image_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   13,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_machine_image_proto_goTypes,
		DependencyIndexes: file_machine_image_proto_depIdxs,
		EnumInfos:         file_machine_image_proto_enumTypes,
		MessageInfos:      file_machine_image_proto_msgTypes,
	}.Build()
	File_machine_image_proto = out.File
	file_machine_image_proto_goTypes = nil
	file_machine_image_proto_depIdxs = nil
}
