// Copyright 2018 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package types

import (
	"reflect"

	structpb "google.golang.org/protobuf/types/known/structpb"
)

// JSON type constants representing the reflected types of protobuf JSON values.
var (
	// JSONValueType describes the protobuf native type for a JSON value.
	JSONValueType  = reflect.TypeFor[*structpb.Value]()
	JSONListType   = reflect.TypeFor[*structpb.ListValue]()
	JSONStructType = reflect.TypeFor[*structpb.Struct]()
	JSONNullType   = reflect.TypeFor[structpb.NullValue]()
)
