// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"net/netip"
)

// PeerInfo represents one peer of an overlay network.
//
// swagger:model PeerInfo
type PeerInfo struct {

	// ID of the peer-node in the Swarm cluster.
	// Example: 6869d7c1732b
	Name string `json:"Name"`

	// IP-address of the peer-node in the Swarm cluster.
	// Example: 10.133.77.91
	IP netip.Addr `json:"IP"`
}
