// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.33.0
// 	protoc        v4.24.4
// source: siderolink/provision.proto

package pb

import (
	reflect "reflect"
	sync "sync"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ProvisionRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	// Node UUID as string.
	NodeUuid string `protobuf:"bytes,1,opt,name=node_uuid,json=nodeUuid,proto3" json:"node_uuid,omitempty"`
	// Wireguard public key (as string) of the node.
	NodePublicKey string `protobuf:"bytes,2,opt,name=node_public_key,json=nodePublicKey,proto3" json:"node_public_key,omitempty"`
	// Join token (as string) of the node.
	JoinToken *string `protobuf:"bytes,3,opt,name=join_token,json=joinToken,proto3,oneof" json:"join_token,omitempty"`
	// Unique token (as string) of the node. Needed in case node provided node_uuid with all zeroes.
	// Or two and more nodes have the same node_uuid.
	NodeUniqueToken *string `protobuf:"bytes,4,opt,name=node_unique_token,json=nodeUniqueToken,proto3,oneof" json:"node_unique_token,omitempty"`
	// Talos Version (as string) of the node.
	TalosVersion *string `protobuf:"bytes,5,opt,name=talos_version,json=talosVersion,proto3,oneof" json:"talos_version,omitempty"`
	// Wireguard over GRPC tunnel request
	WireguardOverGrpc *bool `protobuf:"varint,6,opt,name=wireguard_over_grpc,json=wireguardOverGrpc,proto3,oneof" json:"wireguard_over_grpc,omitempty"`
}

func (x *ProvisionRequest) Reset() {
	*x = ProvisionRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_siderolink_provision_proto_msgTypes[0]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *ProvisionRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProvisionRequest) ProtoMessage() {}

func (x *ProvisionRequest) ProtoReflect() protoreflect.Message {
	mi := &file_siderolink_provision_proto_msgTypes[0]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProvisionRequest.ProtoReflect.Descriptor instead.
func (*ProvisionRequest) Descriptor() ([]byte, []int) {
	return file_siderolink_provision_proto_rawDescGZIP(), []int{0}
}

func (x *ProvisionRequest) GetNodeUuid() string {
	if x != nil {
		return x.NodeUuid
	}
	return ""
}

func (x *ProvisionRequest) GetNodePublicKey() string {
	if x != nil {
		return x.NodePublicKey
	}
	return ""
}

func (x *ProvisionRequest) GetJoinToken() string {
	if x != nil && x.JoinToken != nil {
		return *x.JoinToken
	}
	return ""
}

func (x *ProvisionRequest) GetNodeUniqueToken() string {
	if x != nil && x.NodeUniqueToken != nil {
		return *x.NodeUniqueToken
	}
	return ""
}

func (x *ProvisionRequest) GetTalosVersion() string {
	if x != nil && x.TalosVersion != nil {
		return *x.TalosVersion
	}
	return ""
}

func (x *ProvisionRequest) GetWireguardOverGrpc() bool {
	if x != nil && x.WireguardOverGrpc != nil {
		return *x.WireguardOverGrpc
	}
	return false
}

type ProvisionResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	// Server Wireguard endpoint IP and port.
	ServerEndpoint []string `protobuf:"bytes,1,rep,name=server_endpoint,json=serverEndpoint,proto3" json:"server_endpoint,omitempty"`
	// Server public Wireguard key encoded as string.
	ServerPublicKey string `protobuf:"bytes,3,opt,name=server_public_key,json=serverPublicKey,proto3" json:"server_public_key,omitempty"`
	// Node address with prefix on the Wireguard tunnel.
	NodeAddressPrefix string `protobuf:"bytes,4,opt,name=node_address_prefix,json=nodeAddressPrefix,proto3" json:"node_address_prefix,omitempty"`
	// Server address on the Wireguard tunnel.
	ServerAddress string `protobuf:"bytes,5,opt,name=server_address,json=serverAddress,proto3" json:"server_address,omitempty"`
	// Virtual peer address:port in case of Wireguard over GRPC.
	GrpcPeerAddrPort string `protobuf:"bytes,6,opt,name=grpc_peer_addr_port,json=grpcPeerAddrPort,proto3" json:"grpc_peer_addr_port,omitempty"`
}

func (x *ProvisionResponse) Reset() {
	*x = ProvisionResponse{}
	if protoimpl.UnsafeEnabled {
		mi := &file_siderolink_provision_proto_msgTypes[1]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *ProvisionResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProvisionResponse) ProtoMessage() {}

func (x *ProvisionResponse) ProtoReflect() protoreflect.Message {
	mi := &file_siderolink_provision_proto_msgTypes[1]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProvisionResponse.ProtoReflect.Descriptor instead.
func (*ProvisionResponse) Descriptor() ([]byte, []int) {
	return file_siderolink_provision_proto_rawDescGZIP(), []int{1}
}

func (x *ProvisionResponse) GetServerEndpoint() []string {
	if x != nil {
		return x.ServerEndpoint
	}
	return nil
}

func (x *ProvisionResponse) GetServerPublicKey() string {
	if x != nil {
		return x.ServerPublicKey
	}
	return ""
}

func (x *ProvisionResponse) GetNodeAddressPrefix() string {
	if x != nil {
		return x.NodeAddressPrefix
	}
	return ""
}

func (x *ProvisionResponse) GetServerAddress() string {
	if x != nil {
		return x.ServerAddress
	}
	return ""
}

func (x *ProvisionResponse) GetGrpcPeerAddrPort() string {
	if x != nil {
		return x.GrpcPeerAddrPort
	}
	return ""
}

var File_siderolink_provision_proto protoreflect.FileDescriptor

var file_siderolink_provision_proto_rawDesc = []byte{
	0x0a, 0x1a, 0x73, 0x69, 0x64, 0x65, 0x72, 0x6f, 0x6c, 0x69, 0x6e, 0x6b, 0x2f, 0x70, 0x72, 0x6f,
	0x76, 0x69, 0x73, 0x69, 0x6f, 0x6e, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x12, 0x0b, 0x73, 0x69,
	0x64, 0x65, 0x72, 0x6f, 0x2e, 0x6c, 0x69, 0x6e, 0x6b, 0x22, 0xda, 0x02, 0x0a, 0x10, 0x50, 0x72,
	0x6f, 0x76, 0x69, 0x73, 0x69, 0x6f, 0x6e, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12, 0x1b,
	0x0a, 0x09, 0x6e, 0x6f, 0x64, 0x65, 0x5f, 0x75, 0x75, 0x69, 0x64, 0x18, 0x01, 0x20, 0x01, 0x28,
	0x09, 0x52, 0x08, 0x6e, 0x6f, 0x64, 0x65, 0x55, 0x75, 0x69, 0x64, 0x12, 0x26, 0x0a, 0x0f, 0x6e,
	0x6f, 0x64, 0x65, 0x5f, 0x70, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x5f, 0x6b, 0x65, 0x79, 0x18, 0x02,
	0x20, 0x01, 0x28, 0x09, 0x52, 0x0d, 0x6e, 0x6f, 0x64, 0x65, 0x50, 0x75, 0x62, 0x6c, 0x69, 0x63,
	0x4b, 0x65, 0x79, 0x12, 0x22, 0x0a, 0x0a, 0x6a, 0x6f, 0x69, 0x6e, 0x5f, 0x74, 0x6f, 0x6b, 0x65,
	0x6e, 0x18, 0x03, 0x20, 0x01, 0x28, 0x09, 0x48, 0x00, 0x52, 0x09, 0x6a, 0x6f, 0x69, 0x6e, 0x54,
	0x6f, 0x6b, 0x65, 0x6e, 0x88, 0x01, 0x01, 0x12, 0x2f, 0x0a, 0x11, 0x6e, 0x6f, 0x64, 0x65, 0x5f,
	0x75, 0x6e, 0x69, 0x71, 0x75, 0x65, 0x5f, 0x74, 0x6f, 0x6b, 0x65, 0x6e, 0x18, 0x04, 0x20, 0x01,
	0x28, 0x09, 0x48, 0x01, 0x52, 0x0f, 0x6e, 0x6f, 0x64, 0x65, 0x55, 0x6e, 0x69, 0x71, 0x75, 0x65,
	0x54, 0x6f, 0x6b, 0x65, 0x6e, 0x88, 0x01, 0x01, 0x12, 0x28, 0x0a, 0x0d, 0x74, 0x61, 0x6c, 0x6f,
	0x73, 0x5f, 0x76, 0x65, 0x72, 0x73, 0x69, 0x6f, 0x6e, 0x18, 0x05, 0x20, 0x01, 0x28, 0x09, 0x48,
	0x02, 0x52, 0x0c, 0x74, 0x61, 0x6c, 0x6f, 0x73, 0x56, 0x65, 0x72, 0x73, 0x69, 0x6f, 0x6e, 0x88,
	0x01, 0x01, 0x12, 0x33, 0x0a, 0x13, 0x77, 0x69, 0x72, 0x65, 0x67, 0x75, 0x61, 0x72, 0x64, 0x5f,
	0x6f, 0x76, 0x65, 0x72, 0x5f, 0x67, 0x72, 0x70, 0x63, 0x18, 0x06, 0x20, 0x01, 0x28, 0x08, 0x48,
	0x03, 0x52, 0x11, 0x77, 0x69, 0x72, 0x65, 0x67, 0x75, 0x61, 0x72, 0x64, 0x4f, 0x76, 0x65, 0x72,
	0x47, 0x72, 0x70, 0x63, 0x88, 0x01, 0x01, 0x42, 0x0d, 0x0a, 0x0b, 0x5f, 0x6a, 0x6f, 0x69, 0x6e,
	0x5f, 0x74, 0x6f, 0x6b, 0x65, 0x6e, 0x42, 0x14, 0x0a, 0x12, 0x5f, 0x6e, 0x6f, 0x64, 0x65, 0x5f,
	0x75, 0x6e, 0x69, 0x71, 0x75, 0x65, 0x5f, 0x74, 0x6f, 0x6b, 0x65, 0x6e, 0x42, 0x10, 0x0a, 0x0e,
	0x5f, 0x74, 0x61, 0x6c, 0x6f, 0x73, 0x5f, 0x76, 0x65, 0x72, 0x73, 0x69, 0x6f, 0x6e, 0x42, 0x16,
	0x0a, 0x14, 0x5f, 0x77, 0x69, 0x72, 0x65, 0x67, 0x75, 0x61, 0x72, 0x64, 0x5f, 0x6f, 0x76, 0x65,
	0x72, 0x5f, 0x67, 0x72, 0x70, 0x63, 0x22, 0xee, 0x01, 0x0a, 0x11, 0x50, 0x72, 0x6f, 0x76, 0x69,
	0x73, 0x69, 0x6f, 0x6e, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x27, 0x0a, 0x0f,
	0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x5f, 0x65, 0x6e, 0x64, 0x70, 0x6f, 0x69, 0x6e, 0x74, 0x18,
	0x01, 0x20, 0x03, 0x28, 0x09, 0x52, 0x0e, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x45, 0x6e, 0x64,
	0x70, 0x6f, 0x69, 0x6e, 0x74, 0x12, 0x2a, 0x0a, 0x11, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x5f,
	0x70, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x5f, 0x6b, 0x65, 0x79, 0x18, 0x03, 0x20, 0x01, 0x28, 0x09,
	0x52, 0x0f, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x50, 0x75, 0x62, 0x6c, 0x69, 0x63, 0x4b, 0x65,
	0x79, 0x12, 0x2e, 0x0a, 0x13, 0x6e, 0x6f, 0x64, 0x65, 0x5f, 0x61, 0x64, 0x64, 0x72, 0x65, 0x73,
	0x73, 0x5f, 0x70, 0x72, 0x65, 0x66, 0x69, 0x78, 0x18, 0x04, 0x20, 0x01, 0x28, 0x09, 0x52, 0x11,
	0x6e, 0x6f, 0x64, 0x65, 0x41, 0x64, 0x64, 0x72, 0x65, 0x73, 0x73, 0x50, 0x72, 0x65, 0x66, 0x69,
	0x78, 0x12, 0x25, 0x0a, 0x0e, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x5f, 0x61, 0x64, 0x64, 0x72,
	0x65, 0x73, 0x73, 0x18, 0x05, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0d, 0x73, 0x65, 0x72, 0x76, 0x65,
	0x72, 0x41, 0x64, 0x64, 0x72, 0x65, 0x73, 0x73, 0x12, 0x2d, 0x0a, 0x13, 0x67, 0x72, 0x70, 0x63,
	0x5f, 0x70, 0x65, 0x65, 0x72, 0x5f, 0x61, 0x64, 0x64, 0x72, 0x5f, 0x70, 0x6f, 0x72, 0x74, 0x18,
	0x06, 0x20, 0x01, 0x28, 0x09, 0x52, 0x10, 0x67, 0x72, 0x70, 0x63, 0x50, 0x65, 0x65, 0x72, 0x41,
	0x64, 0x64, 0x72, 0x50, 0x6f, 0x72, 0x74, 0x32, 0x5e, 0x0a, 0x10, 0x50, 0x72, 0x6f, 0x76, 0x69,
	0x73, 0x69, 0x6f, 0x6e, 0x53, 0x65, 0x72, 0x76, 0x69, 0x63, 0x65, 0x12, 0x4a, 0x0a, 0x09, 0x50,
	0x72, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6f, 0x6e, 0x12, 0x1d, 0x2e, 0x73, 0x69, 0x64, 0x65, 0x72,
	0x6f, 0x2e, 0x6c, 0x69, 0x6e, 0x6b, 0x2e, 0x50, 0x72, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6f, 0x6e,
	0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x1e, 0x2e, 0x73, 0x69, 0x64, 0x65, 0x72, 0x6f,
	0x2e, 0x6c, 0x69, 0x6e, 0x6b, 0x2e, 0x50, 0x72, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6f, 0x6e, 0x52,
	0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x42, 0x34, 0x5a, 0x32, 0x67, 0x69, 0x74, 0x68, 0x75,
	0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x73, 0x69, 0x64, 0x65, 0x72, 0x6f, 0x6c, 0x61, 0x62, 0x73,
	0x2f, 0x73, 0x69, 0x64, 0x65, 0x72, 0x6f, 0x6c, 0x69, 0x6e, 0x6b, 0x2f, 0x61, 0x70, 0x69, 0x2f,
	0x73, 0x69, 0x64, 0x65, 0x72, 0x6f, 0x6c, 0x69, 0x6e, 0x6b, 0x2f, 0x70, 0x62, 0x62, 0x06, 0x70,
	0x72, 0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_siderolink_provision_proto_rawDescOnce sync.Once
	file_siderolink_provision_proto_rawDescData = file_siderolink_provision_proto_rawDesc
)

func file_siderolink_provision_proto_rawDescGZIP() []byte {
	file_siderolink_provision_proto_rawDescOnce.Do(func() {
		file_siderolink_provision_proto_rawDescData = protoimpl.X.CompressGZIP(file_siderolink_provision_proto_rawDescData)
	})
	return file_siderolink_provision_proto_rawDescData
}

var file_siderolink_provision_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_siderolink_provision_proto_goTypes = []interface{}{
	(*ProvisionRequest)(nil),  // 0: sidero.link.ProvisionRequest
	(*ProvisionResponse)(nil), // 1: sidero.link.ProvisionResponse
}
var file_siderolink_provision_proto_depIdxs = []int32{
	0, // 0: sidero.link.ProvisionService.Provision:input_type -> sidero.link.ProvisionRequest
	1, // 1: sidero.link.ProvisionService.Provision:output_type -> sidero.link.ProvisionResponse
	1, // [1:2] is the sub-list for method output_type
	0, // [0:1] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_siderolink_provision_proto_init() }
func file_siderolink_provision_proto_init() {
	if File_siderolink_provision_proto != nil {
		return
	}
	if !protoimpl.UnsafeEnabled {
		file_siderolink_provision_proto_msgTypes[0].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*ProvisionRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_siderolink_provision_proto_msgTypes[1].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*ProvisionResponse); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
	}
	file_siderolink_provision_proto_msgTypes[0].OneofWrappers = []interface{}{}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_siderolink_provision_proto_rawDesc,
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_siderolink_provision_proto_goTypes,
		DependencyIndexes: file_siderolink_provision_proto_depIdxs,
		MessageInfos:      file_siderolink_provision_proto_msgTypes,
	}.Build()
	File_siderolink_provision_proto = out.File
	file_siderolink_provision_proto_rawDesc = nil
	file_siderolink_provision_proto_goTypes = nil
	file_siderolink_provision_proto_depIdxs = nil
}
