// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

//nolint:revive
package vm

import "github.com/siderolabs/talos/pkg/provision"

// Deprecated: Use provision.State instead.
type State = provision.State

// Deprecated: Use provision.NewState instead.
var NewState = provision.NewState
