// Code generated by go-swagger; DO NOT EDIT.

package registry

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// AuthResponse An identity token was generated successfully.
//
// swagger:model AuthResponse
type AuthResponse struct {

	// An opaque token used to authenticate a user after a successful login
	// Example: 9cbaf023786cd7...
	IdentityToken string `json:"IdentityToken,omitempty"`

	// The status of the authentication
	// Example: Login Succeeded
	// Required: true
	Status string `json:"Status"`
}
