// Code generated by go-swagger; DO NOT EDIT.

package swarm

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// ServiceCreateResponse contains the information returned to a client on the
// creation of a new service.
//
// swagger:model ServiceCreateResponse
type ServiceCreateResponse struct {

	// The ID of the created service.
	// Example: ak7w3gjqoa3kuz8xcpnyy0pvl
	ID string `json:"ID,omitempty"`

	// Optional warning message.
	//
	// FIXME(thaJeztah): this should have "omitempty" in the generated type.
	//
	// Example: ["unable to pin image doesnotexist:latest to digest: image library/doesnotexist:latest not found"]
	Warnings []string `json:"Warnings"`
}
