// Code generated by go-swagger; DO NOT EDIT.

package storage

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// Storage Information about the storage used by the container.
//
// swagger:model Storage
type Storage struct {

	// Information about the storage used for the container's root filesystem.
	//
	RootFS *RootFSStorage `json:"RootFS,omitempty"`
}
