// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) Go Code Generator. DO NOT EDIT.

package azcertificates

// BackupCertificateResponse contains the response from method Client.BackupCertificate.
type BackupCertificateResponse struct {
	// The backup certificate result, containing the backup blob.
	BackupCertificateResult
}

// CreateCertificateResponse contains the response from method Client.CreateCertificate.
type CreateCertificateResponse struct {
	// A certificate operation is returned in case of asynchronous requests.
	CertificateOperation
}

// DeleteCertificateOperationResponse contains the response from method Client.DeleteCertificateOperation.
type DeleteCertificateOperationResponse struct {
	// A certificate operation is returned in case of asynchronous requests.
	CertificateOperation
}

// DeleteCertificateResponse contains the response from method Client.DeleteCertificate.
type DeleteCertificateResponse struct {
	// A Deleted Certificate consisting of its previous id, attributes and its tags, as well as information on when it will be
	// purged.
	DeletedCertificate
}

// DeleteContactsResponse contains the response from method Client.DeleteContacts.
type DeleteContactsResponse struct {
	// The contacts for the vault certificates.
	Contacts
}

// DeleteIssuerResponse contains the response from method Client.DeleteIssuer.
type DeleteIssuerResponse struct {
	// The issuer for Key Vault certificate.
	Issuer
}

// GetCertificateOperationResponse contains the response from method Client.GetCertificateOperation.
type GetCertificateOperationResponse struct {
	// A certificate operation is returned in case of asynchronous requests.
	CertificateOperation
}

// GetCertificatePolicyResponse contains the response from method Client.GetCertificatePolicy.
type GetCertificatePolicyResponse struct {
	// Management policy for a certificate.
	CertificatePolicy
}

// GetCertificateResponse contains the response from method Client.GetCertificate.
type GetCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// GetContactsResponse contains the response from method Client.GetContacts.
type GetContactsResponse struct {
	// The contacts for the vault certificates.
	Contacts
}

// GetDeletedCertificateResponse contains the response from method Client.GetDeletedCertificate.
type GetDeletedCertificateResponse struct {
	// A Deleted Certificate consisting of its previous id, attributes and its tags, as well as information on when it will be
	// purged.
	DeletedCertificate
}

// GetIssuerResponse contains the response from method Client.GetIssuer.
type GetIssuerResponse struct {
	// The issuer for Key Vault certificate.
	Issuer
}

// ImportCertificateResponse contains the response from method Client.ImportCertificate.
type ImportCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// ListCertificatePropertiesResponse contains the response from method Client.NewListCertificatePropertiesPager.
type ListCertificatePropertiesResponse struct {
	// The certificate list result.
	CertificatePropertiesListResult
}

// ListCertificatePropertiesVersionsResponse contains the response from method Client.NewListCertificatePropertiesVersionsPager.
type ListCertificatePropertiesVersionsResponse struct {
	// The certificate list result.
	CertificatePropertiesListResult
}

// ListDeletedCertificatePropertiesResponse contains the response from method Client.NewListDeletedCertificatePropertiesPager.
type ListDeletedCertificatePropertiesResponse struct {
	// A list of certificates that have been deleted in this vault.
	DeletedCertificatePropertiesListResult
}

// ListIssuerPropertiesResponse contains the response from method Client.NewListIssuerPropertiesPager.
type ListIssuerPropertiesResponse struct {
	// The certificate issuer list result.
	IssuerPropertiesListResult
}

// MergeCertificateResponse contains the response from method Client.MergeCertificate.
type MergeCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// PurgeDeletedCertificateResponse contains the response from method Client.PurgeDeletedCertificate.
type PurgeDeletedCertificateResponse struct {
	// placeholder for future response values
}

// RecoverDeletedCertificateResponse contains the response from method Client.RecoverDeletedCertificate.
type RecoverDeletedCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// RestoreCertificateResponse contains the response from method Client.RestoreCertificate.
type RestoreCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// SetContactsResponse contains the response from method Client.SetContacts.
type SetContactsResponse struct {
	// The contacts for the vault certificates.
	Contacts
}

// SetIssuerResponse contains the response from method Client.SetIssuer.
type SetIssuerResponse struct {
	// The issuer for Key Vault certificate.
	Issuer
}

// UpdateCertificateOperationResponse contains the response from method Client.UpdateCertificateOperation.
type UpdateCertificateOperationResponse struct {
	// A certificate operation is returned in case of asynchronous requests.
	CertificateOperation
}

// UpdateCertificatePolicyResponse contains the response from method Client.UpdateCertificatePolicy.
type UpdateCertificatePolicyResponse struct {
	// Management policy for a certificate.
	CertificatePolicy
}

// UpdateCertificateResponse contains the response from method Client.UpdateCertificate.
type UpdateCertificateResponse struct {
	// A certificate bundle consists of a certificate (X509) plus its attributes.
	Certificate
}

// UpdateIssuerResponse contains the response from method Client.UpdateIssuer.
type UpdateIssuerResponse struct {
	// The issuer for Key Vault certificate.
	Issuer
}
