// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11-devel
// 	protoc        (unknown)
// source: machine/machine.proto

package machine

import (
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"

	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"

	common "github.com/siderolabs/talos/pkg/machinery/api/common"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ApplyConfigurationRequest_Mode int32

const (
	ApplyConfigurationRequest_REBOOT    ApplyConfigurationRequest_Mode = 0
	ApplyConfigurationRequest_AUTO      ApplyConfigurationRequest_Mode = 1
	ApplyConfigurationRequest_NO_REBOOT ApplyConfigurationRequest_Mode = 2
	ApplyConfigurationRequest_STAGED    ApplyConfigurationRequest_Mode = 3
	ApplyConfigurationRequest_TRY       ApplyConfigurationRequest_Mode = 4
)

// Enum value maps for ApplyConfigurationRequest_Mode.
var (
	ApplyConfigurationRequest_Mode_name = map[int32]string{
		0: "REBOOT",
		1: "AUTO",
		2: "NO_REBOOT",
		3: "STAGED",
		4: "TRY",
	}
	ApplyConfigurationRequest_Mode_value = map[string]int32{
		"REBOOT":    0,
		"AUTO":      1,
		"NO_REBOOT": 2,
		"STAGED":    3,
		"TRY":       4,
	}
)

func (x ApplyConfigurationRequest_Mode) Enum() *ApplyConfigurationRequest_Mode {
	p := new(ApplyConfigurationRequest_Mode)
	*p = x
	return p
}

func (x ApplyConfigurationRequest_Mode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ApplyConfigurationRequest_Mode) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[0].Descriptor()
}

func (ApplyConfigurationRequest_Mode) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[0]
}

func (x ApplyConfigurationRequest_Mode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ApplyConfigurationRequest_Mode.Descriptor instead.
func (ApplyConfigurationRequest_Mode) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{0, 0}
}

type RebootRequest_Mode int32

const (
	RebootRequest_DEFAULT    RebootRequest_Mode = 0
	RebootRequest_POWERCYCLE RebootRequest_Mode = 1
	RebootRequest_FORCE      RebootRequest_Mode = 2
)

// Enum value maps for RebootRequest_Mode.
var (
	RebootRequest_Mode_name = map[int32]string{
		0: "DEFAULT",
		1: "POWERCYCLE",
		2: "FORCE",
	}
	RebootRequest_Mode_value = map[string]int32{
		"DEFAULT":    0,
		"POWERCYCLE": 1,
		"FORCE":      2,
	}
)

func (x RebootRequest_Mode) Enum() *RebootRequest_Mode {
	p := new(RebootRequest_Mode)
	*p = x
	return p
}

func (x RebootRequest_Mode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RebootRequest_Mode) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[1].Descriptor()
}

func (RebootRequest_Mode) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[1]
}

func (x RebootRequest_Mode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use RebootRequest_Mode.Descriptor instead.
func (RebootRequest_Mode) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{3, 0}
}

type SequenceEvent_Action int32

const (
	SequenceEvent_NOOP  SequenceEvent_Action = 0
	SequenceEvent_START SequenceEvent_Action = 1
	SequenceEvent_STOP  SequenceEvent_Action = 2
)

// Enum value maps for SequenceEvent_Action.
var (
	SequenceEvent_Action_name = map[int32]string{
		0: "NOOP",
		1: "START",
		2: "STOP",
	}
	SequenceEvent_Action_value = map[string]int32{
		"NOOP":  0,
		"START": 1,
		"STOP":  2,
	}
)

func (x SequenceEvent_Action) Enum() *SequenceEvent_Action {
	p := new(SequenceEvent_Action)
	*p = x
	return p
}

func (x SequenceEvent_Action) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (SequenceEvent_Action) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[2].Descriptor()
}

func (SequenceEvent_Action) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[2]
}

func (x SequenceEvent_Action) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use SequenceEvent_Action.Descriptor instead.
func (SequenceEvent_Action) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{9, 0}
}

type PhaseEvent_Action int32

const (
	PhaseEvent_START PhaseEvent_Action = 0
	PhaseEvent_STOP  PhaseEvent_Action = 1
)

// Enum value maps for PhaseEvent_Action.
var (
	PhaseEvent_Action_name = map[int32]string{
		0: "START",
		1: "STOP",
	}
	PhaseEvent_Action_value = map[string]int32{
		"START": 0,
		"STOP":  1,
	}
)

func (x PhaseEvent_Action) Enum() *PhaseEvent_Action {
	p := new(PhaseEvent_Action)
	*p = x
	return p
}

func (x PhaseEvent_Action) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (PhaseEvent_Action) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[3].Descriptor()
}

func (PhaseEvent_Action) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[3]
}

func (x PhaseEvent_Action) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use PhaseEvent_Action.Descriptor instead.
func (PhaseEvent_Action) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{10, 0}
}

type TaskEvent_Action int32

const (
	TaskEvent_START TaskEvent_Action = 0
	TaskEvent_STOP  TaskEvent_Action = 1
)

// Enum value maps for TaskEvent_Action.
var (
	TaskEvent_Action_name = map[int32]string{
		0: "START",
		1: "STOP",
	}
	TaskEvent_Action_value = map[string]int32{
		"START": 0,
		"STOP":  1,
	}
)

func (x TaskEvent_Action) Enum() *TaskEvent_Action {
	p := new(TaskEvent_Action)
	*p = x
	return p
}

func (x TaskEvent_Action) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (TaskEvent_Action) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[4].Descriptor()
}

func (TaskEvent_Action) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[4]
}

func (x TaskEvent_Action) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use TaskEvent_Action.Descriptor instead.
func (TaskEvent_Action) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{11, 0}
}

type ServiceStateEvent_Action int32

const (
	ServiceStateEvent_INITIALIZED ServiceStateEvent_Action = 0
	ServiceStateEvent_PREPARING   ServiceStateEvent_Action = 1
	ServiceStateEvent_WAITING     ServiceStateEvent_Action = 2
	ServiceStateEvent_RUNNING     ServiceStateEvent_Action = 3
	ServiceStateEvent_STOPPING    ServiceStateEvent_Action = 4
	ServiceStateEvent_FINISHED    ServiceStateEvent_Action = 5
	ServiceStateEvent_FAILED      ServiceStateEvent_Action = 6
	ServiceStateEvent_SKIPPED     ServiceStateEvent_Action = 7
	ServiceStateEvent_STARTING    ServiceStateEvent_Action = 8
)

// Enum value maps for ServiceStateEvent_Action.
var (
	ServiceStateEvent_Action_name = map[int32]string{
		0: "INITIALIZED",
		1: "PREPARING",
		2: "WAITING",
		3: "RUNNING",
		4: "STOPPING",
		5: "FINISHED",
		6: "FAILED",
		7: "SKIPPED",
		8: "STARTING",
	}
	ServiceStateEvent_Action_value = map[string]int32{
		"INITIALIZED": 0,
		"PREPARING":   1,
		"WAITING":     2,
		"RUNNING":     3,
		"STOPPING":    4,
		"FINISHED":    5,
		"FAILED":      6,
		"SKIPPED":     7,
		"STARTING":    8,
	}
)

func (x ServiceStateEvent_Action) Enum() *ServiceStateEvent_Action {
	p := new(ServiceStateEvent_Action)
	*p = x
	return p
}

func (x ServiceStateEvent_Action) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ServiceStateEvent_Action) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[5].Descriptor()
}

func (ServiceStateEvent_Action) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[5]
}

func (x ServiceStateEvent_Action) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ServiceStateEvent_Action.Descriptor instead.
func (ServiceStateEvent_Action) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{12, 0}
}

type MachineStatusEvent_MachineStage int32

const (
	MachineStatusEvent_UNKNOWN       MachineStatusEvent_MachineStage = 0
	MachineStatusEvent_BOOTING       MachineStatusEvent_MachineStage = 1
	MachineStatusEvent_INSTALLING    MachineStatusEvent_MachineStage = 2
	MachineStatusEvent_MAINTENANCE   MachineStatusEvent_MachineStage = 3
	MachineStatusEvent_RUNNING       MachineStatusEvent_MachineStage = 4
	MachineStatusEvent_REBOOTING     MachineStatusEvent_MachineStage = 5
	MachineStatusEvent_SHUTTING_DOWN MachineStatusEvent_MachineStage = 6
	MachineStatusEvent_RESETTING     MachineStatusEvent_MachineStage = 7
	MachineStatusEvent_UPGRADING     MachineStatusEvent_MachineStage = 8
)

// Enum value maps for MachineStatusEvent_MachineStage.
var (
	MachineStatusEvent_MachineStage_name = map[int32]string{
		0: "UNKNOWN",
		1: "BOOTING",
		2: "INSTALLING",
		3: "MAINTENANCE",
		4: "RUNNING",
		5: "REBOOTING",
		6: "SHUTTING_DOWN",
		7: "RESETTING",
		8: "UPGRADING",
	}
	MachineStatusEvent_MachineStage_value = map[string]int32{
		"UNKNOWN":       0,
		"BOOTING":       1,
		"INSTALLING":    2,
		"MAINTENANCE":   3,
		"RUNNING":       4,
		"REBOOTING":     5,
		"SHUTTING_DOWN": 6,
		"RESETTING":     7,
		"UPGRADING":     8,
	}
)

func (x MachineStatusEvent_MachineStage) Enum() *MachineStatusEvent_MachineStage {
	p := new(MachineStatusEvent_MachineStage)
	*p = x
	return p
}

func (x MachineStatusEvent_MachineStage) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (MachineStatusEvent_MachineStage) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[6].Descriptor()
}

func (MachineStatusEvent_MachineStage) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[6]
}

func (x MachineStatusEvent_MachineStage) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use MachineStatusEvent_MachineStage.Descriptor instead.
func (MachineStatusEvent_MachineStage) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{17, 0}
}

type ResetRequest_WipeMode int32

const (
	ResetRequest_ALL         ResetRequest_WipeMode = 0
	ResetRequest_SYSTEM_DISK ResetRequest_WipeMode = 1
	ResetRequest_USER_DISKS  ResetRequest_WipeMode = 2
)

// Enum value maps for ResetRequest_WipeMode.
var (
	ResetRequest_WipeMode_name = map[int32]string{
		0: "ALL",
		1: "SYSTEM_DISK",
		2: "USER_DISKS",
	}
	ResetRequest_WipeMode_value = map[string]int32{
		"ALL":         0,
		"SYSTEM_DISK": 1,
		"USER_DISKS":  2,
	}
)

func (x ResetRequest_WipeMode) Enum() *ResetRequest_WipeMode {
	p := new(ResetRequest_WipeMode)
	*p = x
	return p
}

func (x ResetRequest_WipeMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ResetRequest_WipeMode) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[7].Descriptor()
}

func (ResetRequest_WipeMode) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[7]
}

func (x ResetRequest_WipeMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ResetRequest_WipeMode.Descriptor instead.
func (ResetRequest_WipeMode) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{21, 0}
}

type UpgradeRequest_RebootMode int32

const (
	UpgradeRequest_DEFAULT    UpgradeRequest_RebootMode = 0
	UpgradeRequest_POWERCYCLE UpgradeRequest_RebootMode = 1
)

// Enum value maps for UpgradeRequest_RebootMode.
var (
	UpgradeRequest_RebootMode_name = map[int32]string{
		0: "DEFAULT",
		1: "POWERCYCLE",
	}
	UpgradeRequest_RebootMode_value = map[string]int32{
		"DEFAULT":    0,
		"POWERCYCLE": 1,
	}
)

func (x UpgradeRequest_RebootMode) Enum() *UpgradeRequest_RebootMode {
	p := new(UpgradeRequest_RebootMode)
	*p = x
	return p
}

func (x UpgradeRequest_RebootMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (UpgradeRequest_RebootMode) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[8].Descriptor()
}

func (UpgradeRequest_RebootMode) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[8]
}

func (x UpgradeRequest_RebootMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use UpgradeRequest_RebootMode.Descriptor instead.
func (UpgradeRequest_RebootMode) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{27, 0}
}

// File type.
type ListRequest_Type int32

const (
	// Regular file (not directory, symlink, etc).
	ListRequest_REGULAR ListRequest_Type = 0
	// Directory.
	ListRequest_DIRECTORY ListRequest_Type = 1
	// Symbolic link.
	ListRequest_SYMLINK ListRequest_Type = 2
)

// Enum value maps for ListRequest_Type.
var (
	ListRequest_Type_name = map[int32]string{
		0: "REGULAR",
		1: "DIRECTORY",
		2: "SYMLINK",
	}
	ListRequest_Type_value = map[string]int32{
		"REGULAR":   0,
		"DIRECTORY": 1,
		"SYMLINK":   2,
	}
)

func (x ListRequest_Type) Enum() *ListRequest_Type {
	p := new(ListRequest_Type)
	*p = x
	return p
}

func (x ListRequest_Type) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ListRequest_Type) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[9].Descriptor()
}

func (ListRequest_Type) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[9]
}

func (x ListRequest_Type) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ListRequest_Type.Descriptor instead.
func (ListRequest_Type) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{46, 0}
}

type EtcdMemberAlarm_AlarmType int32

const (
	EtcdMemberAlarm_NONE    EtcdMemberAlarm_AlarmType = 0
	EtcdMemberAlarm_NOSPACE EtcdMemberAlarm_AlarmType = 1
	EtcdMemberAlarm_CORRUPT EtcdMemberAlarm_AlarmType = 2
)

// Enum value maps for EtcdMemberAlarm_AlarmType.
var (
	EtcdMemberAlarm_AlarmType_name = map[int32]string{
		0: "NONE",
		1: "NOSPACE",
		2: "CORRUPT",
	}
	EtcdMemberAlarm_AlarmType_value = map[string]int32{
		"NONE":    0,
		"NOSPACE": 1,
		"CORRUPT": 2,
	}
)

func (x EtcdMemberAlarm_AlarmType) Enum() *EtcdMemberAlarm_AlarmType {
	p := new(EtcdMemberAlarm_AlarmType)
	*p = x
	return p
}

func (x EtcdMemberAlarm_AlarmType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (EtcdMemberAlarm_AlarmType) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[10].Descriptor()
}

func (EtcdMemberAlarm_AlarmType) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[10]
}

func (x EtcdMemberAlarm_AlarmType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use EtcdMemberAlarm_AlarmType.Descriptor instead.
func (EtcdMemberAlarm_AlarmType) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{125, 0}
}

type MachineConfig_MachineType int32

const (
	MachineConfig_TYPE_UNKNOWN       MachineConfig_MachineType = 0
	MachineConfig_TYPE_INIT          MachineConfig_MachineType = 1
	MachineConfig_TYPE_CONTROL_PLANE MachineConfig_MachineType = 2
	MachineConfig_TYPE_WORKER        MachineConfig_MachineType = 3
)

// Enum value maps for MachineConfig_MachineType.
var (
	MachineConfig_MachineType_name = map[int32]string{
		0: "TYPE_UNKNOWN",
		1: "TYPE_INIT",
		2: "TYPE_CONTROL_PLANE",
		3: "TYPE_WORKER",
	}
	MachineConfig_MachineType_value = map[string]int32{
		"TYPE_UNKNOWN":       0,
		"TYPE_INIT":          1,
		"TYPE_CONTROL_PLANE": 2,
		"TYPE_WORKER":        3,
	}
)

func (x MachineConfig_MachineType) Enum() *MachineConfig_MachineType {
	p := new(MachineConfig_MachineType)
	*p = x
	return p
}

func (x MachineConfig_MachineType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (MachineConfig_MachineType) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[11].Descriptor()
}

func (MachineConfig_MachineType) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[11]
}

func (x MachineConfig_MachineType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use MachineConfig_MachineType.Descriptor instead.
func (MachineConfig_MachineType) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{147, 0}
}

type NetstatRequest_Filter int32

const (
	NetstatRequest_ALL       NetstatRequest_Filter = 0
	NetstatRequest_CONNECTED NetstatRequest_Filter = 1
	NetstatRequest_LISTENING NetstatRequest_Filter = 2
)

// Enum value maps for NetstatRequest_Filter.
var (
	NetstatRequest_Filter_name = map[int32]string{
		0: "ALL",
		1: "CONNECTED",
		2: "LISTENING",
	}
	NetstatRequest_Filter_value = map[string]int32{
		"ALL":       0,
		"CONNECTED": 1,
		"LISTENING": 2,
	}
)

func (x NetstatRequest_Filter) Enum() *NetstatRequest_Filter {
	p := new(NetstatRequest_Filter)
	*p = x
	return p
}

func (x NetstatRequest_Filter) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (NetstatRequest_Filter) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[12].Descriptor()
}

func (NetstatRequest_Filter) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[12]
}

func (x NetstatRequest_Filter) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use NetstatRequest_Filter.Descriptor instead.
func (NetstatRequest_Filter) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{157, 0}
}

type ConnectRecord_State int32

const (
	ConnectRecord_RESERVED    ConnectRecord_State = 0
	ConnectRecord_ESTABLISHED ConnectRecord_State = 1
	ConnectRecord_SYN_SENT    ConnectRecord_State = 2
	ConnectRecord_SYN_RECV    ConnectRecord_State = 3
	ConnectRecord_FIN_WAIT1   ConnectRecord_State = 4
	ConnectRecord_FIN_WAIT2   ConnectRecord_State = 5
	ConnectRecord_TIME_WAIT   ConnectRecord_State = 6
	ConnectRecord_CLOSE       ConnectRecord_State = 7
	ConnectRecord_CLOSEWAIT   ConnectRecord_State = 8
	ConnectRecord_LASTACK     ConnectRecord_State = 9
	ConnectRecord_LISTEN      ConnectRecord_State = 10
	ConnectRecord_CLOSING     ConnectRecord_State = 11
)

// Enum value maps for ConnectRecord_State.
var (
	ConnectRecord_State_name = map[int32]string{
		0:  "RESERVED",
		1:  "ESTABLISHED",
		2:  "SYN_SENT",
		3:  "SYN_RECV",
		4:  "FIN_WAIT1",
		5:  "FIN_WAIT2",
		6:  "TIME_WAIT",
		7:  "CLOSE",
		8:  "CLOSEWAIT",
		9:  "LASTACK",
		10: "LISTEN",
		11: "CLOSING",
	}
	ConnectRecord_State_value = map[string]int32{
		"RESERVED":    0,
		"ESTABLISHED": 1,
		"SYN_SENT":    2,
		"SYN_RECV":    3,
		"FIN_WAIT1":   4,
		"FIN_WAIT2":   5,
		"TIME_WAIT":   6,
		"CLOSE":       7,
		"CLOSEWAIT":   8,
		"LASTACK":     9,
		"LISTEN":      10,
		"CLOSING":     11,
	}
)

func (x ConnectRecord_State) Enum() *ConnectRecord_State {
	p := new(ConnectRecord_State)
	*p = x
	return p
}

func (x ConnectRecord_State) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ConnectRecord_State) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[13].Descriptor()
}

func (ConnectRecord_State) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[13]
}

func (x ConnectRecord_State) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ConnectRecord_State.Descriptor instead.
func (ConnectRecord_State) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{158, 0}
}

type ConnectRecord_TimerActive int32

const (
	ConnectRecord_OFF       ConnectRecord_TimerActive = 0
	ConnectRecord_ON        ConnectRecord_TimerActive = 1
	ConnectRecord_KEEPALIVE ConnectRecord_TimerActive = 2
	ConnectRecord_TIMEWAIT  ConnectRecord_TimerActive = 3
	ConnectRecord_PROBE     ConnectRecord_TimerActive = 4
)

// Enum value maps for ConnectRecord_TimerActive.
var (
	ConnectRecord_TimerActive_name = map[int32]string{
		0: "OFF",
		1: "ON",
		2: "KEEPALIVE",
		3: "TIMEWAIT",
		4: "PROBE",
	}
	ConnectRecord_TimerActive_value = map[string]int32{
		"OFF":       0,
		"ON":        1,
		"KEEPALIVE": 2,
		"TIMEWAIT":  3,
		"PROBE":     4,
	}
)

func (x ConnectRecord_TimerActive) Enum() *ConnectRecord_TimerActive {
	p := new(ConnectRecord_TimerActive)
	*p = x
	return p
}

func (x ConnectRecord_TimerActive) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ConnectRecord_TimerActive) Descriptor() protoreflect.EnumDescriptor {
	return file_machine_machine_proto_enumTypes[14].Descriptor()
}

func (ConnectRecord_TimerActive) Type() protoreflect.EnumType {
	return &file_machine_machine_proto_enumTypes[14]
}

func (x ConnectRecord_TimerActive) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ConnectRecord_TimerActive.Descriptor instead.
func (ConnectRecord_TimerActive) EnumDescriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{158, 1}
}

// rpc applyConfiguration
// ApplyConfiguration describes a request to assert a new configuration upon a
// node.
type ApplyConfigurationRequest struct {
	state          protoimpl.MessageState         `protogen:"open.v1"`
	Data           []byte                         `protobuf:"bytes,1,opt,name=data,proto3" json:"data,omitempty"`
	Mode           ApplyConfigurationRequest_Mode `protobuf:"varint,4,opt,name=mode,proto3,enum=machine.ApplyConfigurationRequest_Mode" json:"mode,omitempty"`
	DryRun         bool                           `protobuf:"varint,5,opt,name=dry_run,json=dryRun,proto3" json:"dry_run,omitempty"`
	TryModeTimeout *durationpb.Duration           `protobuf:"bytes,6,opt,name=try_mode_timeout,json=tryModeTimeout,proto3" json:"try_mode_timeout,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ApplyConfigurationRequest) Reset() {
	*x = ApplyConfigurationRequest{}
	mi := &file_machine_machine_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ApplyConfigurationRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ApplyConfigurationRequest) ProtoMessage() {}

func (x *ApplyConfigurationRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ApplyConfigurationRequest.ProtoReflect.Descriptor instead.
func (*ApplyConfigurationRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{0}
}

func (x *ApplyConfigurationRequest) GetData() []byte {
	if x != nil {
		return x.Data
	}
	return nil
}

func (x *ApplyConfigurationRequest) GetMode() ApplyConfigurationRequest_Mode {
	if x != nil {
		return x.Mode
	}
	return ApplyConfigurationRequest_REBOOT
}

func (x *ApplyConfigurationRequest) GetDryRun() bool {
	if x != nil {
		return x.DryRun
	}
	return false
}

func (x *ApplyConfigurationRequest) GetTryModeTimeout() *durationpb.Duration {
	if x != nil {
		return x.TryModeTimeout
	}
	return nil
}

// ApplyConfigurationResponse describes the response to a configuration request.
type ApplyConfiguration struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// Configuration validation warnings.
	Warnings []string `protobuf:"bytes,2,rep,name=warnings,proto3" json:"warnings,omitempty"`
	// States which mode was actually chosen.
	Mode ApplyConfigurationRequest_Mode `protobuf:"varint,3,opt,name=mode,proto3,enum=machine.ApplyConfigurationRequest_Mode" json:"mode,omitempty"`
	// Human-readable message explaining the result of the apply configuration call.
	ModeDetails   string `protobuf:"bytes,4,opt,name=mode_details,json=modeDetails,proto3" json:"mode_details,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ApplyConfiguration) Reset() {
	*x = ApplyConfiguration{}
	mi := &file_machine_machine_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ApplyConfiguration) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ApplyConfiguration) ProtoMessage() {}

func (x *ApplyConfiguration) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ApplyConfiguration.ProtoReflect.Descriptor instead.
func (*ApplyConfiguration) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{1}
}

func (x *ApplyConfiguration) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ApplyConfiguration) GetWarnings() []string {
	if x != nil {
		return x.Warnings
	}
	return nil
}

func (x *ApplyConfiguration) GetMode() ApplyConfigurationRequest_Mode {
	if x != nil {
		return x.Mode
	}
	return ApplyConfigurationRequest_REBOOT
}

func (x *ApplyConfiguration) GetModeDetails() string {
	if x != nil {
		return x.ModeDetails
	}
	return ""
}

type ApplyConfigurationResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ApplyConfiguration  `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ApplyConfigurationResponse) Reset() {
	*x = ApplyConfigurationResponse{}
	mi := &file_machine_machine_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ApplyConfigurationResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ApplyConfigurationResponse) ProtoMessage() {}

func (x *ApplyConfigurationResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ApplyConfigurationResponse.ProtoReflect.Descriptor instead.
func (*ApplyConfigurationResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{2}
}

func (x *ApplyConfigurationResponse) GetMessages() []*ApplyConfiguration {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc reboot
type RebootRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Mode          RebootRequest_Mode     `protobuf:"varint,1,opt,name=mode,proto3,enum=machine.RebootRequest_Mode" json:"mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RebootRequest) Reset() {
	*x = RebootRequest{}
	mi := &file_machine_machine_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RebootRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RebootRequest) ProtoMessage() {}

func (x *RebootRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RebootRequest.ProtoReflect.Descriptor instead.
func (*RebootRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{3}
}

func (x *RebootRequest) GetMode() RebootRequest_Mode {
	if x != nil {
		return x.Mode
	}
	return RebootRequest_DEFAULT
}

// The reboot message containing the reboot status.
type Reboot struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ActorId       string                 `protobuf:"bytes,2,opt,name=actor_id,json=actorId,proto3" json:"actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Reboot) Reset() {
	*x = Reboot{}
	mi := &file_machine_machine_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Reboot) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Reboot) ProtoMessage() {}

func (x *Reboot) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Reboot.ProtoReflect.Descriptor instead.
func (*Reboot) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{4}
}

func (x *Reboot) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Reboot) GetActorId() string {
	if x != nil {
		return x.ActorId
	}
	return ""
}

type RebootResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Reboot              `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RebootResponse) Reset() {
	*x = RebootResponse{}
	mi := &file_machine_machine_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RebootResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RebootResponse) ProtoMessage() {}

func (x *RebootResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RebootResponse.ProtoReflect.Descriptor instead.
func (*RebootResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{5}
}

func (x *RebootResponse) GetMessages() []*Reboot {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc Bootstrap
type BootstrapRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Enable etcd recovery from the snapshot.
	// Snapshot should be uploaded before this call via EtcdRecover RPC.
	RecoverEtcd bool `protobuf:"varint,1,opt,name=recover_etcd,json=recoverEtcd,proto3" json:"recover_etcd,omitempty"`
	// Skip hash check on the snapshot (etcd).
	// Enable this when recovering from data directory copy to skip integrity check.
	RecoverSkipHashCheck bool `protobuf:"varint,2,opt,name=recover_skip_hash_check,json=recoverSkipHashCheck,proto3" json:"recover_skip_hash_check,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *BootstrapRequest) Reset() {
	*x = BootstrapRequest{}
	mi := &file_machine_machine_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BootstrapRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BootstrapRequest) ProtoMessage() {}

func (x *BootstrapRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BootstrapRequest.ProtoReflect.Descriptor instead.
func (*BootstrapRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{6}
}

func (x *BootstrapRequest) GetRecoverEtcd() bool {
	if x != nil {
		return x.RecoverEtcd
	}
	return false
}

func (x *BootstrapRequest) GetRecoverSkipHashCheck() bool {
	if x != nil {
		return x.RecoverSkipHashCheck
	}
	return false
}

// The bootstrap message containing the bootstrap status.
type Bootstrap struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Bootstrap) Reset() {
	*x = Bootstrap{}
	mi := &file_machine_machine_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Bootstrap) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Bootstrap) ProtoMessage() {}

func (x *Bootstrap) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Bootstrap.ProtoReflect.Descriptor instead.
func (*Bootstrap) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{7}
}

func (x *Bootstrap) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type BootstrapResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Bootstrap           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BootstrapResponse) Reset() {
	*x = BootstrapResponse{}
	mi := &file_machine_machine_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BootstrapResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BootstrapResponse) ProtoMessage() {}

func (x *BootstrapResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BootstrapResponse.ProtoReflect.Descriptor instead.
func (*BootstrapResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{8}
}

func (x *BootstrapResponse) GetMessages() []*Bootstrap {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc events
type SequenceEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Sequence      string                 `protobuf:"bytes,1,opt,name=sequence,proto3" json:"sequence,omitempty"`
	Action        SequenceEvent_Action   `protobuf:"varint,2,opt,name=action,proto3,enum=machine.SequenceEvent_Action" json:"action,omitempty"`
	Error         *common.Error          `protobuf:"bytes,3,opt,name=error,proto3" json:"error,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SequenceEvent) Reset() {
	*x = SequenceEvent{}
	mi := &file_machine_machine_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SequenceEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SequenceEvent) ProtoMessage() {}

func (x *SequenceEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SequenceEvent.ProtoReflect.Descriptor instead.
func (*SequenceEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{9}
}

func (x *SequenceEvent) GetSequence() string {
	if x != nil {
		return x.Sequence
	}
	return ""
}

func (x *SequenceEvent) GetAction() SequenceEvent_Action {
	if x != nil {
		return x.Action
	}
	return SequenceEvent_NOOP
}

func (x *SequenceEvent) GetError() *common.Error {
	if x != nil {
		return x.Error
	}
	return nil
}

type PhaseEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Phase         string                 `protobuf:"bytes,1,opt,name=phase,proto3" json:"phase,omitempty"`
	Action        PhaseEvent_Action      `protobuf:"varint,2,opt,name=action,proto3,enum=machine.PhaseEvent_Action" json:"action,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PhaseEvent) Reset() {
	*x = PhaseEvent{}
	mi := &file_machine_machine_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PhaseEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PhaseEvent) ProtoMessage() {}

func (x *PhaseEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PhaseEvent.ProtoReflect.Descriptor instead.
func (*PhaseEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{10}
}

func (x *PhaseEvent) GetPhase() string {
	if x != nil {
		return x.Phase
	}
	return ""
}

func (x *PhaseEvent) GetAction() PhaseEvent_Action {
	if x != nil {
		return x.Action
	}
	return PhaseEvent_START
}

type TaskEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Task          string                 `protobuf:"bytes,1,opt,name=task,proto3" json:"task,omitempty"`
	Action        TaskEvent_Action       `protobuf:"varint,2,opt,name=action,proto3,enum=machine.TaskEvent_Action" json:"action,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TaskEvent) Reset() {
	*x = TaskEvent{}
	mi := &file_machine_machine_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TaskEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TaskEvent) ProtoMessage() {}

func (x *TaskEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TaskEvent.ProtoReflect.Descriptor instead.
func (*TaskEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{11}
}

func (x *TaskEvent) GetTask() string {
	if x != nil {
		return x.Task
	}
	return ""
}

func (x *TaskEvent) GetAction() TaskEvent_Action {
	if x != nil {
		return x.Action
	}
	return TaskEvent_START
}

type ServiceStateEvent struct {
	state         protoimpl.MessageState   `protogen:"open.v1"`
	Service       string                   `protobuf:"bytes,1,opt,name=service,proto3" json:"service,omitempty"`
	Action        ServiceStateEvent_Action `protobuf:"varint,2,opt,name=action,proto3,enum=machine.ServiceStateEvent_Action" json:"action,omitempty"`
	Message       string                   `protobuf:"bytes,3,opt,name=message,proto3" json:"message,omitempty"`
	Health        *ServiceHealth           `protobuf:"bytes,4,opt,name=health,proto3" json:"health,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStateEvent) Reset() {
	*x = ServiceStateEvent{}
	mi := &file_machine_machine_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStateEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStateEvent) ProtoMessage() {}

func (x *ServiceStateEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStateEvent.ProtoReflect.Descriptor instead.
func (*ServiceStateEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{12}
}

func (x *ServiceStateEvent) GetService() string {
	if x != nil {
		return x.Service
	}
	return ""
}

func (x *ServiceStateEvent) GetAction() ServiceStateEvent_Action {
	if x != nil {
		return x.Action
	}
	return ServiceStateEvent_INITIALIZED
}

func (x *ServiceStateEvent) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

func (x *ServiceStateEvent) GetHealth() *ServiceHealth {
	if x != nil {
		return x.Health
	}
	return nil
}

type RestartEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Cmd           int64                  `protobuf:"varint,1,opt,name=cmd,proto3" json:"cmd,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RestartEvent) Reset() {
	*x = RestartEvent{}
	mi := &file_machine_machine_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RestartEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestartEvent) ProtoMessage() {}

func (x *RestartEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestartEvent.ProtoReflect.Descriptor instead.
func (*RestartEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{13}
}

func (x *RestartEvent) GetCmd() int64 {
	if x != nil {
		return x.Cmd
	}
	return 0
}

// ConfigLoadErrorEvent is reported when the config loading has failed.
type ConfigLoadErrorEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Error         string                 `protobuf:"bytes,1,opt,name=error,proto3" json:"error,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConfigLoadErrorEvent) Reset() {
	*x = ConfigLoadErrorEvent{}
	mi := &file_machine_machine_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConfigLoadErrorEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConfigLoadErrorEvent) ProtoMessage() {}

func (x *ConfigLoadErrorEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConfigLoadErrorEvent.ProtoReflect.Descriptor instead.
func (*ConfigLoadErrorEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{14}
}

func (x *ConfigLoadErrorEvent) GetError() string {
	if x != nil {
		return x.Error
	}
	return ""
}

// ConfigValidationErrorEvent is reported when config validation has failed.
type ConfigValidationErrorEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Error         string                 `protobuf:"bytes,1,opt,name=error,proto3" json:"error,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConfigValidationErrorEvent) Reset() {
	*x = ConfigValidationErrorEvent{}
	mi := &file_machine_machine_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConfigValidationErrorEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConfigValidationErrorEvent) ProtoMessage() {}

func (x *ConfigValidationErrorEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConfigValidationErrorEvent.ProtoReflect.Descriptor instead.
func (*ConfigValidationErrorEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{15}
}

func (x *ConfigValidationErrorEvent) GetError() string {
	if x != nil {
		return x.Error
	}
	return ""
}

// AddressEvent reports node endpoints aggregated from k8s.Endpoints and network.Hostname.
type AddressEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Hostname      string                 `protobuf:"bytes,1,opt,name=hostname,proto3" json:"hostname,omitempty"`
	Addresses     []string               `protobuf:"bytes,2,rep,name=addresses,proto3" json:"addresses,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AddressEvent) Reset() {
	*x = AddressEvent{}
	mi := &file_machine_machine_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AddressEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AddressEvent) ProtoMessage() {}

func (x *AddressEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AddressEvent.ProtoReflect.Descriptor instead.
func (*AddressEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{16}
}

func (x *AddressEvent) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

func (x *AddressEvent) GetAddresses() []string {
	if x != nil {
		return x.Addresses
	}
	return nil
}

// MachineStatusEvent reports changes to the MachineStatus resource.
type MachineStatusEvent struct {
	state         protoimpl.MessageState            `protogen:"open.v1"`
	Stage         MachineStatusEvent_MachineStage   `protobuf:"varint,1,opt,name=stage,proto3,enum=machine.MachineStatusEvent_MachineStage" json:"stage,omitempty"`
	Status        *MachineStatusEvent_MachineStatus `protobuf:"bytes,2,opt,name=status,proto3" json:"status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MachineStatusEvent) Reset() {
	*x = MachineStatusEvent{}
	mi := &file_machine_machine_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineStatusEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineStatusEvent) ProtoMessage() {}

func (x *MachineStatusEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineStatusEvent.ProtoReflect.Descriptor instead.
func (*MachineStatusEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{17}
}

func (x *MachineStatusEvent) GetStage() MachineStatusEvent_MachineStage {
	if x != nil {
		return x.Stage
	}
	return MachineStatusEvent_UNKNOWN
}

func (x *MachineStatusEvent) GetStatus() *MachineStatusEvent_MachineStatus {
	if x != nil {
		return x.Status
	}
	return nil
}

type EventsRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	TailEvents    int32                  `protobuf:"varint,1,opt,name=tail_events,json=tailEvents,proto3" json:"tail_events,omitempty"`
	TailId        string                 `protobuf:"bytes,2,opt,name=tail_id,json=tailId,proto3" json:"tail_id,omitempty"`
	TailSeconds   int32                  `protobuf:"varint,3,opt,name=tail_seconds,json=tailSeconds,proto3" json:"tail_seconds,omitempty"`
	WithActorId   string                 `protobuf:"bytes,4,opt,name=with_actor_id,json=withActorId,proto3" json:"with_actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EventsRequest) Reset() {
	*x = EventsRequest{}
	mi := &file_machine_machine_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EventsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EventsRequest) ProtoMessage() {}

func (x *EventsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EventsRequest.ProtoReflect.Descriptor instead.
func (*EventsRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{18}
}

func (x *EventsRequest) GetTailEvents() int32 {
	if x != nil {
		return x.TailEvents
	}
	return 0
}

func (x *EventsRequest) GetTailId() string {
	if x != nil {
		return x.TailId
	}
	return ""
}

func (x *EventsRequest) GetTailSeconds() int32 {
	if x != nil {
		return x.TailSeconds
	}
	return 0
}

func (x *EventsRequest) GetWithActorId() string {
	if x != nil {
		return x.WithActorId
	}
	return ""
}

type Event struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Data          *anypb.Any             `protobuf:"bytes,2,opt,name=data,proto3" json:"data,omitempty"`
	Id            string                 `protobuf:"bytes,3,opt,name=id,proto3" json:"id,omitempty"`
	ActorId       string                 `protobuf:"bytes,4,opt,name=actor_id,json=actorId,proto3" json:"actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Event) Reset() {
	*x = Event{}
	mi := &file_machine_machine_proto_msgTypes[19]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Event) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Event) ProtoMessage() {}

func (x *Event) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[19]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Event.ProtoReflect.Descriptor instead.
func (*Event) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{19}
}

func (x *Event) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Event) GetData() *anypb.Any {
	if x != nil {
		return x.Data
	}
	return nil
}

func (x *Event) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *Event) GetActorId() string {
	if x != nil {
		return x.ActorId
	}
	return ""
}

// rpc reset
type ResetPartitionSpec struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Label         string                 `protobuf:"bytes,1,opt,name=label,proto3" json:"label,omitempty"`
	Wipe          bool                   `protobuf:"varint,2,opt,name=wipe,proto3" json:"wipe,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResetPartitionSpec) Reset() {
	*x = ResetPartitionSpec{}
	mi := &file_machine_machine_proto_msgTypes[20]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResetPartitionSpec) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResetPartitionSpec) ProtoMessage() {}

func (x *ResetPartitionSpec) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[20]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResetPartitionSpec.ProtoReflect.Descriptor instead.
func (*ResetPartitionSpec) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{20}
}

func (x *ResetPartitionSpec) GetLabel() string {
	if x != nil {
		return x.Label
	}
	return ""
}

func (x *ResetPartitionSpec) GetWipe() bool {
	if x != nil {
		return x.Wipe
	}
	return false
}

type ResetRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Graceful indicates whether node should leave etcd before the upgrade, it also
	// enforces etcd checks before leaving.
	Graceful bool `protobuf:"varint,1,opt,name=graceful,proto3" json:"graceful,omitempty"`
	// Reboot indicates whether node should reboot or halt after resetting.
	Reboot bool `protobuf:"varint,2,opt,name=reboot,proto3" json:"reboot,omitempty"`
	// System_partitions_to_wipe lists specific system disk partitions to be reset (wiped).
	// If system_partitions_to_wipe is empty, all the partitions are erased.
	SystemPartitionsToWipe []*ResetPartitionSpec `protobuf:"bytes,3,rep,name=system_partitions_to_wipe,json=systemPartitionsToWipe,proto3" json:"system_partitions_to_wipe,omitempty"`
	// UserDisksToWipe lists specific connected block devices to be reset (wiped).
	UserDisksToWipe []string `protobuf:"bytes,4,rep,name=user_disks_to_wipe,json=userDisksToWipe,proto3" json:"user_disks_to_wipe,omitempty"`
	// WipeMode defines which devices should be wiped.
	Mode          ResetRequest_WipeMode `protobuf:"varint,5,opt,name=mode,proto3,enum=machine.ResetRequest_WipeMode" json:"mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResetRequest) Reset() {
	*x = ResetRequest{}
	mi := &file_machine_machine_proto_msgTypes[21]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResetRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResetRequest) ProtoMessage() {}

func (x *ResetRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[21]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResetRequest.ProtoReflect.Descriptor instead.
func (*ResetRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{21}
}

func (x *ResetRequest) GetGraceful() bool {
	if x != nil {
		return x.Graceful
	}
	return false
}

func (x *ResetRequest) GetReboot() bool {
	if x != nil {
		return x.Reboot
	}
	return false
}

func (x *ResetRequest) GetSystemPartitionsToWipe() []*ResetPartitionSpec {
	if x != nil {
		return x.SystemPartitionsToWipe
	}
	return nil
}

func (x *ResetRequest) GetUserDisksToWipe() []string {
	if x != nil {
		return x.UserDisksToWipe
	}
	return nil
}

func (x *ResetRequest) GetMode() ResetRequest_WipeMode {
	if x != nil {
		return x.Mode
	}
	return ResetRequest_ALL
}

// The reset message containing the restart status.
type Reset struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ActorId       string                 `protobuf:"bytes,2,opt,name=actor_id,json=actorId,proto3" json:"actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Reset) Reset() {
	*x = Reset{}
	mi := &file_machine_machine_proto_msgTypes[22]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Reset) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Reset) ProtoMessage() {}

func (x *Reset) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[22]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Reset.ProtoReflect.Descriptor instead.
func (*Reset) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{22}
}

func (x *Reset) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Reset) GetActorId() string {
	if x != nil {
		return x.ActorId
	}
	return ""
}

type ResetResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Reset               `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResetResponse) Reset() {
	*x = ResetResponse{}
	mi := &file_machine_machine_proto_msgTypes[23]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResetResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResetResponse) ProtoMessage() {}

func (x *ResetResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[23]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResetResponse.ProtoReflect.Descriptor instead.
func (*ResetResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{23}
}

func (x *ResetResponse) GetMessages() []*Reset {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc shutdown
// The messages message containing the shutdown status.
type Shutdown struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ActorId       string                 `protobuf:"bytes,2,opt,name=actor_id,json=actorId,proto3" json:"actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Shutdown) Reset() {
	*x = Shutdown{}
	mi := &file_machine_machine_proto_msgTypes[24]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Shutdown) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Shutdown) ProtoMessage() {}

func (x *Shutdown) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[24]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Shutdown.ProtoReflect.Descriptor instead.
func (*Shutdown) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{24}
}

func (x *Shutdown) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Shutdown) GetActorId() string {
	if x != nil {
		return x.ActorId
	}
	return ""
}

type ShutdownRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Force indicates whether node should shutdown without first cordening and draining
	Force         bool `protobuf:"varint,1,opt,name=force,proto3" json:"force,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ShutdownRequest) Reset() {
	*x = ShutdownRequest{}
	mi := &file_machine_machine_proto_msgTypes[25]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ShutdownRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ShutdownRequest) ProtoMessage() {}

func (x *ShutdownRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[25]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ShutdownRequest.ProtoReflect.Descriptor instead.
func (*ShutdownRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{25}
}

func (x *ShutdownRequest) GetForce() bool {
	if x != nil {
		return x.Force
	}
	return false
}

type ShutdownResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Shutdown            `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ShutdownResponse) Reset() {
	*x = ShutdownResponse{}
	mi := &file_machine_machine_proto_msgTypes[26]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ShutdownResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ShutdownResponse) ProtoMessage() {}

func (x *ShutdownResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[26]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ShutdownResponse.ProtoReflect.Descriptor instead.
func (*ShutdownResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{26}
}

func (x *ShutdownResponse) GetMessages() []*Shutdown {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc upgrade
type UpgradeRequest struct {
	state         protoimpl.MessageState    `protogen:"open.v1"`
	Image         string                    `protobuf:"bytes,1,opt,name=image,proto3" json:"image,omitempty"`
	Preserve      bool                      `protobuf:"varint,2,opt,name=preserve,proto3" json:"preserve,omitempty"`
	Stage         bool                      `protobuf:"varint,3,opt,name=stage,proto3" json:"stage,omitempty"`
	Force         bool                      `protobuf:"varint,4,opt,name=force,proto3" json:"force,omitempty"`
	RebootMode    UpgradeRequest_RebootMode `protobuf:"varint,5,opt,name=reboot_mode,json=rebootMode,proto3,enum=machine.UpgradeRequest_RebootMode" json:"reboot_mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpgradeRequest) Reset() {
	*x = UpgradeRequest{}
	mi := &file_machine_machine_proto_msgTypes[27]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpgradeRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpgradeRequest) ProtoMessage() {}

func (x *UpgradeRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[27]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpgradeRequest.ProtoReflect.Descriptor instead.
func (*UpgradeRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{27}
}

func (x *UpgradeRequest) GetImage() string {
	if x != nil {
		return x.Image
	}
	return ""
}

func (x *UpgradeRequest) GetPreserve() bool {
	if x != nil {
		return x.Preserve
	}
	return false
}

func (x *UpgradeRequest) GetStage() bool {
	if x != nil {
		return x.Stage
	}
	return false
}

func (x *UpgradeRequest) GetForce() bool {
	if x != nil {
		return x.Force
	}
	return false
}

func (x *UpgradeRequest) GetRebootMode() UpgradeRequest_RebootMode {
	if x != nil {
		return x.RebootMode
	}
	return UpgradeRequest_DEFAULT
}

type Upgrade struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Ack           string                 `protobuf:"bytes,2,opt,name=ack,proto3" json:"ack,omitempty"`
	ActorId       string                 `protobuf:"bytes,3,opt,name=actor_id,json=actorId,proto3" json:"actor_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Upgrade) Reset() {
	*x = Upgrade{}
	mi := &file_machine_machine_proto_msgTypes[28]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Upgrade) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Upgrade) ProtoMessage() {}

func (x *Upgrade) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[28]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Upgrade.ProtoReflect.Descriptor instead.
func (*Upgrade) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{28}
}

func (x *Upgrade) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Upgrade) GetAck() string {
	if x != nil {
		return x.Ack
	}
	return ""
}

func (x *Upgrade) GetActorId() string {
	if x != nil {
		return x.ActorId
	}
	return ""
}

type UpgradeResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Upgrade             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpgradeResponse) Reset() {
	*x = UpgradeResponse{}
	mi := &file_machine_machine_proto_msgTypes[29]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpgradeResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpgradeResponse) ProtoMessage() {}

func (x *UpgradeResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[29]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpgradeResponse.ProtoReflect.Descriptor instead.
func (*UpgradeResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{29}
}

func (x *UpgradeResponse) GetMessages() []*Upgrade {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc servicelist
type ServiceList struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Services      []*ServiceInfo         `protobuf:"bytes,2,rep,name=services,proto3" json:"services,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceList) Reset() {
	*x = ServiceList{}
	mi := &file_machine_machine_proto_msgTypes[30]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceList) ProtoMessage() {}

func (x *ServiceList) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[30]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceList.ProtoReflect.Descriptor instead.
func (*ServiceList) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{30}
}

func (x *ServiceList) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ServiceList) GetServices() []*ServiceInfo {
	if x != nil {
		return x.Services
	}
	return nil
}

type ServiceListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ServiceList         `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceListResponse) Reset() {
	*x = ServiceListResponse{}
	mi := &file_machine_machine_proto_msgTypes[31]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceListResponse) ProtoMessage() {}

func (x *ServiceListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[31]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceListResponse.ProtoReflect.Descriptor instead.
func (*ServiceListResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{31}
}

func (x *ServiceListResponse) GetMessages() []*ServiceList {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ServiceInfo struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	State         string                 `protobuf:"bytes,2,opt,name=state,proto3" json:"state,omitempty"`
	Events        *ServiceEvents         `protobuf:"bytes,3,opt,name=events,proto3" json:"events,omitempty"`
	Health        *ServiceHealth         `protobuf:"bytes,4,opt,name=health,proto3" json:"health,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceInfo) Reset() {
	*x = ServiceInfo{}
	mi := &file_machine_machine_proto_msgTypes[32]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceInfo) ProtoMessage() {}

func (x *ServiceInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[32]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceInfo.ProtoReflect.Descriptor instead.
func (*ServiceInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{32}
}

func (x *ServiceInfo) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *ServiceInfo) GetState() string {
	if x != nil {
		return x.State
	}
	return ""
}

func (x *ServiceInfo) GetEvents() *ServiceEvents {
	if x != nil {
		return x.Events
	}
	return nil
}

func (x *ServiceInfo) GetHealth() *ServiceHealth {
	if x != nil {
		return x.Health
	}
	return nil
}

type ServiceEvents struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Events        []*ServiceEvent        `protobuf:"bytes,1,rep,name=events,proto3" json:"events,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceEvents) Reset() {
	*x = ServiceEvents{}
	mi := &file_machine_machine_proto_msgTypes[33]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceEvents) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceEvents) ProtoMessage() {}

func (x *ServiceEvents) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[33]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceEvents.ProtoReflect.Descriptor instead.
func (*ServiceEvents) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{33}
}

func (x *ServiceEvents) GetEvents() []*ServiceEvent {
	if x != nil {
		return x.Events
	}
	return nil
}

type ServiceEvent struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Msg           string                 `protobuf:"bytes,1,opt,name=msg,proto3" json:"msg,omitempty"`
	State         string                 `protobuf:"bytes,2,opt,name=state,proto3" json:"state,omitempty"`
	Ts            *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=ts,proto3" json:"ts,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceEvent) Reset() {
	*x = ServiceEvent{}
	mi := &file_machine_machine_proto_msgTypes[34]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceEvent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceEvent) ProtoMessage() {}

func (x *ServiceEvent) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[34]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceEvent.ProtoReflect.Descriptor instead.
func (*ServiceEvent) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{34}
}

func (x *ServiceEvent) GetMsg() string {
	if x != nil {
		return x.Msg
	}
	return ""
}

func (x *ServiceEvent) GetState() string {
	if x != nil {
		return x.State
	}
	return ""
}

func (x *ServiceEvent) GetTs() *timestamppb.Timestamp {
	if x != nil {
		return x.Ts
	}
	return nil
}

type ServiceHealth struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Unknown       bool                   `protobuf:"varint,1,opt,name=unknown,proto3" json:"unknown,omitempty"`
	Healthy       bool                   `protobuf:"varint,2,opt,name=healthy,proto3" json:"healthy,omitempty"`
	LastMessage   string                 `protobuf:"bytes,3,opt,name=last_message,json=lastMessage,proto3" json:"last_message,omitempty"`
	LastChange    *timestamppb.Timestamp `protobuf:"bytes,4,opt,name=last_change,json=lastChange,proto3" json:"last_change,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceHealth) Reset() {
	*x = ServiceHealth{}
	mi := &file_machine_machine_proto_msgTypes[35]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceHealth) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceHealth) ProtoMessage() {}

func (x *ServiceHealth) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[35]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceHealth.ProtoReflect.Descriptor instead.
func (*ServiceHealth) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{35}
}

func (x *ServiceHealth) GetUnknown() bool {
	if x != nil {
		return x.Unknown
	}
	return false
}

func (x *ServiceHealth) GetHealthy() bool {
	if x != nil {
		return x.Healthy
	}
	return false
}

func (x *ServiceHealth) GetLastMessage() string {
	if x != nil {
		return x.LastMessage
	}
	return ""
}

func (x *ServiceHealth) GetLastChange() *timestamppb.Timestamp {
	if x != nil {
		return x.LastChange
	}
	return nil
}

// rpc servicestart
type ServiceStartRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStartRequest) Reset() {
	*x = ServiceStartRequest{}
	mi := &file_machine_machine_proto_msgTypes[36]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStartRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStartRequest) ProtoMessage() {}

func (x *ServiceStartRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[36]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStartRequest.ProtoReflect.Descriptor instead.
func (*ServiceStartRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{36}
}

func (x *ServiceStartRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

type ServiceStart struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Resp          string                 `protobuf:"bytes,2,opt,name=resp,proto3" json:"resp,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStart) Reset() {
	*x = ServiceStart{}
	mi := &file_machine_machine_proto_msgTypes[37]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStart) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStart) ProtoMessage() {}

func (x *ServiceStart) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[37]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStart.ProtoReflect.Descriptor instead.
func (*ServiceStart) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{37}
}

func (x *ServiceStart) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ServiceStart) GetResp() string {
	if x != nil {
		return x.Resp
	}
	return ""
}

type ServiceStartResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ServiceStart        `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStartResponse) Reset() {
	*x = ServiceStartResponse{}
	mi := &file_machine_machine_proto_msgTypes[38]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStartResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStartResponse) ProtoMessage() {}

func (x *ServiceStartResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[38]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStartResponse.ProtoReflect.Descriptor instead.
func (*ServiceStartResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{38}
}

func (x *ServiceStartResponse) GetMessages() []*ServiceStart {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ServiceStopRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStopRequest) Reset() {
	*x = ServiceStopRequest{}
	mi := &file_machine_machine_proto_msgTypes[39]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStopRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStopRequest) ProtoMessage() {}

func (x *ServiceStopRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[39]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStopRequest.ProtoReflect.Descriptor instead.
func (*ServiceStopRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{39}
}

func (x *ServiceStopRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

type ServiceStop struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Resp          string                 `protobuf:"bytes,2,opt,name=resp,proto3" json:"resp,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStop) Reset() {
	*x = ServiceStop{}
	mi := &file_machine_machine_proto_msgTypes[40]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStop) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStop) ProtoMessage() {}

func (x *ServiceStop) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[40]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStop.ProtoReflect.Descriptor instead.
func (*ServiceStop) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{40}
}

func (x *ServiceStop) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ServiceStop) GetResp() string {
	if x != nil {
		return x.Resp
	}
	return ""
}

type ServiceStopResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ServiceStop         `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceStopResponse) Reset() {
	*x = ServiceStopResponse{}
	mi := &file_machine_machine_proto_msgTypes[41]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceStopResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceStopResponse) ProtoMessage() {}

func (x *ServiceStopResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[41]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceStopResponse.ProtoReflect.Descriptor instead.
func (*ServiceStopResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{41}
}

func (x *ServiceStopResponse) GetMessages() []*ServiceStop {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ServiceRestartRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceRestartRequest) Reset() {
	*x = ServiceRestartRequest{}
	mi := &file_machine_machine_proto_msgTypes[42]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceRestartRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceRestartRequest) ProtoMessage() {}

func (x *ServiceRestartRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[42]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceRestartRequest.ProtoReflect.Descriptor instead.
func (*ServiceRestartRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{42}
}

func (x *ServiceRestartRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

type ServiceRestart struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Resp          string                 `protobuf:"bytes,2,opt,name=resp,proto3" json:"resp,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceRestart) Reset() {
	*x = ServiceRestart{}
	mi := &file_machine_machine_proto_msgTypes[43]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceRestart) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceRestart) ProtoMessage() {}

func (x *ServiceRestart) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[43]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceRestart.ProtoReflect.Descriptor instead.
func (*ServiceRestart) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{43}
}

func (x *ServiceRestart) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ServiceRestart) GetResp() string {
	if x != nil {
		return x.Resp
	}
	return ""
}

type ServiceRestartResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ServiceRestart      `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceRestartResponse) Reset() {
	*x = ServiceRestartResponse{}
	mi := &file_machine_machine_proto_msgTypes[44]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceRestartResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceRestartResponse) ProtoMessage() {}

func (x *ServiceRestartResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[44]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceRestartResponse.ProtoReflect.Descriptor instead.
func (*ServiceRestartResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{44}
}

func (x *ServiceRestartResponse) GetMessages() []*ServiceRestart {
	if x != nil {
		return x.Messages
	}
	return nil
}

// CopyRequest describes a request to copy data out of Talos node
//
// Copy produces .tar.gz archive which is streamed back to the caller
type CopyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Root path to start copying data out, it might be either a file or directory
	RootPath      string `protobuf:"bytes,1,opt,name=root_path,json=rootPath,proto3" json:"root_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CopyRequest) Reset() {
	*x = CopyRequest{}
	mi := &file_machine_machine_proto_msgTypes[45]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CopyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CopyRequest) ProtoMessage() {}

func (x *CopyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[45]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CopyRequest.ProtoReflect.Descriptor instead.
func (*CopyRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{45}
}

func (x *CopyRequest) GetRootPath() string {
	if x != nil {
		return x.RootPath
	}
	return ""
}

// ListRequest describes a request to list the contents of a directory.
type ListRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Root indicates the root directory for the list. If not indicated, '/' is
	// presumed.
	Root string `protobuf:"bytes,1,opt,name=root,proto3" json:"root,omitempty"`
	// Recurse indicates that subdirectories should be recursed.
	Recurse bool `protobuf:"varint,2,opt,name=recurse,proto3" json:"recurse,omitempty"`
	// RecursionDepth indicates how many levels of subdirectories should be
	// recursed. The default (0) indicates that no limit should be enforced.
	RecursionDepth int32 `protobuf:"varint,3,opt,name=recursion_depth,json=recursionDepth,proto3" json:"recursion_depth,omitempty"`
	// Types indicates what file type should be returned. If not indicated,
	// all files will be returned.
	Types []ListRequest_Type `protobuf:"varint,4,rep,packed,name=types,proto3,enum=machine.ListRequest_Type" json:"types,omitempty"`
	// Report xattrs
	ReportXattrs  bool `protobuf:"varint,5,opt,name=report_xattrs,json=reportXattrs,proto3" json:"report_xattrs,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListRequest) Reset() {
	*x = ListRequest{}
	mi := &file_machine_machine_proto_msgTypes[46]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListRequest) ProtoMessage() {}

func (x *ListRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[46]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListRequest.ProtoReflect.Descriptor instead.
func (*ListRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{46}
}

func (x *ListRequest) GetRoot() string {
	if x != nil {
		return x.Root
	}
	return ""
}

func (x *ListRequest) GetRecurse() bool {
	if x != nil {
		return x.Recurse
	}
	return false
}

func (x *ListRequest) GetRecursionDepth() int32 {
	if x != nil {
		return x.RecursionDepth
	}
	return 0
}

func (x *ListRequest) GetTypes() []ListRequest_Type {
	if x != nil {
		return x.Types
	}
	return nil
}

func (x *ListRequest) GetReportXattrs() bool {
	if x != nil {
		return x.ReportXattrs
	}
	return false
}

// DiskUsageRequest describes a request to list disk usage of directories and regular files
type DiskUsageRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// RecursionDepth indicates how many levels of subdirectories should be
	// recursed. The default (0) indicates that no limit should be enforced.
	RecursionDepth int32 `protobuf:"varint,1,opt,name=recursion_depth,json=recursionDepth,proto3" json:"recursion_depth,omitempty"`
	// All write sizes for all files, not just directories.
	All bool `protobuf:"varint,2,opt,name=all,proto3" json:"all,omitempty"`
	// Threshold exclude entries smaller than SIZE if positive,
	// or entries greater than SIZE if negative.
	Threshold int64 `protobuf:"varint,3,opt,name=threshold,proto3" json:"threshold,omitempty"`
	// DiskUsagePaths is the list of directories to calculate disk usage for.
	Paths         []string `protobuf:"bytes,4,rep,name=paths,proto3" json:"paths,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DiskUsageRequest) Reset() {
	*x = DiskUsageRequest{}
	mi := &file_machine_machine_proto_msgTypes[47]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiskUsageRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiskUsageRequest) ProtoMessage() {}

func (x *DiskUsageRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[47]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiskUsageRequest.ProtoReflect.Descriptor instead.
func (*DiskUsageRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{47}
}

func (x *DiskUsageRequest) GetRecursionDepth() int32 {
	if x != nil {
		return x.RecursionDepth
	}
	return 0
}

func (x *DiskUsageRequest) GetAll() bool {
	if x != nil {
		return x.All
	}
	return false
}

func (x *DiskUsageRequest) GetThreshold() int64 {
	if x != nil {
		return x.Threshold
	}
	return 0
}

func (x *DiskUsageRequest) GetPaths() []string {
	if x != nil {
		return x.Paths
	}
	return nil
}

// FileInfo describes a file or directory's information
type FileInfo struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// Name is the name (including prefixed path) of the file or directory
	Name string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	// Size indicates the number of bytes contained within the file
	Size int64 `protobuf:"varint,3,opt,name=size,proto3" json:"size,omitempty"`
	// Mode is the bitmap of UNIX mode/permission flags of the file
	Mode uint32 `protobuf:"varint,4,opt,name=mode,proto3" json:"mode,omitempty"`
	// Modified indicates the UNIX timestamp at which the file was last modified
	Modified int64 `protobuf:"varint,5,opt,name=modified,proto3" json:"modified,omitempty"`
	// IsDir indicates that the file is a directory
	IsDir bool `protobuf:"varint,6,opt,name=is_dir,json=isDir,proto3" json:"is_dir,omitempty"`
	// Error describes any error encountered while trying to read the file
	// information.
	Error string `protobuf:"bytes,7,opt,name=error,proto3" json:"error,omitempty"`
	// Link is filled with symlink target
	Link string `protobuf:"bytes,8,opt,name=link,proto3" json:"link,omitempty"`
	// RelativeName is the name of the file or directory relative to the RootPath
	RelativeName string `protobuf:"bytes,9,opt,name=relative_name,json=relativeName,proto3" json:"relative_name,omitempty"`
	// Owner uid
	Uid uint32 `protobuf:"varint,10,opt,name=uid,proto3" json:"uid,omitempty"`
	// Owner gid
	Gid uint32 `protobuf:"varint,11,opt,name=gid,proto3" json:"gid,omitempty"`
	// Extended attributes (if present and requested)
	Xattrs        []*Xattr `protobuf:"bytes,12,rep,name=xattrs,proto3" json:"xattrs,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileInfo) Reset() {
	*x = FileInfo{}
	mi := &file_machine_machine_proto_msgTypes[48]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileInfo) ProtoMessage() {}

func (x *FileInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[48]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileInfo.ProtoReflect.Descriptor instead.
func (*FileInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{48}
}

func (x *FileInfo) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *FileInfo) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *FileInfo) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *FileInfo) GetMode() uint32 {
	if x != nil {
		return x.Mode
	}
	return 0
}

func (x *FileInfo) GetModified() int64 {
	if x != nil {
		return x.Modified
	}
	return 0
}

func (x *FileInfo) GetIsDir() bool {
	if x != nil {
		return x.IsDir
	}
	return false
}

func (x *FileInfo) GetError() string {
	if x != nil {
		return x.Error
	}
	return ""
}

func (x *FileInfo) GetLink() string {
	if x != nil {
		return x.Link
	}
	return ""
}

func (x *FileInfo) GetRelativeName() string {
	if x != nil {
		return x.RelativeName
	}
	return ""
}

func (x *FileInfo) GetUid() uint32 {
	if x != nil {
		return x.Uid
	}
	return 0
}

func (x *FileInfo) GetGid() uint32 {
	if x != nil {
		return x.Gid
	}
	return 0
}

func (x *FileInfo) GetXattrs() []*Xattr {
	if x != nil {
		return x.Xattrs
	}
	return nil
}

type Xattr struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Data          []byte                 `protobuf:"bytes,2,opt,name=data,proto3" json:"data,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Xattr) Reset() {
	*x = Xattr{}
	mi := &file_machine_machine_proto_msgTypes[49]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Xattr) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Xattr) ProtoMessage() {}

func (x *Xattr) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[49]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Xattr.ProtoReflect.Descriptor instead.
func (*Xattr) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{49}
}

func (x *Xattr) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Xattr) GetData() []byte {
	if x != nil {
		return x.Data
	}
	return nil
}

// DiskUsageInfo describes a file or directory's information for du command
type DiskUsageInfo struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// Name is the name (including prefixed path) of the file or directory
	Name string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	// Size indicates the number of bytes contained within the file
	Size int64 `protobuf:"varint,3,opt,name=size,proto3" json:"size,omitempty"`
	// Error describes any error encountered while trying to read the file
	// information.
	Error string `protobuf:"bytes,4,opt,name=error,proto3" json:"error,omitempty"`
	// RelativeName is the name of the file or directory relative to the RootPath
	RelativeName  string `protobuf:"bytes,5,opt,name=relative_name,json=relativeName,proto3" json:"relative_name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DiskUsageInfo) Reset() {
	*x = DiskUsageInfo{}
	mi := &file_machine_machine_proto_msgTypes[50]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiskUsageInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiskUsageInfo) ProtoMessage() {}

func (x *DiskUsageInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[50]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiskUsageInfo.ProtoReflect.Descriptor instead.
func (*DiskUsageInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{50}
}

func (x *DiskUsageInfo) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *DiskUsageInfo) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *DiskUsageInfo) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *DiskUsageInfo) GetError() string {
	if x != nil {
		return x.Error
	}
	return ""
}

func (x *DiskUsageInfo) GetRelativeName() string {
	if x != nil {
		return x.RelativeName
	}
	return ""
}

// The messages message containing the requested df stats.
type Mounts struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Stats         []*MountStat           `protobuf:"bytes,2,rep,name=stats,proto3" json:"stats,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Mounts) Reset() {
	*x = Mounts{}
	mi := &file_machine_machine_proto_msgTypes[51]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Mounts) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Mounts) ProtoMessage() {}

func (x *Mounts) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[51]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Mounts.ProtoReflect.Descriptor instead.
func (*Mounts) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{51}
}

func (x *Mounts) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Mounts) GetStats() []*MountStat {
	if x != nil {
		return x.Stats
	}
	return nil
}

type MountsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Mounts              `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MountsResponse) Reset() {
	*x = MountsResponse{}
	mi := &file_machine_machine_proto_msgTypes[52]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MountsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MountsResponse) ProtoMessage() {}

func (x *MountsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[52]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MountsResponse.ProtoReflect.Descriptor instead.
func (*MountsResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{52}
}

func (x *MountsResponse) GetMessages() []*Mounts {
	if x != nil {
		return x.Messages
	}
	return nil
}

// The messages message containing the requested processes.
type MountStat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Filesystem    string                 `protobuf:"bytes,1,opt,name=filesystem,proto3" json:"filesystem,omitempty"`
	Size          uint64                 `protobuf:"varint,2,opt,name=size,proto3" json:"size,omitempty"`
	Available     uint64                 `protobuf:"varint,3,opt,name=available,proto3" json:"available,omitempty"`
	MountedOn     string                 `protobuf:"bytes,4,opt,name=mounted_on,json=mountedOn,proto3" json:"mounted_on,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MountStat) Reset() {
	*x = MountStat{}
	mi := &file_machine_machine_proto_msgTypes[53]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MountStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MountStat) ProtoMessage() {}

func (x *MountStat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[53]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MountStat.ProtoReflect.Descriptor instead.
func (*MountStat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{53}
}

func (x *MountStat) GetFilesystem() string {
	if x != nil {
		return x.Filesystem
	}
	return ""
}

func (x *MountStat) GetSize() uint64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *MountStat) GetAvailable() uint64 {
	if x != nil {
		return x.Available
	}
	return 0
}

func (x *MountStat) GetMountedOn() string {
	if x != nil {
		return x.MountedOn
	}
	return ""
}

type Version struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Version  *VersionInfo           `protobuf:"bytes,2,opt,name=version,proto3" json:"version,omitempty"`
	Platform *PlatformInfo          `protobuf:"bytes,3,opt,name=platform,proto3" json:"platform,omitempty"`
	// Features describe individual Talos features that can be switched on or off.
	Features      *FeaturesInfo `protobuf:"bytes,4,opt,name=features,proto3" json:"features,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Version) Reset() {
	*x = Version{}
	mi := &file_machine_machine_proto_msgTypes[54]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Version) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Version) ProtoMessage() {}

func (x *Version) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[54]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Version.ProtoReflect.Descriptor instead.
func (*Version) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{54}
}

func (x *Version) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Version) GetVersion() *VersionInfo {
	if x != nil {
		return x.Version
	}
	return nil
}

func (x *Version) GetPlatform() *PlatformInfo {
	if x != nil {
		return x.Platform
	}
	return nil
}

func (x *Version) GetFeatures() *FeaturesInfo {
	if x != nil {
		return x.Features
	}
	return nil
}

type VersionResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Version             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VersionResponse) Reset() {
	*x = VersionResponse{}
	mi := &file_machine_machine_proto_msgTypes[55]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VersionResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VersionResponse) ProtoMessage() {}

func (x *VersionResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[55]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VersionResponse.ProtoReflect.Descriptor instead.
func (*VersionResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{55}
}

func (x *VersionResponse) GetMessages() []*Version {
	if x != nil {
		return x.Messages
	}
	return nil
}

type VersionInfo struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Tag           string                 `protobuf:"bytes,1,opt,name=tag,proto3" json:"tag,omitempty"`
	Sha           string                 `protobuf:"bytes,2,opt,name=sha,proto3" json:"sha,omitempty"`
	Built         string                 `protobuf:"bytes,3,opt,name=built,proto3" json:"built,omitempty"`
	GoVersion     string                 `protobuf:"bytes,4,opt,name=go_version,json=goVersion,proto3" json:"go_version,omitempty"`
	Os            string                 `protobuf:"bytes,5,opt,name=os,proto3" json:"os,omitempty"`
	Arch          string                 `protobuf:"bytes,6,opt,name=arch,proto3" json:"arch,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VersionInfo) Reset() {
	*x = VersionInfo{}
	mi := &file_machine_machine_proto_msgTypes[56]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VersionInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VersionInfo) ProtoMessage() {}

func (x *VersionInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[56]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VersionInfo.ProtoReflect.Descriptor instead.
func (*VersionInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{56}
}

func (x *VersionInfo) GetTag() string {
	if x != nil {
		return x.Tag
	}
	return ""
}

func (x *VersionInfo) GetSha() string {
	if x != nil {
		return x.Sha
	}
	return ""
}

func (x *VersionInfo) GetBuilt() string {
	if x != nil {
		return x.Built
	}
	return ""
}

func (x *VersionInfo) GetGoVersion() string {
	if x != nil {
		return x.GoVersion
	}
	return ""
}

func (x *VersionInfo) GetOs() string {
	if x != nil {
		return x.Os
	}
	return ""
}

func (x *VersionInfo) GetArch() string {
	if x != nil {
		return x.Arch
	}
	return ""
}

type PlatformInfo struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Mode          string                 `protobuf:"bytes,2,opt,name=mode,proto3" json:"mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PlatformInfo) Reset() {
	*x = PlatformInfo{}
	mi := &file_machine_machine_proto_msgTypes[57]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PlatformInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PlatformInfo) ProtoMessage() {}

func (x *PlatformInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[57]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PlatformInfo.ProtoReflect.Descriptor instead.
func (*PlatformInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{57}
}

func (x *PlatformInfo) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *PlatformInfo) GetMode() string {
	if x != nil {
		return x.Mode
	}
	return ""
}

// FeaturesInfo describes individual Talos features that can be switched on or off.
type FeaturesInfo struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// RBAC is true if role-based access control is enabled.
	Rbac          bool `protobuf:"varint,1,opt,name=rbac,proto3" json:"rbac,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FeaturesInfo) Reset() {
	*x = FeaturesInfo{}
	mi := &file_machine_machine_proto_msgTypes[58]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FeaturesInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FeaturesInfo) ProtoMessage() {}

func (x *FeaturesInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[58]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FeaturesInfo.ProtoReflect.Descriptor instead.
func (*FeaturesInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{58}
}

func (x *FeaturesInfo) GetRbac() bool {
	if x != nil {
		return x.Rbac
	}
	return false
}

// rpc logs
// The request message containing the process name.
type LogsRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	Namespace string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Id        string                 `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	// driver might be default "containerd" or "cri"
	Driver        common.ContainerDriver `protobuf:"varint,3,opt,name=driver,proto3,enum=common.ContainerDriver" json:"driver,omitempty"`
	Follow        bool                   `protobuf:"varint,4,opt,name=follow,proto3" json:"follow,omitempty"`
	TailLines     int32                  `protobuf:"varint,5,opt,name=tail_lines,json=tailLines,proto3" json:"tail_lines,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LogsRequest) Reset() {
	*x = LogsRequest{}
	mi := &file_machine_machine_proto_msgTypes[59]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LogsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LogsRequest) ProtoMessage() {}

func (x *LogsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[59]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LogsRequest.ProtoReflect.Descriptor instead.
func (*LogsRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{59}
}

func (x *LogsRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *LogsRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *LogsRequest) GetDriver() common.ContainerDriver {
	if x != nil {
		return x.Driver
	}
	return common.ContainerDriver(0)
}

func (x *LogsRequest) GetFollow() bool {
	if x != nil {
		return x.Follow
	}
	return false
}

func (x *LogsRequest) GetTailLines() int32 {
	if x != nil {
		return x.TailLines
	}
	return 0
}

type ReadRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Path          string                 `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ReadRequest) Reset() {
	*x = ReadRequest{}
	mi := &file_machine_machine_proto_msgTypes[60]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ReadRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ReadRequest) ProtoMessage() {}

func (x *ReadRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[60]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ReadRequest.ProtoReflect.Descriptor instead.
func (*ReadRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{60}
}

func (x *ReadRequest) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

// LogsContainer describes all available registered log containers.
type LogsContainer struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Ids           []string               `protobuf:"bytes,2,rep,name=ids,proto3" json:"ids,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LogsContainer) Reset() {
	*x = LogsContainer{}
	mi := &file_machine_machine_proto_msgTypes[61]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LogsContainer) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LogsContainer) ProtoMessage() {}

func (x *LogsContainer) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[61]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LogsContainer.ProtoReflect.Descriptor instead.
func (*LogsContainer) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{61}
}

func (x *LogsContainer) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *LogsContainer) GetIds() []string {
	if x != nil {
		return x.Ids
	}
	return nil
}

type LogsContainersResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*LogsContainer       `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LogsContainersResponse) Reset() {
	*x = LogsContainersResponse{}
	mi := &file_machine_machine_proto_msgTypes[62]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LogsContainersResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LogsContainersResponse) ProtoMessage() {}

func (x *LogsContainersResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[62]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LogsContainersResponse.ProtoReflect.Descriptor instead.
func (*LogsContainersResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{62}
}

func (x *LogsContainersResponse) GetMessages() []*LogsContainer {
	if x != nil {
		return x.Messages
	}
	return nil
}

// rpc rollback
type RollbackRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RollbackRequest) Reset() {
	*x = RollbackRequest{}
	mi := &file_machine_machine_proto_msgTypes[63]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RollbackRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RollbackRequest) ProtoMessage() {}

func (x *RollbackRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[63]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RollbackRequest.ProtoReflect.Descriptor instead.
func (*RollbackRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{63}
}

type Rollback struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Rollback) Reset() {
	*x = Rollback{}
	mi := &file_machine_machine_proto_msgTypes[64]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Rollback) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Rollback) ProtoMessage() {}

func (x *Rollback) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[64]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Rollback.ProtoReflect.Descriptor instead.
func (*Rollback) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{64}
}

func (x *Rollback) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type RollbackResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Rollback            `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RollbackResponse) Reset() {
	*x = RollbackResponse{}
	mi := &file_machine_machine_proto_msgTypes[65]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RollbackResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RollbackResponse) ProtoMessage() {}

func (x *RollbackResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[65]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RollbackResponse.ProtoReflect.Descriptor instead.
func (*RollbackResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{65}
}

func (x *RollbackResponse) GetMessages() []*Rollback {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ContainersRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	Namespace string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	// driver might be default "containerd" or "cri"
	Driver        common.ContainerDriver `protobuf:"varint,2,opt,name=driver,proto3,enum=common.ContainerDriver" json:"driver,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ContainersRequest) Reset() {
	*x = ContainersRequest{}
	mi := &file_machine_machine_proto_msgTypes[66]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ContainersRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ContainersRequest) ProtoMessage() {}

func (x *ContainersRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[66]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ContainersRequest.ProtoReflect.Descriptor instead.
func (*ContainersRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{66}
}

func (x *ContainersRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *ContainersRequest) GetDriver() common.ContainerDriver {
	if x != nil {
		return x.Driver
	}
	return common.ContainerDriver(0)
}

// The messages message containing the requested containers.
type ContainerInfo struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	Namespace        string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Id               string                 `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	Uid              string                 `protobuf:"bytes,10,opt,name=uid,proto3" json:"uid,omitempty"`
	InternalId       string                 `protobuf:"bytes,9,opt,name=internal_id,json=internalId,proto3" json:"internal_id,omitempty"`
	Image            string                 `protobuf:"bytes,3,opt,name=image,proto3" json:"image,omitempty"`
	Pid              uint32                 `protobuf:"varint,4,opt,name=pid,proto3" json:"pid,omitempty"`
	Status           string                 `protobuf:"bytes,5,opt,name=status,proto3" json:"status,omitempty"`
	PodId            string                 `protobuf:"bytes,6,opt,name=pod_id,json=podId,proto3" json:"pod_id,omitempty"`
	Name             string                 `protobuf:"bytes,7,opt,name=name,proto3" json:"name,omitempty"`
	NetworkNamespace string                 `protobuf:"bytes,8,opt,name=network_namespace,json=networkNamespace,proto3" json:"network_namespace,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *ContainerInfo) Reset() {
	*x = ContainerInfo{}
	mi := &file_machine_machine_proto_msgTypes[67]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ContainerInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ContainerInfo) ProtoMessage() {}

func (x *ContainerInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[67]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ContainerInfo.ProtoReflect.Descriptor instead.
func (*ContainerInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{67}
}

func (x *ContainerInfo) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *ContainerInfo) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *ContainerInfo) GetUid() string {
	if x != nil {
		return x.Uid
	}
	return ""
}

func (x *ContainerInfo) GetInternalId() string {
	if x != nil {
		return x.InternalId
	}
	return ""
}

func (x *ContainerInfo) GetImage() string {
	if x != nil {
		return x.Image
	}
	return ""
}

func (x *ContainerInfo) GetPid() uint32 {
	if x != nil {
		return x.Pid
	}
	return 0
}

func (x *ContainerInfo) GetStatus() string {
	if x != nil {
		return x.Status
	}
	return ""
}

func (x *ContainerInfo) GetPodId() string {
	if x != nil {
		return x.PodId
	}
	return ""
}

func (x *ContainerInfo) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ContainerInfo) GetNetworkNamespace() string {
	if x != nil {
		return x.NetworkNamespace
	}
	return ""
}

// The messages message containing the requested containers.
type Container struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Containers    []*ContainerInfo       `protobuf:"bytes,2,rep,name=containers,proto3" json:"containers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Container) Reset() {
	*x = Container{}
	mi := &file_machine_machine_proto_msgTypes[68]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Container) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Container) ProtoMessage() {}

func (x *Container) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[68]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Container.ProtoReflect.Descriptor instead.
func (*Container) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{68}
}

func (x *Container) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Container) GetContainers() []*ContainerInfo {
	if x != nil {
		return x.Containers
	}
	return nil
}

type ContainersResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Container           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ContainersResponse) Reset() {
	*x = ContainersResponse{}
	mi := &file_machine_machine_proto_msgTypes[69]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ContainersResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ContainersResponse) ProtoMessage() {}

func (x *ContainersResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[69]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ContainersResponse.ProtoReflect.Descriptor instead.
func (*ContainersResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{69}
}

func (x *ContainersResponse) GetMessages() []*Container {
	if x != nil {
		return x.Messages
	}
	return nil
}

// dmesg
type DmesgRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Follow        bool                   `protobuf:"varint,1,opt,name=follow,proto3" json:"follow,omitempty"`
	Tail          bool                   `protobuf:"varint,2,opt,name=tail,proto3" json:"tail,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DmesgRequest) Reset() {
	*x = DmesgRequest{}
	mi := &file_machine_machine_proto_msgTypes[70]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DmesgRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DmesgRequest) ProtoMessage() {}

func (x *DmesgRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[70]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DmesgRequest.ProtoReflect.Descriptor instead.
func (*DmesgRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{70}
}

func (x *DmesgRequest) GetFollow() bool {
	if x != nil {
		return x.Follow
	}
	return false
}

func (x *DmesgRequest) GetTail() bool {
	if x != nil {
		return x.Tail
	}
	return false
}

// rpc processes
type ProcessesResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Process             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ProcessesResponse) Reset() {
	*x = ProcessesResponse{}
	mi := &file_machine_machine_proto_msgTypes[71]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProcessesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProcessesResponse) ProtoMessage() {}

func (x *ProcessesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[71]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProcessesResponse.ProtoReflect.Descriptor instead.
func (*ProcessesResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{71}
}

func (x *ProcessesResponse) GetMessages() []*Process {
	if x != nil {
		return x.Messages
	}
	return nil
}

type Process struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Processes     []*ProcessInfo         `protobuf:"bytes,2,rep,name=processes,proto3" json:"processes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Process) Reset() {
	*x = Process{}
	mi := &file_machine_machine_proto_msgTypes[72]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Process) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Process) ProtoMessage() {}

func (x *Process) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[72]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Process.ProtoReflect.Descriptor instead.
func (*Process) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{72}
}

func (x *Process) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Process) GetProcesses() []*ProcessInfo {
	if x != nil {
		return x.Processes
	}
	return nil
}

type ProcessInfo struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	Pid            int32                  `protobuf:"varint,1,opt,name=pid,proto3" json:"pid,omitempty"`
	Ppid           int32                  `protobuf:"varint,2,opt,name=ppid,proto3" json:"ppid,omitempty"`
	State          string                 `protobuf:"bytes,3,opt,name=state,proto3" json:"state,omitempty"`
	Threads        int32                  `protobuf:"varint,4,opt,name=threads,proto3" json:"threads,omitempty"`
	CpuTime        float64                `protobuf:"fixed64,5,opt,name=cpu_time,json=cpuTime,proto3" json:"cpu_time,omitempty"`
	VirtualMemory  uint64                 `protobuf:"varint,6,opt,name=virtual_memory,json=virtualMemory,proto3" json:"virtual_memory,omitempty"`
	ResidentMemory uint64                 `protobuf:"varint,7,opt,name=resident_memory,json=residentMemory,proto3" json:"resident_memory,omitempty"`
	Command        string                 `protobuf:"bytes,8,opt,name=command,proto3" json:"command,omitempty"`
	Executable     string                 `protobuf:"bytes,9,opt,name=executable,proto3" json:"executable,omitempty"`
	Args           string                 `protobuf:"bytes,10,opt,name=args,proto3" json:"args,omitempty"`
	Label          string                 `protobuf:"bytes,11,opt,name=label,proto3" json:"label,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ProcessInfo) Reset() {
	*x = ProcessInfo{}
	mi := &file_machine_machine_proto_msgTypes[73]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProcessInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProcessInfo) ProtoMessage() {}

func (x *ProcessInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[73]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProcessInfo.ProtoReflect.Descriptor instead.
func (*ProcessInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{73}
}

func (x *ProcessInfo) GetPid() int32 {
	if x != nil {
		return x.Pid
	}
	return 0
}

func (x *ProcessInfo) GetPpid() int32 {
	if x != nil {
		return x.Ppid
	}
	return 0
}

func (x *ProcessInfo) GetState() string {
	if x != nil {
		return x.State
	}
	return ""
}

func (x *ProcessInfo) GetThreads() int32 {
	if x != nil {
		return x.Threads
	}
	return 0
}

func (x *ProcessInfo) GetCpuTime() float64 {
	if x != nil {
		return x.CpuTime
	}
	return 0
}

func (x *ProcessInfo) GetVirtualMemory() uint64 {
	if x != nil {
		return x.VirtualMemory
	}
	return 0
}

func (x *ProcessInfo) GetResidentMemory() uint64 {
	if x != nil {
		return x.ResidentMemory
	}
	return 0
}

func (x *ProcessInfo) GetCommand() string {
	if x != nil {
		return x.Command
	}
	return ""
}

func (x *ProcessInfo) GetExecutable() string {
	if x != nil {
		return x.Executable
	}
	return ""
}

func (x *ProcessInfo) GetArgs() string {
	if x != nil {
		return x.Args
	}
	return ""
}

func (x *ProcessInfo) GetLabel() string {
	if x != nil {
		return x.Label
	}
	return ""
}

// rpc restart
// The request message containing the process to restart.
type RestartRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	Namespace string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Id        string                 `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	// driver might be default "containerd" or "cri"
	Driver        common.ContainerDriver `protobuf:"varint,3,opt,name=driver,proto3,enum=common.ContainerDriver" json:"driver,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RestartRequest) Reset() {
	*x = RestartRequest{}
	mi := &file_machine_machine_proto_msgTypes[74]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RestartRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestartRequest) ProtoMessage() {}

func (x *RestartRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[74]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestartRequest.ProtoReflect.Descriptor instead.
func (*RestartRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{74}
}

func (x *RestartRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *RestartRequest) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *RestartRequest) GetDriver() common.ContainerDriver {
	if x != nil {
		return x.Driver
	}
	return common.ContainerDriver(0)
}

type Restart struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Restart) Reset() {
	*x = Restart{}
	mi := &file_machine_machine_proto_msgTypes[75]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Restart) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Restart) ProtoMessage() {}

func (x *Restart) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[75]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Restart.ProtoReflect.Descriptor instead.
func (*Restart) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{75}
}

func (x *Restart) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

// The messages message containing the restart status.
type RestartResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Restart             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RestartResponse) Reset() {
	*x = RestartResponse{}
	mi := &file_machine_machine_proto_msgTypes[76]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RestartResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestartResponse) ProtoMessage() {}

func (x *RestartResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[76]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestartResponse.ProtoReflect.Descriptor instead.
func (*RestartResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{76}
}

func (x *RestartResponse) GetMessages() []*Restart {
	if x != nil {
		return x.Messages
	}
	return nil
}

// The request message containing the containerd namespace.
type StatsRequest struct {
	state     protoimpl.MessageState `protogen:"open.v1"`
	Namespace string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	// driver might be default "containerd" or "cri"
	Driver        common.ContainerDriver `protobuf:"varint,2,opt,name=driver,proto3,enum=common.ContainerDriver" json:"driver,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StatsRequest) Reset() {
	*x = StatsRequest{}
	mi := &file_machine_machine_proto_msgTypes[77]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsRequest) ProtoMessage() {}

func (x *StatsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[77]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsRequest.ProtoReflect.Descriptor instead.
func (*StatsRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{77}
}

func (x *StatsRequest) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *StatsRequest) GetDriver() common.ContainerDriver {
	if x != nil {
		return x.Driver
	}
	return common.ContainerDriver(0)
}

// The messages message containing the requested stats.
type Stats struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Stats         []*Stat                `protobuf:"bytes,2,rep,name=stats,proto3" json:"stats,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Stats) Reset() {
	*x = Stats{}
	mi := &file_machine_machine_proto_msgTypes[78]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Stats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Stats) ProtoMessage() {}

func (x *Stats) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[78]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Stats.ProtoReflect.Descriptor instead.
func (*Stats) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{78}
}

func (x *Stats) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Stats) GetStats() []*Stat {
	if x != nil {
		return x.Stats
	}
	return nil
}

type StatsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Stats               `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StatsResponse) Reset() {
	*x = StatsResponse{}
	mi := &file_machine_machine_proto_msgTypes[79]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StatsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StatsResponse) ProtoMessage() {}

func (x *StatsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[79]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StatsResponse.ProtoReflect.Descriptor instead.
func (*StatsResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{79}
}

func (x *StatsResponse) GetMessages() []*Stats {
	if x != nil {
		return x.Messages
	}
	return nil
}

// The messages message containing the requested stat.
type Stat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Namespace     string                 `protobuf:"bytes,1,opt,name=namespace,proto3" json:"namespace,omitempty"`
	Id            string                 `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	MemoryUsage   uint64                 `protobuf:"varint,4,opt,name=memory_usage,json=memoryUsage,proto3" json:"memory_usage,omitempty"`
	CpuUsage      uint64                 `protobuf:"varint,5,opt,name=cpu_usage,json=cpuUsage,proto3" json:"cpu_usage,omitempty"`
	PodId         string                 `protobuf:"bytes,6,opt,name=pod_id,json=podId,proto3" json:"pod_id,omitempty"`
	Name          string                 `protobuf:"bytes,7,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Stat) Reset() {
	*x = Stat{}
	mi := &file_machine_machine_proto_msgTypes[80]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Stat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Stat) ProtoMessage() {}

func (x *Stat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[80]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Stat.ProtoReflect.Descriptor instead.
func (*Stat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{80}
}

func (x *Stat) GetNamespace() string {
	if x != nil {
		return x.Namespace
	}
	return ""
}

func (x *Stat) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *Stat) GetMemoryUsage() uint64 {
	if x != nil {
		return x.MemoryUsage
	}
	return 0
}

func (x *Stat) GetCpuUsage() uint64 {
	if x != nil {
		return x.CpuUsage
	}
	return 0
}

func (x *Stat) GetPodId() string {
	if x != nil {
		return x.PodId
	}
	return ""
}

func (x *Stat) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

type Memory struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Meminfo       *MemInfo               `protobuf:"bytes,2,opt,name=meminfo,proto3" json:"meminfo,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Memory) Reset() {
	*x = Memory{}
	mi := &file_machine_machine_proto_msgTypes[81]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Memory) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Memory) ProtoMessage() {}

func (x *Memory) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[81]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Memory.ProtoReflect.Descriptor instead.
func (*Memory) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{81}
}

func (x *Memory) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Memory) GetMeminfo() *MemInfo {
	if x != nil {
		return x.Meminfo
	}
	return nil
}

type MemoryResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Memory              `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MemoryResponse) Reset() {
	*x = MemoryResponse{}
	mi := &file_machine_machine_proto_msgTypes[82]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MemoryResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MemoryResponse) ProtoMessage() {}

func (x *MemoryResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[82]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MemoryResponse.ProtoReflect.Descriptor instead.
func (*MemoryResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{82}
}

func (x *MemoryResponse) GetMessages() []*Memory {
	if x != nil {
		return x.Messages
	}
	return nil
}

type MemInfo struct {
	state             protoimpl.MessageState `protogen:"open.v1"`
	Memtotal          uint64                 `protobuf:"varint,1,opt,name=memtotal,proto3" json:"memtotal,omitempty"`
	Memfree           uint64                 `protobuf:"varint,2,opt,name=memfree,proto3" json:"memfree,omitempty"`
	Memavailable      uint64                 `protobuf:"varint,3,opt,name=memavailable,proto3" json:"memavailable,omitempty"`
	Buffers           uint64                 `protobuf:"varint,4,opt,name=buffers,proto3" json:"buffers,omitempty"`
	Cached            uint64                 `protobuf:"varint,5,opt,name=cached,proto3" json:"cached,omitempty"`
	Swapcached        uint64                 `protobuf:"varint,6,opt,name=swapcached,proto3" json:"swapcached,omitempty"`
	Active            uint64                 `protobuf:"varint,7,opt,name=active,proto3" json:"active,omitempty"`
	Inactive          uint64                 `protobuf:"varint,8,opt,name=inactive,proto3" json:"inactive,omitempty"`
	Activeanon        uint64                 `protobuf:"varint,9,opt,name=activeanon,proto3" json:"activeanon,omitempty"`
	Inactiveanon      uint64                 `protobuf:"varint,10,opt,name=inactiveanon,proto3" json:"inactiveanon,omitempty"`
	Activefile        uint64                 `protobuf:"varint,11,opt,name=activefile,proto3" json:"activefile,omitempty"`
	Inactivefile      uint64                 `protobuf:"varint,12,opt,name=inactivefile,proto3" json:"inactivefile,omitempty"`
	Unevictable       uint64                 `protobuf:"varint,13,opt,name=unevictable,proto3" json:"unevictable,omitempty"`
	Mlocked           uint64                 `protobuf:"varint,14,opt,name=mlocked,proto3" json:"mlocked,omitempty"`
	Swaptotal         uint64                 `protobuf:"varint,15,opt,name=swaptotal,proto3" json:"swaptotal,omitempty"`
	Swapfree          uint64                 `protobuf:"varint,16,opt,name=swapfree,proto3" json:"swapfree,omitempty"`
	Dirty             uint64                 `protobuf:"varint,17,opt,name=dirty,proto3" json:"dirty,omitempty"`
	Writeback         uint64                 `protobuf:"varint,18,opt,name=writeback,proto3" json:"writeback,omitempty"`
	Anonpages         uint64                 `protobuf:"varint,19,opt,name=anonpages,proto3" json:"anonpages,omitempty"`
	Mapped            uint64                 `protobuf:"varint,20,opt,name=mapped,proto3" json:"mapped,omitempty"`
	Shmem             uint64                 `protobuf:"varint,21,opt,name=shmem,proto3" json:"shmem,omitempty"`
	Slab              uint64                 `protobuf:"varint,22,opt,name=slab,proto3" json:"slab,omitempty"`
	Sreclaimable      uint64                 `protobuf:"varint,23,opt,name=sreclaimable,proto3" json:"sreclaimable,omitempty"`
	Sunreclaim        uint64                 `protobuf:"varint,24,opt,name=sunreclaim,proto3" json:"sunreclaim,omitempty"`
	Kernelstack       uint64                 `protobuf:"varint,25,opt,name=kernelstack,proto3" json:"kernelstack,omitempty"`
	Pagetables        uint64                 `protobuf:"varint,26,opt,name=pagetables,proto3" json:"pagetables,omitempty"`
	Nfsunstable       uint64                 `protobuf:"varint,27,opt,name=nfsunstable,proto3" json:"nfsunstable,omitempty"`
	Bounce            uint64                 `protobuf:"varint,28,opt,name=bounce,proto3" json:"bounce,omitempty"`
	Writebacktmp      uint64                 `protobuf:"varint,29,opt,name=writebacktmp,proto3" json:"writebacktmp,omitempty"`
	Commitlimit       uint64                 `protobuf:"varint,30,opt,name=commitlimit,proto3" json:"commitlimit,omitempty"`
	Committedas       uint64                 `protobuf:"varint,31,opt,name=committedas,proto3" json:"committedas,omitempty"`
	Vmalloctotal      uint64                 `protobuf:"varint,32,opt,name=vmalloctotal,proto3" json:"vmalloctotal,omitempty"`
	Vmallocused       uint64                 `protobuf:"varint,33,opt,name=vmallocused,proto3" json:"vmallocused,omitempty"`
	Vmallocchunk      uint64                 `protobuf:"varint,34,opt,name=vmallocchunk,proto3" json:"vmallocchunk,omitempty"`
	Hardwarecorrupted uint64                 `protobuf:"varint,35,opt,name=hardwarecorrupted,proto3" json:"hardwarecorrupted,omitempty"`
	Anonhugepages     uint64                 `protobuf:"varint,36,opt,name=anonhugepages,proto3" json:"anonhugepages,omitempty"`
	Shmemhugepages    uint64                 `protobuf:"varint,37,opt,name=shmemhugepages,proto3" json:"shmemhugepages,omitempty"`
	Shmempmdmapped    uint64                 `protobuf:"varint,38,opt,name=shmempmdmapped,proto3" json:"shmempmdmapped,omitempty"`
	Cmatotal          uint64                 `protobuf:"varint,39,opt,name=cmatotal,proto3" json:"cmatotal,omitempty"`
	Cmafree           uint64                 `protobuf:"varint,40,opt,name=cmafree,proto3" json:"cmafree,omitempty"`
	Hugepagestotal    uint64                 `protobuf:"varint,41,opt,name=hugepagestotal,proto3" json:"hugepagestotal,omitempty"`
	Hugepagesfree     uint64                 `protobuf:"varint,42,opt,name=hugepagesfree,proto3" json:"hugepagesfree,omitempty"`
	Hugepagesrsvd     uint64                 `protobuf:"varint,43,opt,name=hugepagesrsvd,proto3" json:"hugepagesrsvd,omitempty"`
	Hugepagessurp     uint64                 `protobuf:"varint,44,opt,name=hugepagessurp,proto3" json:"hugepagessurp,omitempty"`
	Hugepagesize      uint64                 `protobuf:"varint,45,opt,name=hugepagesize,proto3" json:"hugepagesize,omitempty"`
	Directmap4K       uint64                 `protobuf:"varint,46,opt,name=directmap4k,proto3" json:"directmap4k,omitempty"`
	Directmap2M       uint64                 `protobuf:"varint,47,opt,name=directmap2m,proto3" json:"directmap2m,omitempty"`
	Directmap1G       uint64                 `protobuf:"varint,48,opt,name=directmap1g,proto3" json:"directmap1g,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *MemInfo) Reset() {
	*x = MemInfo{}
	mi := &file_machine_machine_proto_msgTypes[83]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MemInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MemInfo) ProtoMessage() {}

func (x *MemInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[83]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MemInfo.ProtoReflect.Descriptor instead.
func (*MemInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{83}
}

func (x *MemInfo) GetMemtotal() uint64 {
	if x != nil {
		return x.Memtotal
	}
	return 0
}

func (x *MemInfo) GetMemfree() uint64 {
	if x != nil {
		return x.Memfree
	}
	return 0
}

func (x *MemInfo) GetMemavailable() uint64 {
	if x != nil {
		return x.Memavailable
	}
	return 0
}

func (x *MemInfo) GetBuffers() uint64 {
	if x != nil {
		return x.Buffers
	}
	return 0
}

func (x *MemInfo) GetCached() uint64 {
	if x != nil {
		return x.Cached
	}
	return 0
}

func (x *MemInfo) GetSwapcached() uint64 {
	if x != nil {
		return x.Swapcached
	}
	return 0
}

func (x *MemInfo) GetActive() uint64 {
	if x != nil {
		return x.Active
	}
	return 0
}

func (x *MemInfo) GetInactive() uint64 {
	if x != nil {
		return x.Inactive
	}
	return 0
}

func (x *MemInfo) GetActiveanon() uint64 {
	if x != nil {
		return x.Activeanon
	}
	return 0
}

func (x *MemInfo) GetInactiveanon() uint64 {
	if x != nil {
		return x.Inactiveanon
	}
	return 0
}

func (x *MemInfo) GetActivefile() uint64 {
	if x != nil {
		return x.Activefile
	}
	return 0
}

func (x *MemInfo) GetInactivefile() uint64 {
	if x != nil {
		return x.Inactivefile
	}
	return 0
}

func (x *MemInfo) GetUnevictable() uint64 {
	if x != nil {
		return x.Unevictable
	}
	return 0
}

func (x *MemInfo) GetMlocked() uint64 {
	if x != nil {
		return x.Mlocked
	}
	return 0
}

func (x *MemInfo) GetSwaptotal() uint64 {
	if x != nil {
		return x.Swaptotal
	}
	return 0
}

func (x *MemInfo) GetSwapfree() uint64 {
	if x != nil {
		return x.Swapfree
	}
	return 0
}

func (x *MemInfo) GetDirty() uint64 {
	if x != nil {
		return x.Dirty
	}
	return 0
}

func (x *MemInfo) GetWriteback() uint64 {
	if x != nil {
		return x.Writeback
	}
	return 0
}

func (x *MemInfo) GetAnonpages() uint64 {
	if x != nil {
		return x.Anonpages
	}
	return 0
}

func (x *MemInfo) GetMapped() uint64 {
	if x != nil {
		return x.Mapped
	}
	return 0
}

func (x *MemInfo) GetShmem() uint64 {
	if x != nil {
		return x.Shmem
	}
	return 0
}

func (x *MemInfo) GetSlab() uint64 {
	if x != nil {
		return x.Slab
	}
	return 0
}

func (x *MemInfo) GetSreclaimable() uint64 {
	if x != nil {
		return x.Sreclaimable
	}
	return 0
}

func (x *MemInfo) GetSunreclaim() uint64 {
	if x != nil {
		return x.Sunreclaim
	}
	return 0
}

func (x *MemInfo) GetKernelstack() uint64 {
	if x != nil {
		return x.Kernelstack
	}
	return 0
}

func (x *MemInfo) GetPagetables() uint64 {
	if x != nil {
		return x.Pagetables
	}
	return 0
}

func (x *MemInfo) GetNfsunstable() uint64 {
	if x != nil {
		return x.Nfsunstable
	}
	return 0
}

func (x *MemInfo) GetBounce() uint64 {
	if x != nil {
		return x.Bounce
	}
	return 0
}

func (x *MemInfo) GetWritebacktmp() uint64 {
	if x != nil {
		return x.Writebacktmp
	}
	return 0
}

func (x *MemInfo) GetCommitlimit() uint64 {
	if x != nil {
		return x.Commitlimit
	}
	return 0
}

func (x *MemInfo) GetCommittedas() uint64 {
	if x != nil {
		return x.Committedas
	}
	return 0
}

func (x *MemInfo) GetVmalloctotal() uint64 {
	if x != nil {
		return x.Vmalloctotal
	}
	return 0
}

func (x *MemInfo) GetVmallocused() uint64 {
	if x != nil {
		return x.Vmallocused
	}
	return 0
}

func (x *MemInfo) GetVmallocchunk() uint64 {
	if x != nil {
		return x.Vmallocchunk
	}
	return 0
}

func (x *MemInfo) GetHardwarecorrupted() uint64 {
	if x != nil {
		return x.Hardwarecorrupted
	}
	return 0
}

func (x *MemInfo) GetAnonhugepages() uint64 {
	if x != nil {
		return x.Anonhugepages
	}
	return 0
}

func (x *MemInfo) GetShmemhugepages() uint64 {
	if x != nil {
		return x.Shmemhugepages
	}
	return 0
}

func (x *MemInfo) GetShmempmdmapped() uint64 {
	if x != nil {
		return x.Shmempmdmapped
	}
	return 0
}

func (x *MemInfo) GetCmatotal() uint64 {
	if x != nil {
		return x.Cmatotal
	}
	return 0
}

func (x *MemInfo) GetCmafree() uint64 {
	if x != nil {
		return x.Cmafree
	}
	return 0
}

func (x *MemInfo) GetHugepagestotal() uint64 {
	if x != nil {
		return x.Hugepagestotal
	}
	return 0
}

func (x *MemInfo) GetHugepagesfree() uint64 {
	if x != nil {
		return x.Hugepagesfree
	}
	return 0
}

func (x *MemInfo) GetHugepagesrsvd() uint64 {
	if x != nil {
		return x.Hugepagesrsvd
	}
	return 0
}

func (x *MemInfo) GetHugepagessurp() uint64 {
	if x != nil {
		return x.Hugepagessurp
	}
	return 0
}

func (x *MemInfo) GetHugepagesize() uint64 {
	if x != nil {
		return x.Hugepagesize
	}
	return 0
}

func (x *MemInfo) GetDirectmap4K() uint64 {
	if x != nil {
		return x.Directmap4K
	}
	return 0
}

func (x *MemInfo) GetDirectmap2M() uint64 {
	if x != nil {
		return x.Directmap2M
	}
	return 0
}

func (x *MemInfo) GetDirectmap1G() uint64 {
	if x != nil {
		return x.Directmap1G
	}
	return 0
}

type HostnameResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Hostname            `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HostnameResponse) Reset() {
	*x = HostnameResponse{}
	mi := &file_machine_machine_proto_msgTypes[84]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HostnameResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HostnameResponse) ProtoMessage() {}

func (x *HostnameResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[84]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HostnameResponse.ProtoReflect.Descriptor instead.
func (*HostnameResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{84}
}

func (x *HostnameResponse) GetMessages() []*Hostname {
	if x != nil {
		return x.Messages
	}
	return nil
}

type Hostname struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Hostname      string                 `protobuf:"bytes,2,opt,name=hostname,proto3" json:"hostname,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Hostname) Reset() {
	*x = Hostname{}
	mi := &file_machine_machine_proto_msgTypes[85]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Hostname) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Hostname) ProtoMessage() {}

func (x *Hostname) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[85]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Hostname.ProtoReflect.Descriptor instead.
func (*Hostname) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{85}
}

func (x *Hostname) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Hostname) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

type LoadAvgResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*LoadAvg             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LoadAvgResponse) Reset() {
	*x = LoadAvgResponse{}
	mi := &file_machine_machine_proto_msgTypes[86]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LoadAvgResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LoadAvgResponse) ProtoMessage() {}

func (x *LoadAvgResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[86]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LoadAvgResponse.ProtoReflect.Descriptor instead.
func (*LoadAvgResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{86}
}

func (x *LoadAvgResponse) GetMessages() []*LoadAvg {
	if x != nil {
		return x.Messages
	}
	return nil
}

type LoadAvg struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Load1         float64                `protobuf:"fixed64,2,opt,name=load1,proto3" json:"load1,omitempty"`
	Load5         float64                `protobuf:"fixed64,3,opt,name=load5,proto3" json:"load5,omitempty"`
	Load15        float64                `protobuf:"fixed64,4,opt,name=load15,proto3" json:"load15,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LoadAvg) Reset() {
	*x = LoadAvg{}
	mi := &file_machine_machine_proto_msgTypes[87]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LoadAvg) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LoadAvg) ProtoMessage() {}

func (x *LoadAvg) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[87]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LoadAvg.ProtoReflect.Descriptor instead.
func (*LoadAvg) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{87}
}

func (x *LoadAvg) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *LoadAvg) GetLoad1() float64 {
	if x != nil {
		return x.Load1
	}
	return 0
}

func (x *LoadAvg) GetLoad5() float64 {
	if x != nil {
		return x.Load5
	}
	return 0
}

func (x *LoadAvg) GetLoad15() float64 {
	if x != nil {
		return x.Load15
	}
	return 0
}

type SystemStatResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*SystemStat          `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SystemStatResponse) Reset() {
	*x = SystemStatResponse{}
	mi := &file_machine_machine_proto_msgTypes[88]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SystemStatResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SystemStatResponse) ProtoMessage() {}

func (x *SystemStatResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[88]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SystemStatResponse.ProtoReflect.Descriptor instead.
func (*SystemStatResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{88}
}

func (x *SystemStatResponse) GetMessages() []*SystemStat {
	if x != nil {
		return x.Messages
	}
	return nil
}

type SystemStat struct {
	state           protoimpl.MessageState `protogen:"open.v1"`
	Metadata        *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	BootTime        uint64                 `protobuf:"varint,2,opt,name=boot_time,json=bootTime,proto3" json:"boot_time,omitempty"`
	CpuTotal        *CPUStat               `protobuf:"bytes,3,opt,name=cpu_total,json=cpuTotal,proto3" json:"cpu_total,omitempty"`
	Cpu             []*CPUStat             `protobuf:"bytes,4,rep,name=cpu,proto3" json:"cpu,omitempty"`
	IrqTotal        uint64                 `protobuf:"varint,5,opt,name=irq_total,json=irqTotal,proto3" json:"irq_total,omitempty"`
	Irq             []uint64               `protobuf:"varint,6,rep,packed,name=irq,proto3" json:"irq,omitempty"`
	ContextSwitches uint64                 `protobuf:"varint,7,opt,name=context_switches,json=contextSwitches,proto3" json:"context_switches,omitempty"`
	ProcessCreated  uint64                 `protobuf:"varint,8,opt,name=process_created,json=processCreated,proto3" json:"process_created,omitempty"`
	ProcessRunning  uint64                 `protobuf:"varint,9,opt,name=process_running,json=processRunning,proto3" json:"process_running,omitempty"`
	ProcessBlocked  uint64                 `protobuf:"varint,10,opt,name=process_blocked,json=processBlocked,proto3" json:"process_blocked,omitempty"`
	SoftIrqTotal    uint64                 `protobuf:"varint,11,opt,name=soft_irq_total,json=softIrqTotal,proto3" json:"soft_irq_total,omitempty"`
	SoftIrq         *SoftIRQStat           `protobuf:"bytes,12,opt,name=soft_irq,json=softIrq,proto3" json:"soft_irq,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *SystemStat) Reset() {
	*x = SystemStat{}
	mi := &file_machine_machine_proto_msgTypes[89]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SystemStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SystemStat) ProtoMessage() {}

func (x *SystemStat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[89]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SystemStat.ProtoReflect.Descriptor instead.
func (*SystemStat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{89}
}

func (x *SystemStat) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *SystemStat) GetBootTime() uint64 {
	if x != nil {
		return x.BootTime
	}
	return 0
}

func (x *SystemStat) GetCpuTotal() *CPUStat {
	if x != nil {
		return x.CpuTotal
	}
	return nil
}

func (x *SystemStat) GetCpu() []*CPUStat {
	if x != nil {
		return x.Cpu
	}
	return nil
}

func (x *SystemStat) GetIrqTotal() uint64 {
	if x != nil {
		return x.IrqTotal
	}
	return 0
}

func (x *SystemStat) GetIrq() []uint64 {
	if x != nil {
		return x.Irq
	}
	return nil
}

func (x *SystemStat) GetContextSwitches() uint64 {
	if x != nil {
		return x.ContextSwitches
	}
	return 0
}

func (x *SystemStat) GetProcessCreated() uint64 {
	if x != nil {
		return x.ProcessCreated
	}
	return 0
}

func (x *SystemStat) GetProcessRunning() uint64 {
	if x != nil {
		return x.ProcessRunning
	}
	return 0
}

func (x *SystemStat) GetProcessBlocked() uint64 {
	if x != nil {
		return x.ProcessBlocked
	}
	return 0
}

func (x *SystemStat) GetSoftIrqTotal() uint64 {
	if x != nil {
		return x.SoftIrqTotal
	}
	return 0
}

func (x *SystemStat) GetSoftIrq() *SoftIRQStat {
	if x != nil {
		return x.SoftIrq
	}
	return nil
}

type CPUStat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	User          float64                `protobuf:"fixed64,1,opt,name=user,proto3" json:"user,omitempty"`
	Nice          float64                `protobuf:"fixed64,2,opt,name=nice,proto3" json:"nice,omitempty"`
	System        float64                `protobuf:"fixed64,3,opt,name=system,proto3" json:"system,omitempty"`
	Idle          float64                `protobuf:"fixed64,4,opt,name=idle,proto3" json:"idle,omitempty"`
	Iowait        float64                `protobuf:"fixed64,5,opt,name=iowait,proto3" json:"iowait,omitempty"`
	Irq           float64                `protobuf:"fixed64,6,opt,name=irq,proto3" json:"irq,omitempty"`
	SoftIrq       float64                `protobuf:"fixed64,7,opt,name=soft_irq,json=softIrq,proto3" json:"soft_irq,omitempty"`
	Steal         float64                `protobuf:"fixed64,8,opt,name=steal,proto3" json:"steal,omitempty"`
	Guest         float64                `protobuf:"fixed64,9,opt,name=guest,proto3" json:"guest,omitempty"`
	GuestNice     float64                `protobuf:"fixed64,10,opt,name=guest_nice,json=guestNice,proto3" json:"guest_nice,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CPUStat) Reset() {
	*x = CPUStat{}
	mi := &file_machine_machine_proto_msgTypes[90]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUStat) ProtoMessage() {}

func (x *CPUStat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[90]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUStat.ProtoReflect.Descriptor instead.
func (*CPUStat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{90}
}

func (x *CPUStat) GetUser() float64 {
	if x != nil {
		return x.User
	}
	return 0
}

func (x *CPUStat) GetNice() float64 {
	if x != nil {
		return x.Nice
	}
	return 0
}

func (x *CPUStat) GetSystem() float64 {
	if x != nil {
		return x.System
	}
	return 0
}

func (x *CPUStat) GetIdle() float64 {
	if x != nil {
		return x.Idle
	}
	return 0
}

func (x *CPUStat) GetIowait() float64 {
	if x != nil {
		return x.Iowait
	}
	return 0
}

func (x *CPUStat) GetIrq() float64 {
	if x != nil {
		return x.Irq
	}
	return 0
}

func (x *CPUStat) GetSoftIrq() float64 {
	if x != nil {
		return x.SoftIrq
	}
	return 0
}

func (x *CPUStat) GetSteal() float64 {
	if x != nil {
		return x.Steal
	}
	return 0
}

func (x *CPUStat) GetGuest() float64 {
	if x != nil {
		return x.Guest
	}
	return 0
}

func (x *CPUStat) GetGuestNice() float64 {
	if x != nil {
		return x.GuestNice
	}
	return 0
}

type SoftIRQStat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Hi            uint64                 `protobuf:"varint,1,opt,name=hi,proto3" json:"hi,omitempty"`
	Timer         uint64                 `protobuf:"varint,2,opt,name=timer,proto3" json:"timer,omitempty"`
	NetTx         uint64                 `protobuf:"varint,3,opt,name=net_tx,json=netTx,proto3" json:"net_tx,omitempty"`
	NetRx         uint64                 `protobuf:"varint,4,opt,name=net_rx,json=netRx,proto3" json:"net_rx,omitempty"`
	Block         uint64                 `protobuf:"varint,5,opt,name=block,proto3" json:"block,omitempty"`
	BlockIoPoll   uint64                 `protobuf:"varint,6,opt,name=block_io_poll,json=blockIoPoll,proto3" json:"block_io_poll,omitempty"`
	Tasklet       uint64                 `protobuf:"varint,7,opt,name=tasklet,proto3" json:"tasklet,omitempty"`
	Sched         uint64                 `protobuf:"varint,8,opt,name=sched,proto3" json:"sched,omitempty"`
	Hrtimer       uint64                 `protobuf:"varint,9,opt,name=hrtimer,proto3" json:"hrtimer,omitempty"`
	Rcu           uint64                 `protobuf:"varint,10,opt,name=rcu,proto3" json:"rcu,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SoftIRQStat) Reset() {
	*x = SoftIRQStat{}
	mi := &file_machine_machine_proto_msgTypes[91]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SoftIRQStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SoftIRQStat) ProtoMessage() {}

func (x *SoftIRQStat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[91]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SoftIRQStat.ProtoReflect.Descriptor instead.
func (*SoftIRQStat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{91}
}

func (x *SoftIRQStat) GetHi() uint64 {
	if x != nil {
		return x.Hi
	}
	return 0
}

func (x *SoftIRQStat) GetTimer() uint64 {
	if x != nil {
		return x.Timer
	}
	return 0
}

func (x *SoftIRQStat) GetNetTx() uint64 {
	if x != nil {
		return x.NetTx
	}
	return 0
}

func (x *SoftIRQStat) GetNetRx() uint64 {
	if x != nil {
		return x.NetRx
	}
	return 0
}

func (x *SoftIRQStat) GetBlock() uint64 {
	if x != nil {
		return x.Block
	}
	return 0
}

func (x *SoftIRQStat) GetBlockIoPoll() uint64 {
	if x != nil {
		return x.BlockIoPoll
	}
	return 0
}

func (x *SoftIRQStat) GetTasklet() uint64 {
	if x != nil {
		return x.Tasklet
	}
	return 0
}

func (x *SoftIRQStat) GetSched() uint64 {
	if x != nil {
		return x.Sched
	}
	return 0
}

func (x *SoftIRQStat) GetHrtimer() uint64 {
	if x != nil {
		return x.Hrtimer
	}
	return 0
}

func (x *SoftIRQStat) GetRcu() uint64 {
	if x != nil {
		return x.Rcu
	}
	return 0
}

type CPUFreqStatsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*CPUsFreqStats       `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CPUFreqStatsResponse) Reset() {
	*x = CPUFreqStatsResponse{}
	mi := &file_machine_machine_proto_msgTypes[92]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUFreqStatsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUFreqStatsResponse) ProtoMessage() {}

func (x *CPUFreqStatsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[92]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUFreqStatsResponse.ProtoReflect.Descriptor instead.
func (*CPUFreqStatsResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{92}
}

func (x *CPUFreqStatsResponse) GetMessages() []*CPUsFreqStats {
	if x != nil {
		return x.Messages
	}
	return nil
}

type CPUsFreqStats struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	CpuFreqStats  []*CPUFreqStats        `protobuf:"bytes,2,rep,name=cpu_freq_stats,json=cpuFreqStats,proto3" json:"cpu_freq_stats,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CPUsFreqStats) Reset() {
	*x = CPUsFreqStats{}
	mi := &file_machine_machine_proto_msgTypes[93]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUsFreqStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUsFreqStats) ProtoMessage() {}

func (x *CPUsFreqStats) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[93]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUsFreqStats.ProtoReflect.Descriptor instead.
func (*CPUsFreqStats) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{93}
}

func (x *CPUsFreqStats) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *CPUsFreqStats) GetCpuFreqStats() []*CPUFreqStats {
	if x != nil {
		return x.CpuFreqStats
	}
	return nil
}

type CPUFreqStats struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	CurrentFrequency uint64                 `protobuf:"varint,1,opt,name=current_frequency,json=currentFrequency,proto3" json:"current_frequency,omitempty"`
	MinimumFrequency uint64                 `protobuf:"varint,2,opt,name=minimum_frequency,json=minimumFrequency,proto3" json:"minimum_frequency,omitempty"`
	MaximumFrequency uint64                 `protobuf:"varint,3,opt,name=maximum_frequency,json=maximumFrequency,proto3" json:"maximum_frequency,omitempty"`
	Governor         string                 `protobuf:"bytes,4,opt,name=governor,proto3" json:"governor,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *CPUFreqStats) Reset() {
	*x = CPUFreqStats{}
	mi := &file_machine_machine_proto_msgTypes[94]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUFreqStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUFreqStats) ProtoMessage() {}

func (x *CPUFreqStats) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[94]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUFreqStats.ProtoReflect.Descriptor instead.
func (*CPUFreqStats) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{94}
}

func (x *CPUFreqStats) GetCurrentFrequency() uint64 {
	if x != nil {
		return x.CurrentFrequency
	}
	return 0
}

func (x *CPUFreqStats) GetMinimumFrequency() uint64 {
	if x != nil {
		return x.MinimumFrequency
	}
	return 0
}

func (x *CPUFreqStats) GetMaximumFrequency() uint64 {
	if x != nil {
		return x.MaximumFrequency
	}
	return 0
}

func (x *CPUFreqStats) GetGovernor() string {
	if x != nil {
		return x.Governor
	}
	return ""
}

type CPUInfoResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*CPUsInfo            `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CPUInfoResponse) Reset() {
	*x = CPUInfoResponse{}
	mi := &file_machine_machine_proto_msgTypes[95]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUInfoResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUInfoResponse) ProtoMessage() {}

func (x *CPUInfoResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[95]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUInfoResponse.ProtoReflect.Descriptor instead.
func (*CPUInfoResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{95}
}

func (x *CPUInfoResponse) GetMessages() []*CPUsInfo {
	if x != nil {
		return x.Messages
	}
	return nil
}

type CPUsInfo struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	CpuInfo       []*CPUInfo             `protobuf:"bytes,2,rep,name=cpu_info,json=cpuInfo,proto3" json:"cpu_info,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CPUsInfo) Reset() {
	*x = CPUsInfo{}
	mi := &file_machine_machine_proto_msgTypes[96]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUsInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUsInfo) ProtoMessage() {}

func (x *CPUsInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[96]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUsInfo.ProtoReflect.Descriptor instead.
func (*CPUsInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{96}
}

func (x *CPUsInfo) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *CPUsInfo) GetCpuInfo() []*CPUInfo {
	if x != nil {
		return x.CpuInfo
	}
	return nil
}

type CPUInfo struct {
	state           protoimpl.MessageState `protogen:"open.v1"`
	Processor       uint32                 `protobuf:"varint,1,opt,name=processor,proto3" json:"processor,omitempty"`
	VendorId        string                 `protobuf:"bytes,2,opt,name=vendor_id,json=vendorId,proto3" json:"vendor_id,omitempty"`
	CpuFamily       string                 `protobuf:"bytes,3,opt,name=cpu_family,json=cpuFamily,proto3" json:"cpu_family,omitempty"`
	Model           string                 `protobuf:"bytes,4,opt,name=model,proto3" json:"model,omitempty"`
	ModelName       string                 `protobuf:"bytes,5,opt,name=model_name,json=modelName,proto3" json:"model_name,omitempty"`
	Stepping        string                 `protobuf:"bytes,6,opt,name=stepping,proto3" json:"stepping,omitempty"`
	Microcode       string                 `protobuf:"bytes,7,opt,name=microcode,proto3" json:"microcode,omitempty"`
	CpuMhz          float64                `protobuf:"fixed64,8,opt,name=cpu_mhz,json=cpuMhz,proto3" json:"cpu_mhz,omitempty"`
	CacheSize       string                 `protobuf:"bytes,9,opt,name=cache_size,json=cacheSize,proto3" json:"cache_size,omitempty"`
	PhysicalId      string                 `protobuf:"bytes,10,opt,name=physical_id,json=physicalId,proto3" json:"physical_id,omitempty"`
	Siblings        uint32                 `protobuf:"varint,11,opt,name=siblings,proto3" json:"siblings,omitempty"`
	CoreId          string                 `protobuf:"bytes,12,opt,name=core_id,json=coreId,proto3" json:"core_id,omitempty"`
	CpuCores        uint32                 `protobuf:"varint,13,opt,name=cpu_cores,json=cpuCores,proto3" json:"cpu_cores,omitempty"`
	ApicId          string                 `protobuf:"bytes,14,opt,name=apic_id,json=apicId,proto3" json:"apic_id,omitempty"`
	InitialApicId   string                 `protobuf:"bytes,15,opt,name=initial_apic_id,json=initialApicId,proto3" json:"initial_apic_id,omitempty"`
	Fpu             string                 `protobuf:"bytes,16,opt,name=fpu,proto3" json:"fpu,omitempty"`
	FpuException    string                 `protobuf:"bytes,17,opt,name=fpu_exception,json=fpuException,proto3" json:"fpu_exception,omitempty"`
	CpuIdLevel      uint32                 `protobuf:"varint,18,opt,name=cpu_id_level,json=cpuIdLevel,proto3" json:"cpu_id_level,omitempty"`
	Wp              string                 `protobuf:"bytes,19,opt,name=wp,proto3" json:"wp,omitempty"`
	Flags           []string               `protobuf:"bytes,20,rep,name=flags,proto3" json:"flags,omitempty"`
	Bugs            []string               `protobuf:"bytes,21,rep,name=bugs,proto3" json:"bugs,omitempty"`
	BogoMips        float64                `protobuf:"fixed64,22,opt,name=bogo_mips,json=bogoMips,proto3" json:"bogo_mips,omitempty"`
	ClFlushSize     uint32                 `protobuf:"varint,23,opt,name=cl_flush_size,json=clFlushSize,proto3" json:"cl_flush_size,omitempty"`
	CacheAlignment  uint32                 `protobuf:"varint,24,opt,name=cache_alignment,json=cacheAlignment,proto3" json:"cache_alignment,omitempty"`
	AddressSizes    string                 `protobuf:"bytes,25,opt,name=address_sizes,json=addressSizes,proto3" json:"address_sizes,omitempty"`
	PowerManagement string                 `protobuf:"bytes,26,opt,name=power_management,json=powerManagement,proto3" json:"power_management,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *CPUInfo) Reset() {
	*x = CPUInfo{}
	mi := &file_machine_machine_proto_msgTypes[97]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CPUInfo) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CPUInfo) ProtoMessage() {}

func (x *CPUInfo) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[97]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CPUInfo.ProtoReflect.Descriptor instead.
func (*CPUInfo) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{97}
}

func (x *CPUInfo) GetProcessor() uint32 {
	if x != nil {
		return x.Processor
	}
	return 0
}

func (x *CPUInfo) GetVendorId() string {
	if x != nil {
		return x.VendorId
	}
	return ""
}

func (x *CPUInfo) GetCpuFamily() string {
	if x != nil {
		return x.CpuFamily
	}
	return ""
}

func (x *CPUInfo) GetModel() string {
	if x != nil {
		return x.Model
	}
	return ""
}

func (x *CPUInfo) GetModelName() string {
	if x != nil {
		return x.ModelName
	}
	return ""
}

func (x *CPUInfo) GetStepping() string {
	if x != nil {
		return x.Stepping
	}
	return ""
}

func (x *CPUInfo) GetMicrocode() string {
	if x != nil {
		return x.Microcode
	}
	return ""
}

func (x *CPUInfo) GetCpuMhz() float64 {
	if x != nil {
		return x.CpuMhz
	}
	return 0
}

func (x *CPUInfo) GetCacheSize() string {
	if x != nil {
		return x.CacheSize
	}
	return ""
}

func (x *CPUInfo) GetPhysicalId() string {
	if x != nil {
		return x.PhysicalId
	}
	return ""
}

func (x *CPUInfo) GetSiblings() uint32 {
	if x != nil {
		return x.Siblings
	}
	return 0
}

func (x *CPUInfo) GetCoreId() string {
	if x != nil {
		return x.CoreId
	}
	return ""
}

func (x *CPUInfo) GetCpuCores() uint32 {
	if x != nil {
		return x.CpuCores
	}
	return 0
}

func (x *CPUInfo) GetApicId() string {
	if x != nil {
		return x.ApicId
	}
	return ""
}

func (x *CPUInfo) GetInitialApicId() string {
	if x != nil {
		return x.InitialApicId
	}
	return ""
}

func (x *CPUInfo) GetFpu() string {
	if x != nil {
		return x.Fpu
	}
	return ""
}

func (x *CPUInfo) GetFpuException() string {
	if x != nil {
		return x.FpuException
	}
	return ""
}

func (x *CPUInfo) GetCpuIdLevel() uint32 {
	if x != nil {
		return x.CpuIdLevel
	}
	return 0
}

func (x *CPUInfo) GetWp() string {
	if x != nil {
		return x.Wp
	}
	return ""
}

func (x *CPUInfo) GetFlags() []string {
	if x != nil {
		return x.Flags
	}
	return nil
}

func (x *CPUInfo) GetBugs() []string {
	if x != nil {
		return x.Bugs
	}
	return nil
}

func (x *CPUInfo) GetBogoMips() float64 {
	if x != nil {
		return x.BogoMips
	}
	return 0
}

func (x *CPUInfo) GetClFlushSize() uint32 {
	if x != nil {
		return x.ClFlushSize
	}
	return 0
}

func (x *CPUInfo) GetCacheAlignment() uint32 {
	if x != nil {
		return x.CacheAlignment
	}
	return 0
}

func (x *CPUInfo) GetAddressSizes() string {
	if x != nil {
		return x.AddressSizes
	}
	return ""
}

func (x *CPUInfo) GetPowerManagement() string {
	if x != nil {
		return x.PowerManagement
	}
	return ""
}

type NetworkDeviceStatsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*NetworkDeviceStats  `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetworkDeviceStatsResponse) Reset() {
	*x = NetworkDeviceStatsResponse{}
	mi := &file_machine_machine_proto_msgTypes[98]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkDeviceStatsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkDeviceStatsResponse) ProtoMessage() {}

func (x *NetworkDeviceStatsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[98]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkDeviceStatsResponse.ProtoReflect.Descriptor instead.
func (*NetworkDeviceStatsResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{98}
}

func (x *NetworkDeviceStatsResponse) GetMessages() []*NetworkDeviceStats {
	if x != nil {
		return x.Messages
	}
	return nil
}

type NetworkDeviceStats struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Total         *NetDev                `protobuf:"bytes,2,opt,name=total,proto3" json:"total,omitempty"`
	Devices       []*NetDev              `protobuf:"bytes,3,rep,name=devices,proto3" json:"devices,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetworkDeviceStats) Reset() {
	*x = NetworkDeviceStats{}
	mi := &file_machine_machine_proto_msgTypes[99]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkDeviceStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkDeviceStats) ProtoMessage() {}

func (x *NetworkDeviceStats) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[99]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkDeviceStats.ProtoReflect.Descriptor instead.
func (*NetworkDeviceStats) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{99}
}

func (x *NetworkDeviceStats) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *NetworkDeviceStats) GetTotal() *NetDev {
	if x != nil {
		return x.Total
	}
	return nil
}

func (x *NetworkDeviceStats) GetDevices() []*NetDev {
	if x != nil {
		return x.Devices
	}
	return nil
}

type NetDev struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	RxBytes       uint64                 `protobuf:"varint,2,opt,name=rx_bytes,json=rxBytes,proto3" json:"rx_bytes,omitempty"`
	RxPackets     uint64                 `protobuf:"varint,3,opt,name=rx_packets,json=rxPackets,proto3" json:"rx_packets,omitempty"`
	RxErrors      uint64                 `protobuf:"varint,4,opt,name=rx_errors,json=rxErrors,proto3" json:"rx_errors,omitempty"`
	RxDropped     uint64                 `protobuf:"varint,5,opt,name=rx_dropped,json=rxDropped,proto3" json:"rx_dropped,omitempty"`
	RxFifo        uint64                 `protobuf:"varint,6,opt,name=rx_fifo,json=rxFifo,proto3" json:"rx_fifo,omitempty"`
	RxFrame       uint64                 `protobuf:"varint,7,opt,name=rx_frame,json=rxFrame,proto3" json:"rx_frame,omitempty"`
	RxCompressed  uint64                 `protobuf:"varint,8,opt,name=rx_compressed,json=rxCompressed,proto3" json:"rx_compressed,omitempty"`
	RxMulticast   uint64                 `protobuf:"varint,9,opt,name=rx_multicast,json=rxMulticast,proto3" json:"rx_multicast,omitempty"`
	TxBytes       uint64                 `protobuf:"varint,10,opt,name=tx_bytes,json=txBytes,proto3" json:"tx_bytes,omitempty"`
	TxPackets     uint64                 `protobuf:"varint,11,opt,name=tx_packets,json=txPackets,proto3" json:"tx_packets,omitempty"`
	TxErrors      uint64                 `protobuf:"varint,12,opt,name=tx_errors,json=txErrors,proto3" json:"tx_errors,omitempty"`
	TxDropped     uint64                 `protobuf:"varint,13,opt,name=tx_dropped,json=txDropped,proto3" json:"tx_dropped,omitempty"`
	TxFifo        uint64                 `protobuf:"varint,14,opt,name=tx_fifo,json=txFifo,proto3" json:"tx_fifo,omitempty"`
	TxCollisions  uint64                 `protobuf:"varint,15,opt,name=tx_collisions,json=txCollisions,proto3" json:"tx_collisions,omitempty"`
	TxCarrier     uint64                 `protobuf:"varint,16,opt,name=tx_carrier,json=txCarrier,proto3" json:"tx_carrier,omitempty"`
	TxCompressed  uint64                 `protobuf:"varint,17,opt,name=tx_compressed,json=txCompressed,proto3" json:"tx_compressed,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetDev) Reset() {
	*x = NetDev{}
	mi := &file_machine_machine_proto_msgTypes[100]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetDev) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetDev) ProtoMessage() {}

func (x *NetDev) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[100]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetDev.ProtoReflect.Descriptor instead.
func (*NetDev) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{100}
}

func (x *NetDev) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *NetDev) GetRxBytes() uint64 {
	if x != nil {
		return x.RxBytes
	}
	return 0
}

func (x *NetDev) GetRxPackets() uint64 {
	if x != nil {
		return x.RxPackets
	}
	return 0
}

func (x *NetDev) GetRxErrors() uint64 {
	if x != nil {
		return x.RxErrors
	}
	return 0
}

func (x *NetDev) GetRxDropped() uint64 {
	if x != nil {
		return x.RxDropped
	}
	return 0
}

func (x *NetDev) GetRxFifo() uint64 {
	if x != nil {
		return x.RxFifo
	}
	return 0
}

func (x *NetDev) GetRxFrame() uint64 {
	if x != nil {
		return x.RxFrame
	}
	return 0
}

func (x *NetDev) GetRxCompressed() uint64 {
	if x != nil {
		return x.RxCompressed
	}
	return 0
}

func (x *NetDev) GetRxMulticast() uint64 {
	if x != nil {
		return x.RxMulticast
	}
	return 0
}

func (x *NetDev) GetTxBytes() uint64 {
	if x != nil {
		return x.TxBytes
	}
	return 0
}

func (x *NetDev) GetTxPackets() uint64 {
	if x != nil {
		return x.TxPackets
	}
	return 0
}

func (x *NetDev) GetTxErrors() uint64 {
	if x != nil {
		return x.TxErrors
	}
	return 0
}

func (x *NetDev) GetTxDropped() uint64 {
	if x != nil {
		return x.TxDropped
	}
	return 0
}

func (x *NetDev) GetTxFifo() uint64 {
	if x != nil {
		return x.TxFifo
	}
	return 0
}

func (x *NetDev) GetTxCollisions() uint64 {
	if x != nil {
		return x.TxCollisions
	}
	return 0
}

func (x *NetDev) GetTxCarrier() uint64 {
	if x != nil {
		return x.TxCarrier
	}
	return 0
}

func (x *NetDev) GetTxCompressed() uint64 {
	if x != nil {
		return x.TxCompressed
	}
	return 0
}

type DiskStatsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*DiskStats           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DiskStatsResponse) Reset() {
	*x = DiskStatsResponse{}
	mi := &file_machine_machine_proto_msgTypes[101]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiskStatsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiskStatsResponse) ProtoMessage() {}

func (x *DiskStatsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[101]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiskStatsResponse.ProtoReflect.Descriptor instead.
func (*DiskStatsResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{101}
}

func (x *DiskStatsResponse) GetMessages() []*DiskStats {
	if x != nil {
		return x.Messages
	}
	return nil
}

type DiskStats struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Total         *DiskStat              `protobuf:"bytes,2,opt,name=total,proto3" json:"total,omitempty"`
	Devices       []*DiskStat            `protobuf:"bytes,3,rep,name=devices,proto3" json:"devices,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DiskStats) Reset() {
	*x = DiskStats{}
	mi := &file_machine_machine_proto_msgTypes[102]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiskStats) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiskStats) ProtoMessage() {}

func (x *DiskStats) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[102]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiskStats.ProtoReflect.Descriptor instead.
func (*DiskStats) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{102}
}

func (x *DiskStats) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *DiskStats) GetTotal() *DiskStat {
	if x != nil {
		return x.Total
	}
	return nil
}

func (x *DiskStats) GetDevices() []*DiskStat {
	if x != nil {
		return x.Devices
	}
	return nil
}

type DiskStat struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	Name             string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	ReadCompleted    uint64                 `protobuf:"varint,2,opt,name=read_completed,json=readCompleted,proto3" json:"read_completed,omitempty"`
	ReadMerged       uint64                 `protobuf:"varint,3,opt,name=read_merged,json=readMerged,proto3" json:"read_merged,omitempty"`
	ReadSectors      uint64                 `protobuf:"varint,4,opt,name=read_sectors,json=readSectors,proto3" json:"read_sectors,omitempty"`
	ReadTimeMs       uint64                 `protobuf:"varint,5,opt,name=read_time_ms,json=readTimeMs,proto3" json:"read_time_ms,omitempty"`
	WriteCompleted   uint64                 `protobuf:"varint,6,opt,name=write_completed,json=writeCompleted,proto3" json:"write_completed,omitempty"`
	WriteMerged      uint64                 `protobuf:"varint,7,opt,name=write_merged,json=writeMerged,proto3" json:"write_merged,omitempty"`
	WriteSectors     uint64                 `protobuf:"varint,8,opt,name=write_sectors,json=writeSectors,proto3" json:"write_sectors,omitempty"`
	WriteTimeMs      uint64                 `protobuf:"varint,9,opt,name=write_time_ms,json=writeTimeMs,proto3" json:"write_time_ms,omitempty"`
	IoInProgress     uint64                 `protobuf:"varint,10,opt,name=io_in_progress,json=ioInProgress,proto3" json:"io_in_progress,omitempty"`
	IoTimeMs         uint64                 `protobuf:"varint,11,opt,name=io_time_ms,json=ioTimeMs,proto3" json:"io_time_ms,omitempty"`
	IoTimeWeightedMs uint64                 `protobuf:"varint,12,opt,name=io_time_weighted_ms,json=ioTimeWeightedMs,proto3" json:"io_time_weighted_ms,omitempty"`
	DiscardCompleted uint64                 `protobuf:"varint,13,opt,name=discard_completed,json=discardCompleted,proto3" json:"discard_completed,omitempty"`
	DiscardMerged    uint64                 `protobuf:"varint,14,opt,name=discard_merged,json=discardMerged,proto3" json:"discard_merged,omitempty"`
	DiscardSectors   uint64                 `protobuf:"varint,15,opt,name=discard_sectors,json=discardSectors,proto3" json:"discard_sectors,omitempty"`
	DiscardTimeMs    uint64                 `protobuf:"varint,16,opt,name=discard_time_ms,json=discardTimeMs,proto3" json:"discard_time_ms,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *DiskStat) Reset() {
	*x = DiskStat{}
	mi := &file_machine_machine_proto_msgTypes[103]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DiskStat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DiskStat) ProtoMessage() {}

func (x *DiskStat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[103]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DiskStat.ProtoReflect.Descriptor instead.
func (*DiskStat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{103}
}

func (x *DiskStat) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *DiskStat) GetReadCompleted() uint64 {
	if x != nil {
		return x.ReadCompleted
	}
	return 0
}

func (x *DiskStat) GetReadMerged() uint64 {
	if x != nil {
		return x.ReadMerged
	}
	return 0
}

func (x *DiskStat) GetReadSectors() uint64 {
	if x != nil {
		return x.ReadSectors
	}
	return 0
}

func (x *DiskStat) GetReadTimeMs() uint64 {
	if x != nil {
		return x.ReadTimeMs
	}
	return 0
}

func (x *DiskStat) GetWriteCompleted() uint64 {
	if x != nil {
		return x.WriteCompleted
	}
	return 0
}

func (x *DiskStat) GetWriteMerged() uint64 {
	if x != nil {
		return x.WriteMerged
	}
	return 0
}

func (x *DiskStat) GetWriteSectors() uint64 {
	if x != nil {
		return x.WriteSectors
	}
	return 0
}

func (x *DiskStat) GetWriteTimeMs() uint64 {
	if x != nil {
		return x.WriteTimeMs
	}
	return 0
}

func (x *DiskStat) GetIoInProgress() uint64 {
	if x != nil {
		return x.IoInProgress
	}
	return 0
}

func (x *DiskStat) GetIoTimeMs() uint64 {
	if x != nil {
		return x.IoTimeMs
	}
	return 0
}

func (x *DiskStat) GetIoTimeWeightedMs() uint64 {
	if x != nil {
		return x.IoTimeWeightedMs
	}
	return 0
}

func (x *DiskStat) GetDiscardCompleted() uint64 {
	if x != nil {
		return x.DiscardCompleted
	}
	return 0
}

func (x *DiskStat) GetDiscardMerged() uint64 {
	if x != nil {
		return x.DiscardMerged
	}
	return 0
}

func (x *DiskStat) GetDiscardSectors() uint64 {
	if x != nil {
		return x.DiscardSectors
	}
	return 0
}

func (x *DiskStat) GetDiscardTimeMs() uint64 {
	if x != nil {
		return x.DiscardTimeMs
	}
	return 0
}

type EtcdLeaveClusterRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdLeaveClusterRequest) Reset() {
	*x = EtcdLeaveClusterRequest{}
	mi := &file_machine_machine_proto_msgTypes[104]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdLeaveClusterRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdLeaveClusterRequest) ProtoMessage() {}

func (x *EtcdLeaveClusterRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[104]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdLeaveClusterRequest.ProtoReflect.Descriptor instead.
func (*EtcdLeaveClusterRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{104}
}

type EtcdLeaveCluster struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdLeaveCluster) Reset() {
	*x = EtcdLeaveCluster{}
	mi := &file_machine_machine_proto_msgTypes[105]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdLeaveCluster) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdLeaveCluster) ProtoMessage() {}

func (x *EtcdLeaveCluster) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[105]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdLeaveCluster.ProtoReflect.Descriptor instead.
func (*EtcdLeaveCluster) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{105}
}

func (x *EtcdLeaveCluster) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EtcdLeaveClusterResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdLeaveCluster    `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdLeaveClusterResponse) Reset() {
	*x = EtcdLeaveClusterResponse{}
	mi := &file_machine_machine_proto_msgTypes[106]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdLeaveClusterResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdLeaveClusterResponse) ProtoMessage() {}

func (x *EtcdLeaveClusterResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[106]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdLeaveClusterResponse.ProtoReflect.Descriptor instead.
func (*EtcdLeaveClusterResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{106}
}

func (x *EtcdLeaveClusterResponse) GetMessages() []*EtcdLeaveCluster {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdRemoveMemberRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Member        string                 `protobuf:"bytes,1,opt,name=member,proto3" json:"member,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMemberRequest) Reset() {
	*x = EtcdRemoveMemberRequest{}
	mi := &file_machine_machine_proto_msgTypes[107]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMemberRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMemberRequest) ProtoMessage() {}

func (x *EtcdRemoveMemberRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[107]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMemberRequest.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMemberRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{107}
}

func (x *EtcdRemoveMemberRequest) GetMember() string {
	if x != nil {
		return x.Member
	}
	return ""
}

type EtcdRemoveMember struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMember) Reset() {
	*x = EtcdRemoveMember{}
	mi := &file_machine_machine_proto_msgTypes[108]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMember) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMember) ProtoMessage() {}

func (x *EtcdRemoveMember) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[108]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMember.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMember) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{108}
}

func (x *EtcdRemoveMember) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EtcdRemoveMemberResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdRemoveMember    `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMemberResponse) Reset() {
	*x = EtcdRemoveMemberResponse{}
	mi := &file_machine_machine_proto_msgTypes[109]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMemberResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMemberResponse) ProtoMessage() {}

func (x *EtcdRemoveMemberResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[109]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMemberResponse.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMemberResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{109}
}

func (x *EtcdRemoveMemberResponse) GetMessages() []*EtcdRemoveMember {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdRemoveMemberByIDRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	MemberId      uint64                 `protobuf:"varint,1,opt,name=member_id,json=memberId,proto3" json:"member_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMemberByIDRequest) Reset() {
	*x = EtcdRemoveMemberByIDRequest{}
	mi := &file_machine_machine_proto_msgTypes[110]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMemberByIDRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMemberByIDRequest) ProtoMessage() {}

func (x *EtcdRemoveMemberByIDRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[110]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMemberByIDRequest.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMemberByIDRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{110}
}

func (x *EtcdRemoveMemberByIDRequest) GetMemberId() uint64 {
	if x != nil {
		return x.MemberId
	}
	return 0
}

type EtcdRemoveMemberByID struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMemberByID) Reset() {
	*x = EtcdRemoveMemberByID{}
	mi := &file_machine_machine_proto_msgTypes[111]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMemberByID) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMemberByID) ProtoMessage() {}

func (x *EtcdRemoveMemberByID) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[111]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMemberByID.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMemberByID) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{111}
}

func (x *EtcdRemoveMemberByID) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EtcdRemoveMemberByIDResponse struct {
	state         protoimpl.MessageState  `protogen:"open.v1"`
	Messages      []*EtcdRemoveMemberByID `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRemoveMemberByIDResponse) Reset() {
	*x = EtcdRemoveMemberByIDResponse{}
	mi := &file_machine_machine_proto_msgTypes[112]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRemoveMemberByIDResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRemoveMemberByIDResponse) ProtoMessage() {}

func (x *EtcdRemoveMemberByIDResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[112]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRemoveMemberByIDResponse.ProtoReflect.Descriptor instead.
func (*EtcdRemoveMemberByIDResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{112}
}

func (x *EtcdRemoveMemberByIDResponse) GetMessages() []*EtcdRemoveMemberByID {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdForfeitLeadershipRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdForfeitLeadershipRequest) Reset() {
	*x = EtcdForfeitLeadershipRequest{}
	mi := &file_machine_machine_proto_msgTypes[113]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdForfeitLeadershipRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdForfeitLeadershipRequest) ProtoMessage() {}

func (x *EtcdForfeitLeadershipRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[113]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdForfeitLeadershipRequest.ProtoReflect.Descriptor instead.
func (*EtcdForfeitLeadershipRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{113}
}

type EtcdForfeitLeadership struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Member        string                 `protobuf:"bytes,2,opt,name=member,proto3" json:"member,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdForfeitLeadership) Reset() {
	*x = EtcdForfeitLeadership{}
	mi := &file_machine_machine_proto_msgTypes[114]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdForfeitLeadership) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdForfeitLeadership) ProtoMessage() {}

func (x *EtcdForfeitLeadership) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[114]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdForfeitLeadership.ProtoReflect.Descriptor instead.
func (*EtcdForfeitLeadership) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{114}
}

func (x *EtcdForfeitLeadership) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdForfeitLeadership) GetMember() string {
	if x != nil {
		return x.Member
	}
	return ""
}

type EtcdForfeitLeadershipResponse struct {
	state         protoimpl.MessageState   `protogen:"open.v1"`
	Messages      []*EtcdForfeitLeadership `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdForfeitLeadershipResponse) Reset() {
	*x = EtcdForfeitLeadershipResponse{}
	mi := &file_machine_machine_proto_msgTypes[115]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdForfeitLeadershipResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdForfeitLeadershipResponse) ProtoMessage() {}

func (x *EtcdForfeitLeadershipResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[115]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdForfeitLeadershipResponse.ProtoReflect.Descriptor instead.
func (*EtcdForfeitLeadershipResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{115}
}

func (x *EtcdForfeitLeadershipResponse) GetMessages() []*EtcdForfeitLeadership {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdMemberListRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	QueryLocal    bool                   `protobuf:"varint,1,opt,name=query_local,json=queryLocal,proto3" json:"query_local,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdMemberListRequest) Reset() {
	*x = EtcdMemberListRequest{}
	mi := &file_machine_machine_proto_msgTypes[116]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMemberListRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMemberListRequest) ProtoMessage() {}

func (x *EtcdMemberListRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[116]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMemberListRequest.ProtoReflect.Descriptor instead.
func (*EtcdMemberListRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{116}
}

func (x *EtcdMemberListRequest) GetQueryLocal() bool {
	if x != nil {
		return x.QueryLocal
	}
	return false
}

// EtcdMember describes a single etcd member.
type EtcdMember struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// member ID.
	Id uint64 `protobuf:"varint,2,opt,name=id,proto3" json:"id,omitempty"`
	// human-readable name of the member.
	Hostname string `protobuf:"bytes,3,opt,name=hostname,proto3" json:"hostname,omitempty"`
	// the list of URLs the member exposes to clients for communication.
	PeerUrls []string `protobuf:"bytes,4,rep,name=peer_urls,json=peerUrls,proto3" json:"peer_urls,omitempty"`
	// the list of URLs the member exposes to the cluster for communication.
	ClientUrls []string `protobuf:"bytes,5,rep,name=client_urls,json=clientUrls,proto3" json:"client_urls,omitempty"`
	// learner flag
	IsLearner     bool `protobuf:"varint,6,opt,name=is_learner,json=isLearner,proto3" json:"is_learner,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdMember) Reset() {
	*x = EtcdMember{}
	mi := &file_machine_machine_proto_msgTypes[117]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMember) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMember) ProtoMessage() {}

func (x *EtcdMember) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[117]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMember.ProtoReflect.Descriptor instead.
func (*EtcdMember) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{117}
}

func (x *EtcdMember) GetId() uint64 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *EtcdMember) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

func (x *EtcdMember) GetPeerUrls() []string {
	if x != nil {
		return x.PeerUrls
	}
	return nil
}

func (x *EtcdMember) GetClientUrls() []string {
	if x != nil {
		return x.ClientUrls
	}
	return nil
}

func (x *EtcdMember) GetIsLearner() bool {
	if x != nil {
		return x.IsLearner
	}
	return false
}

// EtcdMembers contains the list of members registered on the host.
type EtcdMembers struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// list of member hostnames.
	LegacyMembers []string `protobuf:"bytes,2,rep,name=legacy_members,json=legacyMembers,proto3" json:"legacy_members,omitempty"`
	// the list of etcd members registered on the node.
	Members       []*EtcdMember `protobuf:"bytes,3,rep,name=members,proto3" json:"members,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdMembers) Reset() {
	*x = EtcdMembers{}
	mi := &file_machine_machine_proto_msgTypes[118]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMembers) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMembers) ProtoMessage() {}

func (x *EtcdMembers) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[118]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMembers.ProtoReflect.Descriptor instead.
func (*EtcdMembers) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{118}
}

func (x *EtcdMembers) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdMembers) GetLegacyMembers() []string {
	if x != nil {
		return x.LegacyMembers
	}
	return nil
}

func (x *EtcdMembers) GetMembers() []*EtcdMember {
	if x != nil {
		return x.Members
	}
	return nil
}

type EtcdMemberListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdMembers         `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdMemberListResponse) Reset() {
	*x = EtcdMemberListResponse{}
	mi := &file_machine_machine_proto_msgTypes[119]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMemberListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMemberListResponse) ProtoMessage() {}

func (x *EtcdMemberListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[119]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMemberListResponse.ProtoReflect.Descriptor instead.
func (*EtcdMemberListResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{119}
}

func (x *EtcdMemberListResponse) GetMessages() []*EtcdMembers {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdSnapshotRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdSnapshotRequest) Reset() {
	*x = EtcdSnapshotRequest{}
	mi := &file_machine_machine_proto_msgTypes[120]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdSnapshotRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdSnapshotRequest) ProtoMessage() {}

func (x *EtcdSnapshotRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[120]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdSnapshotRequest.ProtoReflect.Descriptor instead.
func (*EtcdSnapshotRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{120}
}

type EtcdRecover struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRecover) Reset() {
	*x = EtcdRecover{}
	mi := &file_machine_machine_proto_msgTypes[121]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRecover) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRecover) ProtoMessage() {}

func (x *EtcdRecover) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[121]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRecover.ProtoReflect.Descriptor instead.
func (*EtcdRecover) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{121}
}

func (x *EtcdRecover) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EtcdRecoverResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdRecover         `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdRecoverResponse) Reset() {
	*x = EtcdRecoverResponse{}
	mi := &file_machine_machine_proto_msgTypes[122]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdRecoverResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdRecoverResponse) ProtoMessage() {}

func (x *EtcdRecoverResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[122]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdRecoverResponse.ProtoReflect.Descriptor instead.
func (*EtcdRecoverResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{122}
}

func (x *EtcdRecoverResponse) GetMessages() []*EtcdRecover {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdAlarmListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdAlarm           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdAlarmListResponse) Reset() {
	*x = EtcdAlarmListResponse{}
	mi := &file_machine_machine_proto_msgTypes[123]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdAlarmListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdAlarmListResponse) ProtoMessage() {}

func (x *EtcdAlarmListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[123]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdAlarmListResponse.ProtoReflect.Descriptor instead.
func (*EtcdAlarmListResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{123}
}

func (x *EtcdAlarmListResponse) GetMessages() []*EtcdAlarm {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdAlarm struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	MemberAlarms  []*EtcdMemberAlarm     `protobuf:"bytes,2,rep,name=member_alarms,json=memberAlarms,proto3" json:"member_alarms,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdAlarm) Reset() {
	*x = EtcdAlarm{}
	mi := &file_machine_machine_proto_msgTypes[124]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdAlarm) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdAlarm) ProtoMessage() {}

func (x *EtcdAlarm) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[124]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdAlarm.ProtoReflect.Descriptor instead.
func (*EtcdAlarm) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{124}
}

func (x *EtcdAlarm) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdAlarm) GetMemberAlarms() []*EtcdMemberAlarm {
	if x != nil {
		return x.MemberAlarms
	}
	return nil
}

type EtcdMemberAlarm struct {
	state         protoimpl.MessageState    `protogen:"open.v1"`
	MemberId      uint64                    `protobuf:"varint,1,opt,name=member_id,json=memberId,proto3" json:"member_id,omitempty"`
	Alarm         EtcdMemberAlarm_AlarmType `protobuf:"varint,2,opt,name=alarm,proto3,enum=machine.EtcdMemberAlarm_AlarmType" json:"alarm,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdMemberAlarm) Reset() {
	*x = EtcdMemberAlarm{}
	mi := &file_machine_machine_proto_msgTypes[125]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMemberAlarm) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMemberAlarm) ProtoMessage() {}

func (x *EtcdMemberAlarm) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[125]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMemberAlarm.ProtoReflect.Descriptor instead.
func (*EtcdMemberAlarm) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{125}
}

func (x *EtcdMemberAlarm) GetMemberId() uint64 {
	if x != nil {
		return x.MemberId
	}
	return 0
}

func (x *EtcdMemberAlarm) GetAlarm() EtcdMemberAlarm_AlarmType {
	if x != nil {
		return x.Alarm
	}
	return EtcdMemberAlarm_NONE
}

type EtcdAlarmDisarmResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdAlarmDisarm     `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdAlarmDisarmResponse) Reset() {
	*x = EtcdAlarmDisarmResponse{}
	mi := &file_machine_machine_proto_msgTypes[126]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdAlarmDisarmResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdAlarmDisarmResponse) ProtoMessage() {}

func (x *EtcdAlarmDisarmResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[126]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdAlarmDisarmResponse.ProtoReflect.Descriptor instead.
func (*EtcdAlarmDisarmResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{126}
}

func (x *EtcdAlarmDisarmResponse) GetMessages() []*EtcdAlarmDisarm {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdAlarmDisarm struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	MemberAlarms  []*EtcdMemberAlarm     `protobuf:"bytes,2,rep,name=member_alarms,json=memberAlarms,proto3" json:"member_alarms,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdAlarmDisarm) Reset() {
	*x = EtcdAlarmDisarm{}
	mi := &file_machine_machine_proto_msgTypes[127]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdAlarmDisarm) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdAlarmDisarm) ProtoMessage() {}

func (x *EtcdAlarmDisarm) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[127]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdAlarmDisarm.ProtoReflect.Descriptor instead.
func (*EtcdAlarmDisarm) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{127}
}

func (x *EtcdAlarmDisarm) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdAlarmDisarm) GetMemberAlarms() []*EtcdMemberAlarm {
	if x != nil {
		return x.MemberAlarms
	}
	return nil
}

type EtcdDefragmentResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdDefragment      `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDefragmentResponse) Reset() {
	*x = EtcdDefragmentResponse{}
	mi := &file_machine_machine_proto_msgTypes[128]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDefragmentResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDefragmentResponse) ProtoMessage() {}

func (x *EtcdDefragmentResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[128]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDefragmentResponse.ProtoReflect.Descriptor instead.
func (*EtcdDefragmentResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{128}
}

func (x *EtcdDefragmentResponse) GetMessages() []*EtcdDefragment {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdDefragment struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDefragment) Reset() {
	*x = EtcdDefragment{}
	mi := &file_machine_machine_proto_msgTypes[129]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDefragment) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDefragment) ProtoMessage() {}

func (x *EtcdDefragment) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[129]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDefragment.ProtoReflect.Descriptor instead.
func (*EtcdDefragment) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{129}
}

func (x *EtcdDefragment) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type EtcdStatusResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdStatus          `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdStatusResponse) Reset() {
	*x = EtcdStatusResponse{}
	mi := &file_machine_machine_proto_msgTypes[130]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdStatusResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdStatusResponse) ProtoMessage() {}

func (x *EtcdStatusResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[130]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdStatusResponse.ProtoReflect.Descriptor instead.
func (*EtcdStatusResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{130}
}

func (x *EtcdStatusResponse) GetMessages() []*EtcdStatus {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdStatus struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	MemberStatus  *EtcdMemberStatus      `protobuf:"bytes,2,opt,name=member_status,json=memberStatus,proto3" json:"member_status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdStatus) Reset() {
	*x = EtcdStatus{}
	mi := &file_machine_machine_proto_msgTypes[131]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdStatus) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdStatus) ProtoMessage() {}

func (x *EtcdStatus) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[131]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdStatus.ProtoReflect.Descriptor instead.
func (*EtcdStatus) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{131}
}

func (x *EtcdStatus) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdStatus) GetMemberStatus() *EtcdMemberStatus {
	if x != nil {
		return x.MemberStatus
	}
	return nil
}

type EtcdMemberStatus struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	StorageVersion   string                 `protobuf:"bytes,11,opt,name=storage_version,json=storageVersion,proto3" json:"storage_version,omitempty"`
	MemberId         uint64                 `protobuf:"varint,10,opt,name=member_id,json=memberId,proto3" json:"member_id,omitempty"`
	ProtocolVersion  string                 `protobuf:"bytes,1,opt,name=protocol_version,json=protocolVersion,proto3" json:"protocol_version,omitempty"`
	DbSize           int64                  `protobuf:"varint,2,opt,name=db_size,json=dbSize,proto3" json:"db_size,omitempty"`
	DbSizeInUse      int64                  `protobuf:"varint,3,opt,name=db_size_in_use,json=dbSizeInUse,proto3" json:"db_size_in_use,omitempty"`
	Leader           uint64                 `protobuf:"varint,4,opt,name=leader,proto3" json:"leader,omitempty"`
	RaftIndex        uint64                 `protobuf:"varint,5,opt,name=raft_index,json=raftIndex,proto3" json:"raft_index,omitempty"`
	RaftTerm         uint64                 `protobuf:"varint,6,opt,name=raft_term,json=raftTerm,proto3" json:"raft_term,omitempty"`
	RaftAppliedIndex uint64                 `protobuf:"varint,7,opt,name=raft_applied_index,json=raftAppliedIndex,proto3" json:"raft_applied_index,omitempty"`
	Errors           []string               `protobuf:"bytes,8,rep,name=errors,proto3" json:"errors,omitempty"`
	IsLearner        bool                   `protobuf:"varint,9,opt,name=is_learner,json=isLearner,proto3" json:"is_learner,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *EtcdMemberStatus) Reset() {
	*x = EtcdMemberStatus{}
	mi := &file_machine_machine_proto_msgTypes[132]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdMemberStatus) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdMemberStatus) ProtoMessage() {}

func (x *EtcdMemberStatus) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[132]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdMemberStatus.ProtoReflect.Descriptor instead.
func (*EtcdMemberStatus) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{132}
}

func (x *EtcdMemberStatus) GetStorageVersion() string {
	if x != nil {
		return x.StorageVersion
	}
	return ""
}

func (x *EtcdMemberStatus) GetMemberId() uint64 {
	if x != nil {
		return x.MemberId
	}
	return 0
}

func (x *EtcdMemberStatus) GetProtocolVersion() string {
	if x != nil {
		return x.ProtocolVersion
	}
	return ""
}

func (x *EtcdMemberStatus) GetDbSize() int64 {
	if x != nil {
		return x.DbSize
	}
	return 0
}

func (x *EtcdMemberStatus) GetDbSizeInUse() int64 {
	if x != nil {
		return x.DbSizeInUse
	}
	return 0
}

func (x *EtcdMemberStatus) GetLeader() uint64 {
	if x != nil {
		return x.Leader
	}
	return 0
}

func (x *EtcdMemberStatus) GetRaftIndex() uint64 {
	if x != nil {
		return x.RaftIndex
	}
	return 0
}

func (x *EtcdMemberStatus) GetRaftTerm() uint64 {
	if x != nil {
		return x.RaftTerm
	}
	return 0
}

func (x *EtcdMemberStatus) GetRaftAppliedIndex() uint64 {
	if x != nil {
		return x.RaftAppliedIndex
	}
	return 0
}

func (x *EtcdMemberStatus) GetErrors() []string {
	if x != nil {
		return x.Errors
	}
	return nil
}

func (x *EtcdMemberStatus) GetIsLearner() bool {
	if x != nil {
		return x.IsLearner
	}
	return false
}

type EtcdDowngradeValidateRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Version       string                 `protobuf:"bytes,1,opt,name=version,proto3" json:"version,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDowngradeValidateRequest) Reset() {
	*x = EtcdDowngradeValidateRequest{}
	mi := &file_machine_machine_proto_msgTypes[133]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeValidateRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeValidateRequest) ProtoMessage() {}

func (x *EtcdDowngradeValidateRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[133]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeValidateRequest.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeValidateRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{133}
}

func (x *EtcdDowngradeValidateRequest) GetVersion() string {
	if x != nil {
		return x.Version
	}
	return ""
}

type EtcdDowngradeValidateResponse struct {
	state         protoimpl.MessageState   `protogen:"open.v1"`
	Messages      []*EtcdDowngradeValidate `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDowngradeValidateResponse) Reset() {
	*x = EtcdDowngradeValidateResponse{}
	mi := &file_machine_machine_proto_msgTypes[134]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeValidateResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeValidateResponse) ProtoMessage() {}

func (x *EtcdDowngradeValidateResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[134]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeValidateResponse.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeValidateResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{134}
}

func (x *EtcdDowngradeValidateResponse) GetMessages() []*EtcdDowngradeValidate {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdDowngradeValidate struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	Metadata         *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ClusterDowngrade *EtcdClusterDowngrade  `protobuf:"bytes,2,opt,name=cluster_downgrade,json=clusterDowngrade,proto3" json:"cluster_downgrade,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *EtcdDowngradeValidate) Reset() {
	*x = EtcdDowngradeValidate{}
	mi := &file_machine_machine_proto_msgTypes[135]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeValidate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeValidate) ProtoMessage() {}

func (x *EtcdDowngradeValidate) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[135]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeValidate.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeValidate) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{135}
}

func (x *EtcdDowngradeValidate) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdDowngradeValidate) GetClusterDowngrade() *EtcdClusterDowngrade {
	if x != nil {
		return x.ClusterDowngrade
	}
	return nil
}

type EtcdDowngradeEnableRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Version       string                 `protobuf:"bytes,1,opt,name=version,proto3" json:"version,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDowngradeEnableRequest) Reset() {
	*x = EtcdDowngradeEnableRequest{}
	mi := &file_machine_machine_proto_msgTypes[136]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeEnableRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeEnableRequest) ProtoMessage() {}

func (x *EtcdDowngradeEnableRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[136]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeEnableRequest.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeEnableRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{136}
}

func (x *EtcdDowngradeEnableRequest) GetVersion() string {
	if x != nil {
		return x.Version
	}
	return ""
}

type EtcdDowngradeEnableResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdDowngradeEnable `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDowngradeEnableResponse) Reset() {
	*x = EtcdDowngradeEnableResponse{}
	mi := &file_machine_machine_proto_msgTypes[137]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeEnableResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeEnableResponse) ProtoMessage() {}

func (x *EtcdDowngradeEnableResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[137]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeEnableResponse.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeEnableResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{137}
}

func (x *EtcdDowngradeEnableResponse) GetMessages() []*EtcdDowngradeEnable {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdDowngradeEnable struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	Metadata         *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ClusterDowngrade *EtcdClusterDowngrade  `protobuf:"bytes,2,opt,name=cluster_downgrade,json=clusterDowngrade,proto3" json:"cluster_downgrade,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *EtcdDowngradeEnable) Reset() {
	*x = EtcdDowngradeEnable{}
	mi := &file_machine_machine_proto_msgTypes[138]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeEnable) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeEnable) ProtoMessage() {}

func (x *EtcdDowngradeEnable) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[138]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeEnable.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeEnable) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{138}
}

func (x *EtcdDowngradeEnable) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdDowngradeEnable) GetClusterDowngrade() *EtcdClusterDowngrade {
	if x != nil {
		return x.ClusterDowngrade
	}
	return nil
}

type EtcdDowngradeCancelResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*EtcdDowngradeCancel `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *EtcdDowngradeCancelResponse) Reset() {
	*x = EtcdDowngradeCancelResponse{}
	mi := &file_machine_machine_proto_msgTypes[139]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeCancelResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeCancelResponse) ProtoMessage() {}

func (x *EtcdDowngradeCancelResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[139]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeCancelResponse.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeCancelResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{139}
}

func (x *EtcdDowngradeCancelResponse) GetMessages() []*EtcdDowngradeCancel {
	if x != nil {
		return x.Messages
	}
	return nil
}

type EtcdDowngradeCancel struct {
	state            protoimpl.MessageState `protogen:"open.v1"`
	Metadata         *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	ClusterDowngrade *EtcdClusterDowngrade  `protobuf:"bytes,2,opt,name=cluster_downgrade,json=clusterDowngrade,proto3" json:"cluster_downgrade,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *EtcdDowngradeCancel) Reset() {
	*x = EtcdDowngradeCancel{}
	mi := &file_machine_machine_proto_msgTypes[140]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdDowngradeCancel) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdDowngradeCancel) ProtoMessage() {}

func (x *EtcdDowngradeCancel) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[140]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdDowngradeCancel.ProtoReflect.Descriptor instead.
func (*EtcdDowngradeCancel) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{140}
}

func (x *EtcdDowngradeCancel) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *EtcdDowngradeCancel) GetClusterDowngrade() *EtcdClusterDowngrade {
	if x != nil {
		return x.ClusterDowngrade
	}
	return nil
}

type EtcdClusterDowngrade struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	ClusterVersion string                 `protobuf:"bytes,1,opt,name=cluster_version,json=clusterVersion,proto3" json:"cluster_version,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *EtcdClusterDowngrade) Reset() {
	*x = EtcdClusterDowngrade{}
	mi := &file_machine_machine_proto_msgTypes[141]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *EtcdClusterDowngrade) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*EtcdClusterDowngrade) ProtoMessage() {}

func (x *EtcdClusterDowngrade) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[141]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use EtcdClusterDowngrade.ProtoReflect.Descriptor instead.
func (*EtcdClusterDowngrade) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{141}
}

func (x *EtcdClusterDowngrade) GetClusterVersion() string {
	if x != nil {
		return x.ClusterVersion
	}
	return ""
}

type RouteConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Network       string                 `protobuf:"bytes,1,opt,name=network,proto3" json:"network,omitempty"`
	Gateway       string                 `protobuf:"bytes,2,opt,name=gateway,proto3" json:"gateway,omitempty"`
	Metric        uint32                 `protobuf:"varint,3,opt,name=metric,proto3" json:"metric,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RouteConfig) Reset() {
	*x = RouteConfig{}
	mi := &file_machine_machine_proto_msgTypes[142]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RouteConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RouteConfig) ProtoMessage() {}

func (x *RouteConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[142]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RouteConfig.ProtoReflect.Descriptor instead.
func (*RouteConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{142}
}

func (x *RouteConfig) GetNetwork() string {
	if x != nil {
		return x.Network
	}
	return ""
}

func (x *RouteConfig) GetGateway() string {
	if x != nil {
		return x.Gateway
	}
	return ""
}

func (x *RouteConfig) GetMetric() uint32 {
	if x != nil {
		return x.Metric
	}
	return 0
}

type DHCPOptionsConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	RouteMetric   uint32                 `protobuf:"varint,1,opt,name=route_metric,json=routeMetric,proto3" json:"route_metric,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DHCPOptionsConfig) Reset() {
	*x = DHCPOptionsConfig{}
	mi := &file_machine_machine_proto_msgTypes[143]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DHCPOptionsConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DHCPOptionsConfig) ProtoMessage() {}

func (x *DHCPOptionsConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[143]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DHCPOptionsConfig.ProtoReflect.Descriptor instead.
func (*DHCPOptionsConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{143}
}

func (x *DHCPOptionsConfig) GetRouteMetric() uint32 {
	if x != nil {
		return x.RouteMetric
	}
	return 0
}

type NetworkDeviceConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Interface     string                 `protobuf:"bytes,1,opt,name=interface,proto3" json:"interface,omitempty"`
	Cidr          string                 `protobuf:"bytes,2,opt,name=cidr,proto3" json:"cidr,omitempty"`
	Mtu           int32                  `protobuf:"varint,3,opt,name=mtu,proto3" json:"mtu,omitempty"`
	Dhcp          bool                   `protobuf:"varint,4,opt,name=dhcp,proto3" json:"dhcp,omitempty"`
	Ignore        bool                   `protobuf:"varint,5,opt,name=ignore,proto3" json:"ignore,omitempty"`
	DhcpOptions   *DHCPOptionsConfig     `protobuf:"bytes,6,opt,name=dhcp_options,json=dhcpOptions,proto3" json:"dhcp_options,omitempty"`
	Routes        []*RouteConfig         `protobuf:"bytes,7,rep,name=routes,proto3" json:"routes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetworkDeviceConfig) Reset() {
	*x = NetworkDeviceConfig{}
	mi := &file_machine_machine_proto_msgTypes[144]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkDeviceConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkDeviceConfig) ProtoMessage() {}

func (x *NetworkDeviceConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[144]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkDeviceConfig.ProtoReflect.Descriptor instead.
func (*NetworkDeviceConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{144}
}

func (x *NetworkDeviceConfig) GetInterface() string {
	if x != nil {
		return x.Interface
	}
	return ""
}

func (x *NetworkDeviceConfig) GetCidr() string {
	if x != nil {
		return x.Cidr
	}
	return ""
}

func (x *NetworkDeviceConfig) GetMtu() int32 {
	if x != nil {
		return x.Mtu
	}
	return 0
}

func (x *NetworkDeviceConfig) GetDhcp() bool {
	if x != nil {
		return x.Dhcp
	}
	return false
}

func (x *NetworkDeviceConfig) GetIgnore() bool {
	if x != nil {
		return x.Ignore
	}
	return false
}

func (x *NetworkDeviceConfig) GetDhcpOptions() *DHCPOptionsConfig {
	if x != nil {
		return x.DhcpOptions
	}
	return nil
}

func (x *NetworkDeviceConfig) GetRoutes() []*RouteConfig {
	if x != nil {
		return x.Routes
	}
	return nil
}

type NetworkConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Hostname      string                 `protobuf:"bytes,1,opt,name=hostname,proto3" json:"hostname,omitempty"`
	Interfaces    []*NetworkDeviceConfig `protobuf:"bytes,2,rep,name=interfaces,proto3" json:"interfaces,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetworkConfig) Reset() {
	*x = NetworkConfig{}
	mi := &file_machine_machine_proto_msgTypes[145]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkConfig) ProtoMessage() {}

func (x *NetworkConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[145]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkConfig.ProtoReflect.Descriptor instead.
func (*NetworkConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{145}
}

func (x *NetworkConfig) GetHostname() string {
	if x != nil {
		return x.Hostname
	}
	return ""
}

func (x *NetworkConfig) GetInterfaces() []*NetworkDeviceConfig {
	if x != nil {
		return x.Interfaces
	}
	return nil
}

type InstallConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	InstallDisk   string                 `protobuf:"bytes,1,opt,name=install_disk,json=installDisk,proto3" json:"install_disk,omitempty"`
	InstallImage  string                 `protobuf:"bytes,2,opt,name=install_image,json=installImage,proto3" json:"install_image,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *InstallConfig) Reset() {
	*x = InstallConfig{}
	mi := &file_machine_machine_proto_msgTypes[146]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *InstallConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*InstallConfig) ProtoMessage() {}

func (x *InstallConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[146]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use InstallConfig.ProtoReflect.Descriptor instead.
func (*InstallConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{146}
}

func (x *InstallConfig) GetInstallDisk() string {
	if x != nil {
		return x.InstallDisk
	}
	return ""
}

func (x *InstallConfig) GetInstallImage() string {
	if x != nil {
		return x.InstallImage
	}
	return ""
}

type MachineConfig struct {
	state             protoimpl.MessageState    `protogen:"open.v1"`
	Type              MachineConfig_MachineType `protobuf:"varint,1,opt,name=type,proto3,enum=machine.MachineConfig_MachineType" json:"type,omitempty"`
	InstallConfig     *InstallConfig            `protobuf:"bytes,2,opt,name=install_config,json=installConfig,proto3" json:"install_config,omitempty"`
	NetworkConfig     *NetworkConfig            `protobuf:"bytes,3,opt,name=network_config,json=networkConfig,proto3" json:"network_config,omitempty"`
	KubernetesVersion string                    `protobuf:"bytes,4,opt,name=kubernetes_version,json=kubernetesVersion,proto3" json:"kubernetes_version,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *MachineConfig) Reset() {
	*x = MachineConfig{}
	mi := &file_machine_machine_proto_msgTypes[147]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineConfig) ProtoMessage() {}

func (x *MachineConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[147]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineConfig.ProtoReflect.Descriptor instead.
func (*MachineConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{147}
}

func (x *MachineConfig) GetType() MachineConfig_MachineType {
	if x != nil {
		return x.Type
	}
	return MachineConfig_TYPE_UNKNOWN
}

func (x *MachineConfig) GetInstallConfig() *InstallConfig {
	if x != nil {
		return x.InstallConfig
	}
	return nil
}

func (x *MachineConfig) GetNetworkConfig() *NetworkConfig {
	if x != nil {
		return x.NetworkConfig
	}
	return nil
}

func (x *MachineConfig) GetKubernetesVersion() string {
	if x != nil {
		return x.KubernetesVersion
	}
	return ""
}

type ControlPlaneConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Endpoint      string                 `protobuf:"bytes,1,opt,name=endpoint,proto3" json:"endpoint,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ControlPlaneConfig) Reset() {
	*x = ControlPlaneConfig{}
	mi := &file_machine_machine_proto_msgTypes[148]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ControlPlaneConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ControlPlaneConfig) ProtoMessage() {}

func (x *ControlPlaneConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[148]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ControlPlaneConfig.ProtoReflect.Descriptor instead.
func (*ControlPlaneConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{148}
}

func (x *ControlPlaneConfig) GetEndpoint() string {
	if x != nil {
		return x.Endpoint
	}
	return ""
}

type CNIConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Urls          []string               `protobuf:"bytes,2,rep,name=urls,proto3" json:"urls,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CNIConfig) Reset() {
	*x = CNIConfig{}
	mi := &file_machine_machine_proto_msgTypes[149]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CNIConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CNIConfig) ProtoMessage() {}

func (x *CNIConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[149]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CNIConfig.ProtoReflect.Descriptor instead.
func (*CNIConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{149}
}

func (x *CNIConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *CNIConfig) GetUrls() []string {
	if x != nil {
		return x.Urls
	}
	return nil
}

type ClusterNetworkConfig struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	DnsDomain     string                 `protobuf:"bytes,1,opt,name=dns_domain,json=dnsDomain,proto3" json:"dns_domain,omitempty"`
	CniConfig     *CNIConfig             `protobuf:"bytes,2,opt,name=cni_config,json=cniConfig,proto3" json:"cni_config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ClusterNetworkConfig) Reset() {
	*x = ClusterNetworkConfig{}
	mi := &file_machine_machine_proto_msgTypes[150]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterNetworkConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterNetworkConfig) ProtoMessage() {}

func (x *ClusterNetworkConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[150]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterNetworkConfig.ProtoReflect.Descriptor instead.
func (*ClusterNetworkConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{150}
}

func (x *ClusterNetworkConfig) GetDnsDomain() string {
	if x != nil {
		return x.DnsDomain
	}
	return ""
}

func (x *ClusterNetworkConfig) GetCniConfig() *CNIConfig {
	if x != nil {
		return x.CniConfig
	}
	return nil
}

type ClusterConfig struct {
	state                          protoimpl.MessageState `protogen:"open.v1"`
	Name                           string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	ControlPlane                   *ControlPlaneConfig    `protobuf:"bytes,2,opt,name=control_plane,json=controlPlane,proto3" json:"control_plane,omitempty"`
	ClusterNetwork                 *ClusterNetworkConfig  `protobuf:"bytes,3,opt,name=cluster_network,json=clusterNetwork,proto3" json:"cluster_network,omitempty"`
	AllowSchedulingOnControlPlanes bool                   `protobuf:"varint,4,opt,name=allow_scheduling_on_control_planes,json=allowSchedulingOnControlPlanes,proto3" json:"allow_scheduling_on_control_planes,omitempty"`
	unknownFields                  protoimpl.UnknownFields
	sizeCache                      protoimpl.SizeCache
}

func (x *ClusterConfig) Reset() {
	*x = ClusterConfig{}
	mi := &file_machine_machine_proto_msgTypes[151]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ClusterConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ClusterConfig) ProtoMessage() {}

func (x *ClusterConfig) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[151]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ClusterConfig.ProtoReflect.Descriptor instead.
func (*ClusterConfig) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{151}
}

func (x *ClusterConfig) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ClusterConfig) GetControlPlane() *ControlPlaneConfig {
	if x != nil {
		return x.ControlPlane
	}
	return nil
}

func (x *ClusterConfig) GetClusterNetwork() *ClusterNetworkConfig {
	if x != nil {
		return x.ClusterNetwork
	}
	return nil
}

func (x *ClusterConfig) GetAllowSchedulingOnControlPlanes() bool {
	if x != nil {
		return x.AllowSchedulingOnControlPlanes
	}
	return false
}

type GenerateClientConfigurationRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Roles in the generated client certificate.
	Roles []string `protobuf:"bytes,1,rep,name=roles,proto3" json:"roles,omitempty"`
	// Client certificate TTL.
	CrtTtl        *durationpb.Duration `protobuf:"bytes,2,opt,name=crt_ttl,json=crtTtl,proto3" json:"crt_ttl,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GenerateClientConfigurationRequest) Reset() {
	*x = GenerateClientConfigurationRequest{}
	mi := &file_machine_machine_proto_msgTypes[152]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GenerateClientConfigurationRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GenerateClientConfigurationRequest) ProtoMessage() {}

func (x *GenerateClientConfigurationRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[152]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GenerateClientConfigurationRequest.ProtoReflect.Descriptor instead.
func (*GenerateClientConfigurationRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{152}
}

func (x *GenerateClientConfigurationRequest) GetRoles() []string {
	if x != nil {
		return x.Roles
	}
	return nil
}

func (x *GenerateClientConfigurationRequest) GetCrtTtl() *durationpb.Duration {
	if x != nil {
		return x.CrtTtl
	}
	return nil
}

type GenerateClientConfiguration struct {
	state    protoimpl.MessageState `protogen:"open.v1"`
	Metadata *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	// PEM-encoded CA certificate.
	Ca []byte `protobuf:"bytes,2,opt,name=ca,proto3" json:"ca,omitempty"`
	// PEM-encoded generated client certificate.
	Crt []byte `protobuf:"bytes,3,opt,name=crt,proto3" json:"crt,omitempty"`
	// PEM-encoded generated client key.
	Key []byte `protobuf:"bytes,4,opt,name=key,proto3" json:"key,omitempty"`
	// Client configuration (talosconfig) file content.
	Talosconfig   []byte `protobuf:"bytes,5,opt,name=talosconfig,proto3" json:"talosconfig,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GenerateClientConfiguration) Reset() {
	*x = GenerateClientConfiguration{}
	mi := &file_machine_machine_proto_msgTypes[153]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GenerateClientConfiguration) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GenerateClientConfiguration) ProtoMessage() {}

func (x *GenerateClientConfiguration) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[153]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GenerateClientConfiguration.ProtoReflect.Descriptor instead.
func (*GenerateClientConfiguration) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{153}
}

func (x *GenerateClientConfiguration) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *GenerateClientConfiguration) GetCa() []byte {
	if x != nil {
		return x.Ca
	}
	return nil
}

func (x *GenerateClientConfiguration) GetCrt() []byte {
	if x != nil {
		return x.Crt
	}
	return nil
}

func (x *GenerateClientConfiguration) GetKey() []byte {
	if x != nil {
		return x.Key
	}
	return nil
}

func (x *GenerateClientConfiguration) GetTalosconfig() []byte {
	if x != nil {
		return x.Talosconfig
	}
	return nil
}

type GenerateClientConfigurationResponse struct {
	state         protoimpl.MessageState         `protogen:"open.v1"`
	Messages      []*GenerateClientConfiguration `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GenerateClientConfigurationResponse) Reset() {
	*x = GenerateClientConfigurationResponse{}
	mi := &file_machine_machine_proto_msgTypes[154]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GenerateClientConfigurationResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GenerateClientConfigurationResponse) ProtoMessage() {}

func (x *GenerateClientConfigurationResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[154]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GenerateClientConfigurationResponse.ProtoReflect.Descriptor instead.
func (*GenerateClientConfigurationResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{154}
}

func (x *GenerateClientConfigurationResponse) GetMessages() []*GenerateClientConfiguration {
	if x != nil {
		return x.Messages
	}
	return nil
}

type PacketCaptureRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Interface name to perform packet capture on.
	Interface string `protobuf:"bytes,1,opt,name=interface,proto3" json:"interface,omitempty"`
	// Enable promiscuous mode.
	Promiscuous bool `protobuf:"varint,2,opt,name=promiscuous,proto3" json:"promiscuous,omitempty"`
	// Snap length in bytes.
	SnapLen uint32 `protobuf:"varint,3,opt,name=snap_len,json=snapLen,proto3" json:"snap_len,omitempty"`
	// BPF filter.
	BpfFilter     []*BPFInstruction `protobuf:"bytes,4,rep,name=bpf_filter,json=bpfFilter,proto3" json:"bpf_filter,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PacketCaptureRequest) Reset() {
	*x = PacketCaptureRequest{}
	mi := &file_machine_machine_proto_msgTypes[155]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PacketCaptureRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PacketCaptureRequest) ProtoMessage() {}

func (x *PacketCaptureRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[155]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PacketCaptureRequest.ProtoReflect.Descriptor instead.
func (*PacketCaptureRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{155}
}

func (x *PacketCaptureRequest) GetInterface() string {
	if x != nil {
		return x.Interface
	}
	return ""
}

func (x *PacketCaptureRequest) GetPromiscuous() bool {
	if x != nil {
		return x.Promiscuous
	}
	return false
}

func (x *PacketCaptureRequest) GetSnapLen() uint32 {
	if x != nil {
		return x.SnapLen
	}
	return 0
}

func (x *PacketCaptureRequest) GetBpfFilter() []*BPFInstruction {
	if x != nil {
		return x.BpfFilter
	}
	return nil
}

type BPFInstruction struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Op            uint32                 `protobuf:"varint,1,opt,name=op,proto3" json:"op,omitempty"`
	Jt            uint32                 `protobuf:"varint,2,opt,name=jt,proto3" json:"jt,omitempty"`
	Jf            uint32                 `protobuf:"varint,3,opt,name=jf,proto3" json:"jf,omitempty"`
	K             uint32                 `protobuf:"varint,4,opt,name=k,proto3" json:"k,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BPFInstruction) Reset() {
	*x = BPFInstruction{}
	mi := &file_machine_machine_proto_msgTypes[156]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BPFInstruction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BPFInstruction) ProtoMessage() {}

func (x *BPFInstruction) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[156]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BPFInstruction.ProtoReflect.Descriptor instead.
func (*BPFInstruction) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{156}
}

func (x *BPFInstruction) GetOp() uint32 {
	if x != nil {
		return x.Op
	}
	return 0
}

func (x *BPFInstruction) GetJt() uint32 {
	if x != nil {
		return x.Jt
	}
	return 0
}

func (x *BPFInstruction) GetJf() uint32 {
	if x != nil {
		return x.Jf
	}
	return 0
}

func (x *BPFInstruction) GetK() uint32 {
	if x != nil {
		return x.K
	}
	return 0
}

type NetstatRequest struct {
	state         protoimpl.MessageState  `protogen:"open.v1"`
	Filter        NetstatRequest_Filter   `protobuf:"varint,1,opt,name=filter,proto3,enum=machine.NetstatRequest_Filter" json:"filter,omitempty"`
	Feature       *NetstatRequest_Feature `protobuf:"bytes,2,opt,name=feature,proto3" json:"feature,omitempty"`
	L4Proto       *NetstatRequest_L4Proto `protobuf:"bytes,3,opt,name=l4proto,proto3" json:"l4proto,omitempty"`
	Netns         *NetstatRequest_NetNS   `protobuf:"bytes,4,opt,name=netns,proto3" json:"netns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetstatRequest) Reset() {
	*x = NetstatRequest{}
	mi := &file_machine_machine_proto_msgTypes[157]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetstatRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetstatRequest) ProtoMessage() {}

func (x *NetstatRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[157]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetstatRequest.ProtoReflect.Descriptor instead.
func (*NetstatRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{157}
}

func (x *NetstatRequest) GetFilter() NetstatRequest_Filter {
	if x != nil {
		return x.Filter
	}
	return NetstatRequest_ALL
}

func (x *NetstatRequest) GetFeature() *NetstatRequest_Feature {
	if x != nil {
		return x.Feature
	}
	return nil
}

func (x *NetstatRequest) GetL4Proto() *NetstatRequest_L4Proto {
	if x != nil {
		return x.L4Proto
	}
	return nil
}

func (x *NetstatRequest) GetNetns() *NetstatRequest_NetNS {
	if x != nil {
		return x.Netns
	}
	return nil
}

type ConnectRecord struct {
	state         protoimpl.MessageState    `protogen:"open.v1"`
	L4Proto       string                    `protobuf:"bytes,1,opt,name=l4proto,proto3" json:"l4proto,omitempty"`
	Localip       string                    `protobuf:"bytes,2,opt,name=localip,proto3" json:"localip,omitempty"`
	Localport     uint32                    `protobuf:"varint,3,opt,name=localport,proto3" json:"localport,omitempty"`
	Remoteip      string                    `protobuf:"bytes,4,opt,name=remoteip,proto3" json:"remoteip,omitempty"`
	Remoteport    uint32                    `protobuf:"varint,5,opt,name=remoteport,proto3" json:"remoteport,omitempty"`
	State         ConnectRecord_State       `protobuf:"varint,6,opt,name=state,proto3,enum=machine.ConnectRecord_State" json:"state,omitempty"`
	Txqueue       uint64                    `protobuf:"varint,7,opt,name=txqueue,proto3" json:"txqueue,omitempty"`
	Rxqueue       uint64                    `protobuf:"varint,8,opt,name=rxqueue,proto3" json:"rxqueue,omitempty"`
	Tr            ConnectRecord_TimerActive `protobuf:"varint,9,opt,name=tr,proto3,enum=machine.ConnectRecord_TimerActive" json:"tr,omitempty"`
	Timerwhen     uint64                    `protobuf:"varint,10,opt,name=timerwhen,proto3" json:"timerwhen,omitempty"`
	Retrnsmt      uint64                    `protobuf:"varint,11,opt,name=retrnsmt,proto3" json:"retrnsmt,omitempty"`
	Uid           uint32                    `protobuf:"varint,12,opt,name=uid,proto3" json:"uid,omitempty"`
	Timeout       uint64                    `protobuf:"varint,13,opt,name=timeout,proto3" json:"timeout,omitempty"`
	Inode         uint64                    `protobuf:"varint,14,opt,name=inode,proto3" json:"inode,omitempty"`
	Ref           uint64                    `protobuf:"varint,15,opt,name=ref,proto3" json:"ref,omitempty"`
	Pointer       uint64                    `protobuf:"varint,16,opt,name=pointer,proto3" json:"pointer,omitempty"`
	Process       *ConnectRecord_Process    `protobuf:"bytes,17,opt,name=process,proto3" json:"process,omitempty"`
	Netns         string                    `protobuf:"bytes,18,opt,name=netns,proto3" json:"netns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConnectRecord) Reset() {
	*x = ConnectRecord{}
	mi := &file_machine_machine_proto_msgTypes[158]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConnectRecord) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConnectRecord) ProtoMessage() {}

func (x *ConnectRecord) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[158]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConnectRecord.ProtoReflect.Descriptor instead.
func (*ConnectRecord) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{158}
}

func (x *ConnectRecord) GetL4Proto() string {
	if x != nil {
		return x.L4Proto
	}
	return ""
}

func (x *ConnectRecord) GetLocalip() string {
	if x != nil {
		return x.Localip
	}
	return ""
}

func (x *ConnectRecord) GetLocalport() uint32 {
	if x != nil {
		return x.Localport
	}
	return 0
}

func (x *ConnectRecord) GetRemoteip() string {
	if x != nil {
		return x.Remoteip
	}
	return ""
}

func (x *ConnectRecord) GetRemoteport() uint32 {
	if x != nil {
		return x.Remoteport
	}
	return 0
}

func (x *ConnectRecord) GetState() ConnectRecord_State {
	if x != nil {
		return x.State
	}
	return ConnectRecord_RESERVED
}

func (x *ConnectRecord) GetTxqueue() uint64 {
	if x != nil {
		return x.Txqueue
	}
	return 0
}

func (x *ConnectRecord) GetRxqueue() uint64 {
	if x != nil {
		return x.Rxqueue
	}
	return 0
}

func (x *ConnectRecord) GetTr() ConnectRecord_TimerActive {
	if x != nil {
		return x.Tr
	}
	return ConnectRecord_OFF
}

func (x *ConnectRecord) GetTimerwhen() uint64 {
	if x != nil {
		return x.Timerwhen
	}
	return 0
}

func (x *ConnectRecord) GetRetrnsmt() uint64 {
	if x != nil {
		return x.Retrnsmt
	}
	return 0
}

func (x *ConnectRecord) GetUid() uint32 {
	if x != nil {
		return x.Uid
	}
	return 0
}

func (x *ConnectRecord) GetTimeout() uint64 {
	if x != nil {
		return x.Timeout
	}
	return 0
}

func (x *ConnectRecord) GetInode() uint64 {
	if x != nil {
		return x.Inode
	}
	return 0
}

func (x *ConnectRecord) GetRef() uint64 {
	if x != nil {
		return x.Ref
	}
	return 0
}

func (x *ConnectRecord) GetPointer() uint64 {
	if x != nil {
		return x.Pointer
	}
	return 0
}

func (x *ConnectRecord) GetProcess() *ConnectRecord_Process {
	if x != nil {
		return x.Process
	}
	return nil
}

func (x *ConnectRecord) GetNetns() string {
	if x != nil {
		return x.Netns
	}
	return ""
}

type Netstat struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Connectrecord []*ConnectRecord       `protobuf:"bytes,2,rep,name=connectrecord,proto3" json:"connectrecord,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Netstat) Reset() {
	*x = Netstat{}
	mi := &file_machine_machine_proto_msgTypes[159]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Netstat) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Netstat) ProtoMessage() {}

func (x *Netstat) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[159]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Netstat.ProtoReflect.Descriptor instead.
func (*Netstat) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{159}
}

func (x *Netstat) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *Netstat) GetConnectrecord() []*ConnectRecord {
	if x != nil {
		return x.Connectrecord
	}
	return nil
}

type NetstatResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*Netstat             `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetstatResponse) Reset() {
	*x = NetstatResponse{}
	mi := &file_machine_machine_proto_msgTypes[160]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetstatResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetstatResponse) ProtoMessage() {}

func (x *NetstatResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[160]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetstatResponse.ProtoReflect.Descriptor instead.
func (*NetstatResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{160}
}

func (x *NetstatResponse) GetMessages() []*Netstat {
	if x != nil {
		return x.Messages
	}
	return nil
}

type MetaWriteRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Key           uint32                 `protobuf:"varint,1,opt,name=key,proto3" json:"key,omitempty"`
	Value         []byte                 `protobuf:"bytes,2,opt,name=value,proto3" json:"value,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaWriteRequest) Reset() {
	*x = MetaWriteRequest{}
	mi := &file_machine_machine_proto_msgTypes[161]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaWriteRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaWriteRequest) ProtoMessage() {}

func (x *MetaWriteRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[161]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaWriteRequest.ProtoReflect.Descriptor instead.
func (*MetaWriteRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{161}
}

func (x *MetaWriteRequest) GetKey() uint32 {
	if x != nil {
		return x.Key
	}
	return 0
}

func (x *MetaWriteRequest) GetValue() []byte {
	if x != nil {
		return x.Value
	}
	return nil
}

type MetaWrite struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaWrite) Reset() {
	*x = MetaWrite{}
	mi := &file_machine_machine_proto_msgTypes[162]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaWrite) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaWrite) ProtoMessage() {}

func (x *MetaWrite) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[162]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaWrite.ProtoReflect.Descriptor instead.
func (*MetaWrite) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{162}
}

func (x *MetaWrite) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type MetaWriteResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*MetaWrite           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaWriteResponse) Reset() {
	*x = MetaWriteResponse{}
	mi := &file_machine_machine_proto_msgTypes[163]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaWriteResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaWriteResponse) ProtoMessage() {}

func (x *MetaWriteResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[163]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaWriteResponse.ProtoReflect.Descriptor instead.
func (*MetaWriteResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{163}
}

func (x *MetaWriteResponse) GetMessages() []*MetaWrite {
	if x != nil {
		return x.Messages
	}
	return nil
}

type MetaDeleteRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Key           uint32                 `protobuf:"varint,1,opt,name=key,proto3" json:"key,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaDeleteRequest) Reset() {
	*x = MetaDeleteRequest{}
	mi := &file_machine_machine_proto_msgTypes[164]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaDeleteRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaDeleteRequest) ProtoMessage() {}

func (x *MetaDeleteRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[164]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaDeleteRequest.ProtoReflect.Descriptor instead.
func (*MetaDeleteRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{164}
}

func (x *MetaDeleteRequest) GetKey() uint32 {
	if x != nil {
		return x.Key
	}
	return 0
}

type MetaDelete struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaDelete) Reset() {
	*x = MetaDelete{}
	mi := &file_machine_machine_proto_msgTypes[165]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaDelete) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaDelete) ProtoMessage() {}

func (x *MetaDelete) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[165]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaDelete.ProtoReflect.Descriptor instead.
func (*MetaDelete) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{165}
}

func (x *MetaDelete) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type MetaDeleteResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*MetaDelete          `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MetaDeleteResponse) Reset() {
	*x = MetaDeleteResponse{}
	mi := &file_machine_machine_proto_msgTypes[166]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MetaDeleteResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MetaDeleteResponse) ProtoMessage() {}

func (x *MetaDeleteResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[166]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MetaDeleteResponse.ProtoReflect.Descriptor instead.
func (*MetaDeleteResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{166}
}

func (x *MetaDeleteResponse) GetMessages() []*MetaDelete {
	if x != nil {
		return x.Messages
	}
	return nil
}

type ImageListRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Containerd namespace to use.
	Namespace     common.ContainerdNamespace `protobuf:"varint,1,opt,name=namespace,proto3,enum=common.ContainerdNamespace" json:"namespace,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageListRequest) Reset() {
	*x = ImageListRequest{}
	mi := &file_machine_machine_proto_msgTypes[167]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageListRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageListRequest) ProtoMessage() {}

func (x *ImageListRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[167]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageListRequest.ProtoReflect.Descriptor instead.
func (*ImageListRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{167}
}

func (x *ImageListRequest) GetNamespace() common.ContainerdNamespace {
	if x != nil {
		return x.Namespace
	}
	return common.ContainerdNamespace(0)
}

type ImageListResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	Name          string                 `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	Digest        string                 `protobuf:"bytes,3,opt,name=digest,proto3" json:"digest,omitempty"`
	Size          int64                  `protobuf:"varint,4,opt,name=size,proto3" json:"size,omitempty"`
	CreatedAt     *timestamppb.Timestamp `protobuf:"bytes,5,opt,name=created_at,json=createdAt,proto3" json:"created_at,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImageListResponse) Reset() {
	*x = ImageListResponse{}
	mi := &file_machine_machine_proto_msgTypes[168]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImageListResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImageListResponse) ProtoMessage() {}

func (x *ImageListResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[168]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImageListResponse.ProtoReflect.Descriptor instead.
func (*ImageListResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{168}
}

func (x *ImageListResponse) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

func (x *ImageListResponse) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ImageListResponse) GetDigest() string {
	if x != nil {
		return x.Digest
	}
	return ""
}

func (x *ImageListResponse) GetSize() int64 {
	if x != nil {
		return x.Size
	}
	return 0
}

func (x *ImageListResponse) GetCreatedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.CreatedAt
	}
	return nil
}

type ImagePullRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Containerd namespace to use.
	Namespace common.ContainerdNamespace `protobuf:"varint,1,opt,name=namespace,proto3,enum=common.ContainerdNamespace" json:"namespace,omitempty"`
	// Image reference to pull.
	Reference     string `protobuf:"bytes,2,opt,name=reference,proto3" json:"reference,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImagePullRequest) Reset() {
	*x = ImagePullRequest{}
	mi := &file_machine_machine_proto_msgTypes[169]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImagePullRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImagePullRequest) ProtoMessage() {}

func (x *ImagePullRequest) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[169]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImagePullRequest.ProtoReflect.Descriptor instead.
func (*ImagePullRequest) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{169}
}

func (x *ImagePullRequest) GetNamespace() common.ContainerdNamespace {
	if x != nil {
		return x.Namespace
	}
	return common.ContainerdNamespace(0)
}

func (x *ImagePullRequest) GetReference() string {
	if x != nil {
		return x.Reference
	}
	return ""
}

type ImagePull struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Metadata      *common.Metadata       `protobuf:"bytes,1,opt,name=metadata,proto3" json:"metadata,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImagePull) Reset() {
	*x = ImagePull{}
	mi := &file_machine_machine_proto_msgTypes[170]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImagePull) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImagePull) ProtoMessage() {}

func (x *ImagePull) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[170]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImagePull.ProtoReflect.Descriptor instead.
func (*ImagePull) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{170}
}

func (x *ImagePull) GetMetadata() *common.Metadata {
	if x != nil {
		return x.Metadata
	}
	return nil
}

type ImagePullResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Messages      []*ImagePull           `protobuf:"bytes,1,rep,name=messages,proto3" json:"messages,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ImagePullResponse) Reset() {
	*x = ImagePullResponse{}
	mi := &file_machine_machine_proto_msgTypes[171]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ImagePullResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ImagePullResponse) ProtoMessage() {}

func (x *ImagePullResponse) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[171]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ImagePullResponse.ProtoReflect.Descriptor instead.
func (*ImagePullResponse) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{171}
}

func (x *ImagePullResponse) GetMessages() []*ImagePull {
	if x != nil {
		return x.Messages
	}
	return nil
}

type MachineStatusEvent_MachineStatus struct {
	state           protoimpl.MessageState                             `protogen:"open.v1"`
	Ready           bool                                               `protobuf:"varint,1,opt,name=ready,proto3" json:"ready,omitempty"`
	UnmetConditions []*MachineStatusEvent_MachineStatus_UnmetCondition `protobuf:"bytes,2,rep,name=unmet_conditions,json=unmetConditions,proto3" json:"unmet_conditions,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *MachineStatusEvent_MachineStatus) Reset() {
	*x = MachineStatusEvent_MachineStatus{}
	mi := &file_machine_machine_proto_msgTypes[172]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineStatusEvent_MachineStatus) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineStatusEvent_MachineStatus) ProtoMessage() {}

func (x *MachineStatusEvent_MachineStatus) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[172]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineStatusEvent_MachineStatus.ProtoReflect.Descriptor instead.
func (*MachineStatusEvent_MachineStatus) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{17, 0}
}

func (x *MachineStatusEvent_MachineStatus) GetReady() bool {
	if x != nil {
		return x.Ready
	}
	return false
}

func (x *MachineStatusEvent_MachineStatus) GetUnmetConditions() []*MachineStatusEvent_MachineStatus_UnmetCondition {
	if x != nil {
		return x.UnmetConditions
	}
	return nil
}

type MachineStatusEvent_MachineStatus_UnmetCondition struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Name          string                 `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Reason        string                 `protobuf:"bytes,2,opt,name=reason,proto3" json:"reason,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MachineStatusEvent_MachineStatus_UnmetCondition) Reset() {
	*x = MachineStatusEvent_MachineStatus_UnmetCondition{}
	mi := &file_machine_machine_proto_msgTypes[173]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MachineStatusEvent_MachineStatus_UnmetCondition) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MachineStatusEvent_MachineStatus_UnmetCondition) ProtoMessage() {}

func (x *MachineStatusEvent_MachineStatus_UnmetCondition) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[173]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MachineStatusEvent_MachineStatus_UnmetCondition.ProtoReflect.Descriptor instead.
func (*MachineStatusEvent_MachineStatus_UnmetCondition) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{17, 0, 0}
}

func (x *MachineStatusEvent_MachineStatus_UnmetCondition) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *MachineStatusEvent_MachineStatus_UnmetCondition) GetReason() string {
	if x != nil {
		return x.Reason
	}
	return ""
}

type NetstatRequest_Feature struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Pid           bool                   `protobuf:"varint,1,opt,name=pid,proto3" json:"pid,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetstatRequest_Feature) Reset() {
	*x = NetstatRequest_Feature{}
	mi := &file_machine_machine_proto_msgTypes[174]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetstatRequest_Feature) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetstatRequest_Feature) ProtoMessage() {}

func (x *NetstatRequest_Feature) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[174]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetstatRequest_Feature.ProtoReflect.Descriptor instead.
func (*NetstatRequest_Feature) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{157, 0}
}

func (x *NetstatRequest_Feature) GetPid() bool {
	if x != nil {
		return x.Pid
	}
	return false
}

type NetstatRequest_L4Proto struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Tcp           bool                   `protobuf:"varint,1,opt,name=tcp,proto3" json:"tcp,omitempty"`
	Tcp6          bool                   `protobuf:"varint,2,opt,name=tcp6,proto3" json:"tcp6,omitempty"`
	Udp           bool                   `protobuf:"varint,3,opt,name=udp,proto3" json:"udp,omitempty"`
	Udp6          bool                   `protobuf:"varint,4,opt,name=udp6,proto3" json:"udp6,omitempty"`
	Udplite       bool                   `protobuf:"varint,5,opt,name=udplite,proto3" json:"udplite,omitempty"`
	Udplite6      bool                   `protobuf:"varint,6,opt,name=udplite6,proto3" json:"udplite6,omitempty"`
	Raw           bool                   `protobuf:"varint,7,opt,name=raw,proto3" json:"raw,omitempty"`
	Raw6          bool                   `protobuf:"varint,8,opt,name=raw6,proto3" json:"raw6,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetstatRequest_L4Proto) Reset() {
	*x = NetstatRequest_L4Proto{}
	mi := &file_machine_machine_proto_msgTypes[175]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetstatRequest_L4Proto) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetstatRequest_L4Proto) ProtoMessage() {}

func (x *NetstatRequest_L4Proto) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[175]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetstatRequest_L4Proto.ProtoReflect.Descriptor instead.
func (*NetstatRequest_L4Proto) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{157, 1}
}

func (x *NetstatRequest_L4Proto) GetTcp() bool {
	if x != nil {
		return x.Tcp
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetTcp6() bool {
	if x != nil {
		return x.Tcp6
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetUdp() bool {
	if x != nil {
		return x.Udp
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetUdp6() bool {
	if x != nil {
		return x.Udp6
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetUdplite() bool {
	if x != nil {
		return x.Udplite
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetUdplite6() bool {
	if x != nil {
		return x.Udplite6
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetRaw() bool {
	if x != nil {
		return x.Raw
	}
	return false
}

func (x *NetstatRequest_L4Proto) GetRaw6() bool {
	if x != nil {
		return x.Raw6
	}
	return false
}

type NetstatRequest_NetNS struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Hostnetwork   bool                   `protobuf:"varint,1,opt,name=hostnetwork,proto3" json:"hostnetwork,omitempty"`
	Netns         []string               `protobuf:"bytes,2,rep,name=netns,proto3" json:"netns,omitempty"`
	Allnetns      bool                   `protobuf:"varint,3,opt,name=allnetns,proto3" json:"allnetns,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetstatRequest_NetNS) Reset() {
	*x = NetstatRequest_NetNS{}
	mi := &file_machine_machine_proto_msgTypes[176]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetstatRequest_NetNS) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetstatRequest_NetNS) ProtoMessage() {}

func (x *NetstatRequest_NetNS) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[176]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetstatRequest_NetNS.ProtoReflect.Descriptor instead.
func (*NetstatRequest_NetNS) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{157, 2}
}

func (x *NetstatRequest_NetNS) GetHostnetwork() bool {
	if x != nil {
		return x.Hostnetwork
	}
	return false
}

func (x *NetstatRequest_NetNS) GetNetns() []string {
	if x != nil {
		return x.Netns
	}
	return nil
}

func (x *NetstatRequest_NetNS) GetAllnetns() bool {
	if x != nil {
		return x.Allnetns
	}
	return false
}

type ConnectRecord_Process struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Pid           uint32                 `protobuf:"varint,1,opt,name=pid,proto3" json:"pid,omitempty"`
	Name          string                 `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ConnectRecord_Process) Reset() {
	*x = ConnectRecord_Process{}
	mi := &file_machine_machine_proto_msgTypes[177]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ConnectRecord_Process) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ConnectRecord_Process) ProtoMessage() {}

func (x *ConnectRecord_Process) ProtoReflect() protoreflect.Message {
	mi := &file_machine_machine_proto_msgTypes[177]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ConnectRecord_Process.ProtoReflect.Descriptor instead.
func (*ConnectRecord_Process) Descriptor() ([]byte, []int) {
	return file_machine_machine_proto_rawDescGZIP(), []int{158, 0}
}

func (x *ConnectRecord_Process) GetPid() uint32 {
	if x != nil {
		return x.Pid
	}
	return 0
}

func (x *ConnectRecord_Process) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

var File_machine_machine_proto protoreflect.FileDescriptor

const file_machine_machine_proto_rawDesc = "" +
	"\n" +
	"\x15machine/machine.proto\x12\amachine\x1a\x13common/common.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x8c\x02\n" +
	"\x19ApplyConfigurationRequest\x12\x12\n" +
	"\x04data\x18\x01 \x01(\fR\x04data\x12;\n" +
	"\x04mode\x18\x04 \x01(\x0e2'.machine.ApplyConfigurationRequest.ModeR\x04mode\x12\x17\n" +
	"\adry_run\x18\x05 \x01(\bR\x06dryRun\x12C\n" +
	"\x10try_mode_timeout\x18\x06 \x01(\v2\x19.google.protobuf.DurationR\x0etryModeTimeout\"@\n" +
	"\x04Mode\x12\n" +
	"\n" +
	"\x06REBOOT\x10\x00\x12\b\n" +
	"\x04AUTO\x10\x01\x12\r\n" +
	"\tNO_REBOOT\x10\x02\x12\n" +
	"\n" +
	"\x06STAGED\x10\x03\x12\a\n" +
	"\x03TRY\x10\x04\"\xbe\x01\n" +
	"\x12ApplyConfiguration\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x1a\n" +
	"\bwarnings\x18\x02 \x03(\tR\bwarnings\x12;\n" +
	"\x04mode\x18\x03 \x01(\x0e2'.machine.ApplyConfigurationRequest.ModeR\x04mode\x12!\n" +
	"\fmode_details\x18\x04 \x01(\tR\vmodeDetails\"U\n" +
	"\x1aApplyConfigurationResponse\x127\n" +
	"\bmessages\x18\x01 \x03(\v2\x1b.machine.ApplyConfigurationR\bmessages\"p\n" +
	"\rRebootRequest\x12/\n" +
	"\x04mode\x18\x01 \x01(\x0e2\x1b.machine.RebootRequest.ModeR\x04mode\".\n" +
	"\x04Mode\x12\v\n" +
	"\aDEFAULT\x10\x00\x12\x0e\n" +
	"\n" +
	"POWERCYCLE\x10\x01\x12\t\n" +
	"\x05FORCE\x10\x02\"Q\n" +
	"\x06Reboot\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x19\n" +
	"\bactor_id\x18\x02 \x01(\tR\aactorId\"=\n" +
	"\x0eRebootResponse\x12+\n" +
	"\bmessages\x18\x01 \x03(\v2\x0f.machine.RebootR\bmessages\"l\n" +
	"\x10BootstrapRequest\x12!\n" +
	"\frecover_etcd\x18\x01 \x01(\bR\vrecoverEtcd\x125\n" +
	"\x17recover_skip_hash_check\x18\x02 \x01(\bR\x14recoverSkipHashCheck\"9\n" +
	"\tBootstrap\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"C\n" +
	"\x11BootstrapResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.BootstrapR\bmessages\"\xb0\x01\n" +
	"\rSequenceEvent\x12\x1a\n" +
	"\bsequence\x18\x01 \x01(\tR\bsequence\x125\n" +
	"\x06action\x18\x02 \x01(\x0e2\x1d.machine.SequenceEvent.ActionR\x06action\x12#\n" +
	"\x05error\x18\x03 \x01(\v2\r.common.ErrorR\x05error\"'\n" +
	"\x06Action\x12\b\n" +
	"\x04NOOP\x10\x00\x12\t\n" +
	"\x05START\x10\x01\x12\b\n" +
	"\x04STOP\x10\x02\"u\n" +
	"\n" +
	"PhaseEvent\x12\x14\n" +
	"\x05phase\x18\x01 \x01(\tR\x05phase\x122\n" +
	"\x06action\x18\x02 \x01(\x0e2\x1a.machine.PhaseEvent.ActionR\x06action\"\x1d\n" +
	"\x06Action\x12\t\n" +
	"\x05START\x10\x00\x12\b\n" +
	"\x04STOP\x10\x01\"q\n" +
	"\tTaskEvent\x12\x12\n" +
	"\x04task\x18\x01 \x01(\tR\x04task\x121\n" +
	"\x06action\x18\x02 \x01(\x0e2\x19.machine.TaskEvent.ActionR\x06action\"\x1d\n" +
	"\x06Action\x12\t\n" +
	"\x05START\x10\x00\x12\b\n" +
	"\x04STOP\x10\x01\"\xba\x02\n" +
	"\x11ServiceStateEvent\x12\x18\n" +
	"\aservice\x18\x01 \x01(\tR\aservice\x129\n" +
	"\x06action\x18\x02 \x01(\x0e2!.machine.ServiceStateEvent.ActionR\x06action\x12\x18\n" +
	"\amessage\x18\x03 \x01(\tR\amessage\x12.\n" +
	"\x06health\x18\x04 \x01(\v2\x16.machine.ServiceHealthR\x06health\"\x85\x01\n" +
	"\x06Action\x12\x0f\n" +
	"\vINITIALIZED\x10\x00\x12\r\n" +
	"\tPREPARING\x10\x01\x12\v\n" +
	"\aWAITING\x10\x02\x12\v\n" +
	"\aRUNNING\x10\x03\x12\f\n" +
	"\bSTOPPING\x10\x04\x12\f\n" +
	"\bFINISHED\x10\x05\x12\n" +
	"\n" +
	"\x06FAILED\x10\x06\x12\v\n" +
	"\aSKIPPED\x10\a\x12\f\n" +
	"\bSTARTING\x10\b\" \n" +
	"\fRestartEvent\x12\x10\n" +
	"\x03cmd\x18\x01 \x01(\x03R\x03cmd\",\n" +
	"\x14ConfigLoadErrorEvent\x12\x14\n" +
	"\x05error\x18\x01 \x01(\tR\x05error\"2\n" +
	"\x1aConfigValidationErrorEvent\x12\x14\n" +
	"\x05error\x18\x01 \x01(\tR\x05error\"H\n" +
	"\fAddressEvent\x12\x1a\n" +
	"\bhostname\x18\x01 \x01(\tR\bhostname\x12\x1c\n" +
	"\taddresses\x18\x02 \x03(\tR\taddresses\"\xfb\x03\n" +
	"\x12MachineStatusEvent\x12>\n" +
	"\x05stage\x18\x01 \x01(\x0e2(.machine.MachineStatusEvent.MachineStageR\x05stage\x12A\n" +
	"\x06status\x18\x02 \x01(\v2).machine.MachineStatusEvent.MachineStatusR\x06status\x1a\xc8\x01\n" +
	"\rMachineStatus\x12\x14\n" +
	"\x05ready\x18\x01 \x01(\bR\x05ready\x12c\n" +
	"\x10unmet_conditions\x18\x02 \x03(\v28.machine.MachineStatusEvent.MachineStatus.UnmetConditionR\x0funmetConditions\x1a<\n" +
	"\x0eUnmetCondition\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n" +
	"\x06reason\x18\x02 \x01(\tR\x06reason\"\x96\x01\n" +
	"\fMachineStage\x12\v\n" +
	"\aUNKNOWN\x10\x00\x12\v\n" +
	"\aBOOTING\x10\x01\x12\x0e\n" +
	"\n" +
	"INSTALLING\x10\x02\x12\x0f\n" +
	"\vMAINTENANCE\x10\x03\x12\v\n" +
	"\aRUNNING\x10\x04\x12\r\n" +
	"\tREBOOTING\x10\x05\x12\x11\n" +
	"\rSHUTTING_DOWN\x10\x06\x12\r\n" +
	"\tRESETTING\x10\a\x12\r\n" +
	"\tUPGRADING\x10\b\"\x90\x01\n" +
	"\rEventsRequest\x12\x1f\n" +
	"\vtail_events\x18\x01 \x01(\x05R\n" +
	"tailEvents\x12\x17\n" +
	"\atail_id\x18\x02 \x01(\tR\x06tailId\x12!\n" +
	"\ftail_seconds\x18\x03 \x01(\x05R\vtailSeconds\x12\"\n" +
	"\rwith_actor_id\x18\x04 \x01(\tR\vwithActorId\"\x8a\x01\n" +
	"\x05Event\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12(\n" +
	"\x04data\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\x04data\x12\x0e\n" +
	"\x02id\x18\x03 \x01(\tR\x02id\x12\x19\n" +
	"\bactor_id\x18\x04 \x01(\tR\aactorId\">\n" +
	"\x12ResetPartitionSpec\x12\x14\n" +
	"\x05label\x18\x01 \x01(\tR\x05label\x12\x12\n" +
	"\x04wipe\x18\x02 \x01(\bR\x04wipe\"\xb1\x02\n" +
	"\fResetRequest\x12\x1a\n" +
	"\bgraceful\x18\x01 \x01(\bR\bgraceful\x12\x16\n" +
	"\x06reboot\x18\x02 \x01(\bR\x06reboot\x12V\n" +
	"\x19system_partitions_to_wipe\x18\x03 \x03(\v2\x1b.machine.ResetPartitionSpecR\x16systemPartitionsToWipe\x12+\n" +
	"\x12user_disks_to_wipe\x18\x04 \x03(\tR\x0fuserDisksToWipe\x122\n" +
	"\x04mode\x18\x05 \x01(\x0e2\x1e.machine.ResetRequest.WipeModeR\x04mode\"4\n" +
	"\bWipeMode\x12\a\n" +
	"\x03ALL\x10\x00\x12\x0f\n" +
	"\vSYSTEM_DISK\x10\x01\x12\x0e\n" +
	"\n" +
	"USER_DISKS\x10\x02\"P\n" +
	"\x05Reset\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x19\n" +
	"\bactor_id\x18\x02 \x01(\tR\aactorId\";\n" +
	"\rResetResponse\x12*\n" +
	"\bmessages\x18\x01 \x03(\v2\x0e.machine.ResetR\bmessages\"S\n" +
	"\bShutdown\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x19\n" +
	"\bactor_id\x18\x02 \x01(\tR\aactorId\"'\n" +
	"\x0fShutdownRequest\x12\x14\n" +
	"\x05force\x18\x01 \x01(\bR\x05force\"A\n" +
	"\x10ShutdownResponse\x12-\n" +
	"\bmessages\x18\x01 \x03(\v2\x11.machine.ShutdownR\bmessages\"\xde\x01\n" +
	"\x0eUpgradeRequest\x12\x14\n" +
	"\x05image\x18\x01 \x01(\tR\x05image\x12\x1a\n" +
	"\bpreserve\x18\x02 \x01(\bR\bpreserve\x12\x14\n" +
	"\x05stage\x18\x03 \x01(\bR\x05stage\x12\x14\n" +
	"\x05force\x18\x04 \x01(\bR\x05force\x12C\n" +
	"\vreboot_mode\x18\x05 \x01(\x0e2\".machine.UpgradeRequest.RebootModeR\n" +
	"rebootMode\")\n" +
	"\n" +
	"RebootMode\x12\v\n" +
	"\aDEFAULT\x10\x00\x12\x0e\n" +
	"\n" +
	"POWERCYCLE\x10\x01\"d\n" +
	"\aUpgrade\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x10\n" +
	"\x03ack\x18\x02 \x01(\tR\x03ack\x12\x19\n" +
	"\bactor_id\x18\x03 \x01(\tR\aactorId\"?\n" +
	"\x0fUpgradeResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.UpgradeR\bmessages\"m\n" +
	"\vServiceList\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x120\n" +
	"\bservices\x18\x02 \x03(\v2\x14.machine.ServiceInfoR\bservices\"G\n" +
	"\x13ServiceListResponse\x120\n" +
	"\bmessages\x18\x01 \x03(\v2\x14.machine.ServiceListR\bmessages\"\x93\x01\n" +
	"\vServiceInfo\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x12\x14\n" +
	"\x05state\x18\x02 \x01(\tR\x05state\x12.\n" +
	"\x06events\x18\x03 \x01(\v2\x16.machine.ServiceEventsR\x06events\x12.\n" +
	"\x06health\x18\x04 \x01(\v2\x16.machine.ServiceHealthR\x06health\">\n" +
	"\rServiceEvents\x12-\n" +
	"\x06events\x18\x01 \x03(\v2\x15.machine.ServiceEventR\x06events\"b\n" +
	"\fServiceEvent\x12\x10\n" +
	"\x03msg\x18\x01 \x01(\tR\x03msg\x12\x14\n" +
	"\x05state\x18\x02 \x01(\tR\x05state\x12*\n" +
	"\x02ts\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\x02ts\"\xa3\x01\n" +
	"\rServiceHealth\x12\x18\n" +
	"\aunknown\x18\x01 \x01(\bR\aunknown\x12\x18\n" +
	"\ahealthy\x18\x02 \x01(\bR\ahealthy\x12!\n" +
	"\flast_message\x18\x03 \x01(\tR\vlastMessage\x12;\n" +
	"\vlast_change\x18\x04 \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"lastChange\"%\n" +
	"\x13ServiceStartRequest\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\"P\n" +
	"\fServiceStart\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04resp\x18\x02 \x01(\tR\x04resp\"I\n" +
	"\x14ServiceStartResponse\x121\n" +
	"\bmessages\x18\x01 \x03(\v2\x15.machine.ServiceStartR\bmessages\"$\n" +
	"\x12ServiceStopRequest\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\"O\n" +
	"\vServiceStop\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04resp\x18\x02 \x01(\tR\x04resp\"G\n" +
	"\x13ServiceStopResponse\x120\n" +
	"\bmessages\x18\x01 \x03(\v2\x14.machine.ServiceStopR\bmessages\"'\n" +
	"\x15ServiceRestartRequest\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\"R\n" +
	"\x0eServiceRestart\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04resp\x18\x02 \x01(\tR\x04resp\"M\n" +
	"\x16ServiceRestartResponse\x123\n" +
	"\bmessages\x18\x01 \x03(\v2\x17.machine.ServiceRestartR\bmessages\"*\n" +
	"\vCopyRequest\x12\x1b\n" +
	"\troot_path\x18\x01 \x01(\tR\brootPath\"\xeb\x01\n" +
	"\vListRequest\x12\x12\n" +
	"\x04root\x18\x01 \x01(\tR\x04root\x12\x18\n" +
	"\arecurse\x18\x02 \x01(\bR\arecurse\x12'\n" +
	"\x0frecursion_depth\x18\x03 \x01(\x05R\x0erecursionDepth\x12/\n" +
	"\x05types\x18\x04 \x03(\x0e2\x19.machine.ListRequest.TypeR\x05types\x12#\n" +
	"\rreport_xattrs\x18\x05 \x01(\bR\freportXattrs\"/\n" +
	"\x04Type\x12\v\n" +
	"\aREGULAR\x10\x00\x12\r\n" +
	"\tDIRECTORY\x10\x01\x12\v\n" +
	"\aSYMLINK\x10\x02\"\x81\x01\n" +
	"\x10DiskUsageRequest\x12'\n" +
	"\x0frecursion_depth\x18\x01 \x01(\x05R\x0erecursionDepth\x12\x10\n" +
	"\x03all\x18\x02 \x01(\bR\x03all\x12\x1c\n" +
	"\tthreshold\x18\x03 \x01(\x03R\tthreshold\x12\x14\n" +
	"\x05paths\x18\x04 \x03(\tR\x05paths\"\xc2\x02\n" +
	"\bFileInfo\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\x12\x12\n" +
	"\x04size\x18\x03 \x01(\x03R\x04size\x12\x12\n" +
	"\x04mode\x18\x04 \x01(\rR\x04mode\x12\x1a\n" +
	"\bmodified\x18\x05 \x01(\x03R\bmodified\x12\x15\n" +
	"\x06is_dir\x18\x06 \x01(\bR\x05isDir\x12\x14\n" +
	"\x05error\x18\a \x01(\tR\x05error\x12\x12\n" +
	"\x04link\x18\b \x01(\tR\x04link\x12#\n" +
	"\rrelative_name\x18\t \x01(\tR\frelativeName\x12\x10\n" +
	"\x03uid\x18\n" +
	" \x01(\rR\x03uid\x12\x10\n" +
	"\x03gid\x18\v \x01(\rR\x03gid\x12&\n" +
	"\x06xattrs\x18\f \x03(\v2\x0e.machine.XattrR\x06xattrs\"/\n" +
	"\x05Xattr\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n" +
	"\x04data\x18\x02 \x01(\fR\x04data\"\xa0\x01\n" +
	"\rDiskUsageInfo\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\x12\x12\n" +
	"\x04size\x18\x03 \x01(\x03R\x04size\x12\x14\n" +
	"\x05error\x18\x04 \x01(\tR\x05error\x12#\n" +
	"\rrelative_name\x18\x05 \x01(\tR\frelativeName\"`\n" +
	"\x06Mounts\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12(\n" +
	"\x05stats\x18\x02 \x03(\v2\x12.machine.MountStatR\x05stats\"=\n" +
	"\x0eMountsResponse\x12+\n" +
	"\bmessages\x18\x01 \x03(\v2\x0f.machine.MountsR\bmessages\"|\n" +
	"\tMountStat\x12\x1e\n" +
	"\n" +
	"filesystem\x18\x01 \x01(\tR\n" +
	"filesystem\x12\x12\n" +
	"\x04size\x18\x02 \x01(\x04R\x04size\x12\x1c\n" +
	"\tavailable\x18\x03 \x01(\x04R\tavailable\x12\x1d\n" +
	"\n" +
	"mounted_on\x18\x04 \x01(\tR\tmountedOn\"\xcd\x01\n" +
	"\aVersion\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12.\n" +
	"\aversion\x18\x02 \x01(\v2\x14.machine.VersionInfoR\aversion\x121\n" +
	"\bplatform\x18\x03 \x01(\v2\x15.machine.PlatformInfoR\bplatform\x121\n" +
	"\bfeatures\x18\x04 \x01(\v2\x15.machine.FeaturesInfoR\bfeatures\"?\n" +
	"\x0fVersionResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.VersionR\bmessages\"\x8a\x01\n" +
	"\vVersionInfo\x12\x10\n" +
	"\x03tag\x18\x01 \x01(\tR\x03tag\x12\x10\n" +
	"\x03sha\x18\x02 \x01(\tR\x03sha\x12\x14\n" +
	"\x05built\x18\x03 \x01(\tR\x05built\x12\x1d\n" +
	"\n" +
	"go_version\x18\x04 \x01(\tR\tgoVersion\x12\x0e\n" +
	"\x02os\x18\x05 \x01(\tR\x02os\x12\x12\n" +
	"\x04arch\x18\x06 \x01(\tR\x04arch\"6\n" +
	"\fPlatformInfo\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n" +
	"\x04mode\x18\x02 \x01(\tR\x04mode\"\"\n" +
	"\fFeaturesInfo\x12\x12\n" +
	"\x04rbac\x18\x01 \x01(\bR\x04rbac\"\xa3\x01\n" +
	"\vLogsRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\tR\x02id\x12/\n" +
	"\x06driver\x18\x03 \x01(\x0e2\x17.common.ContainerDriverR\x06driver\x12\x16\n" +
	"\x06follow\x18\x04 \x01(\bR\x06follow\x12\x1d\n" +
	"\n" +
	"tail_lines\x18\x05 \x01(\x05R\ttailLines\"!\n" +
	"\vReadRequest\x12\x12\n" +
	"\x04path\x18\x01 \x01(\tR\x04path\"O\n" +
	"\rLogsContainer\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x10\n" +
	"\x03ids\x18\x02 \x03(\tR\x03ids\"L\n" +
	"\x16LogsContainersResponse\x122\n" +
	"\bmessages\x18\x01 \x03(\v2\x16.machine.LogsContainerR\bmessages\"\x11\n" +
	"\x0fRollbackRequest\"8\n" +
	"\bRollback\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"A\n" +
	"\x10RollbackResponse\x12-\n" +
	"\bmessages\x18\x01 \x03(\v2\x11.machine.RollbackR\bmessages\"b\n" +
	"\x11ContainersRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12/\n" +
	"\x06driver\x18\x02 \x01(\x0e2\x17.common.ContainerDriverR\x06driver\"\x88\x02\n" +
	"\rContainerInfo\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\tR\x02id\x12\x10\n" +
	"\x03uid\x18\n" +
	" \x01(\tR\x03uid\x12\x1f\n" +
	"\vinternal_id\x18\t \x01(\tR\n" +
	"internalId\x12\x14\n" +
	"\x05image\x18\x03 \x01(\tR\x05image\x12\x10\n" +
	"\x03pid\x18\x04 \x01(\rR\x03pid\x12\x16\n" +
	"\x06status\x18\x05 \x01(\tR\x06status\x12\x15\n" +
	"\x06pod_id\x18\x06 \x01(\tR\x05podId\x12\x12\n" +
	"\x04name\x18\a \x01(\tR\x04name\x12+\n" +
	"\x11network_namespace\x18\b \x01(\tR\x10networkNamespace\"q\n" +
	"\tContainer\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x126\n" +
	"\n" +
	"containers\x18\x02 \x03(\v2\x16.machine.ContainerInfoR\n" +
	"containers\"D\n" +
	"\x12ContainersResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.ContainerR\bmessages\":\n" +
	"\fDmesgRequest\x12\x16\n" +
	"\x06follow\x18\x01 \x01(\bR\x06follow\x12\x12\n" +
	"\x04tail\x18\x02 \x01(\bR\x04tail\"A\n" +
	"\x11ProcessesResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.ProcessR\bmessages\"k\n" +
	"\aProcess\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x122\n" +
	"\tprocesses\x18\x02 \x03(\v2\x14.machine.ProcessInfoR\tprocesses\"\xb2\x02\n" +
	"\vProcessInfo\x12\x10\n" +
	"\x03pid\x18\x01 \x01(\x05R\x03pid\x12\x12\n" +
	"\x04ppid\x18\x02 \x01(\x05R\x04ppid\x12\x14\n" +
	"\x05state\x18\x03 \x01(\tR\x05state\x12\x18\n" +
	"\athreads\x18\x04 \x01(\x05R\athreads\x12\x19\n" +
	"\bcpu_time\x18\x05 \x01(\x01R\acpuTime\x12%\n" +
	"\x0evirtual_memory\x18\x06 \x01(\x04R\rvirtualMemory\x12'\n" +
	"\x0fresident_memory\x18\a \x01(\x04R\x0eresidentMemory\x12\x18\n" +
	"\acommand\x18\b \x01(\tR\acommand\x12\x1e\n" +
	"\n" +
	"executable\x18\t \x01(\tR\n" +
	"executable\x12\x12\n" +
	"\x04args\x18\n" +
	" \x01(\tR\x04args\x12\x14\n" +
	"\x05label\x18\v \x01(\tR\x05label\"o\n" +
	"\x0eRestartRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\tR\x02id\x12/\n" +
	"\x06driver\x18\x03 \x01(\x0e2\x17.common.ContainerDriverR\x06driver\"7\n" +
	"\aRestart\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"?\n" +
	"\x0fRestartResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.RestartR\bmessages\"]\n" +
	"\fStatsRequest\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12/\n" +
	"\x06driver\x18\x02 \x01(\x0e2\x17.common.ContainerDriverR\x06driver\"Z\n" +
	"\x05Stats\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12#\n" +
	"\x05stats\x18\x02 \x03(\v2\r.machine.StatR\x05stats\";\n" +
	"\rStatsResponse\x12*\n" +
	"\bmessages\x18\x01 \x03(\v2\x0e.machine.StatsR\bmessages\"\x9f\x01\n" +
	"\x04Stat\x12\x1c\n" +
	"\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\tR\x02id\x12!\n" +
	"\fmemory_usage\x18\x04 \x01(\x04R\vmemoryUsage\x12\x1b\n" +
	"\tcpu_usage\x18\x05 \x01(\x04R\bcpuUsage\x12\x15\n" +
	"\x06pod_id\x18\x06 \x01(\tR\x05podId\x12\x12\n" +
	"\x04name\x18\a \x01(\tR\x04name\"b\n" +
	"\x06Memory\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12*\n" +
	"\ameminfo\x18\x02 \x01(\v2\x10.machine.MemInfoR\ameminfo\"=\n" +
	"\x0eMemoryResponse\x12+\n" +
	"\bmessages\x18\x01 \x03(\v2\x0f.machine.MemoryR\bmessages\"\x8b\f\n" +
	"\aMemInfo\x12\x1a\n" +
	"\bmemtotal\x18\x01 \x01(\x04R\bmemtotal\x12\x18\n" +
	"\amemfree\x18\x02 \x01(\x04R\amemfree\x12\"\n" +
	"\fmemavailable\x18\x03 \x01(\x04R\fmemavailable\x12\x18\n" +
	"\abuffers\x18\x04 \x01(\x04R\abuffers\x12\x16\n" +
	"\x06cached\x18\x05 \x01(\x04R\x06cached\x12\x1e\n" +
	"\n" +
	"swapcached\x18\x06 \x01(\x04R\n" +
	"swapcached\x12\x16\n" +
	"\x06active\x18\a \x01(\x04R\x06active\x12\x1a\n" +
	"\binactive\x18\b \x01(\x04R\binactive\x12\x1e\n" +
	"\n" +
	"activeanon\x18\t \x01(\x04R\n" +
	"activeanon\x12\"\n" +
	"\finactiveanon\x18\n" +
	" \x01(\x04R\finactiveanon\x12\x1e\n" +
	"\n" +
	"activefile\x18\v \x01(\x04R\n" +
	"activefile\x12\"\n" +
	"\finactivefile\x18\f \x01(\x04R\finactivefile\x12 \n" +
	"\vunevictable\x18\r \x01(\x04R\vunevictable\x12\x18\n" +
	"\amlocked\x18\x0e \x01(\x04R\amlocked\x12\x1c\n" +
	"\tswaptotal\x18\x0f \x01(\x04R\tswaptotal\x12\x1a\n" +
	"\bswapfree\x18\x10 \x01(\x04R\bswapfree\x12\x14\n" +
	"\x05dirty\x18\x11 \x01(\x04R\x05dirty\x12\x1c\n" +
	"\twriteback\x18\x12 \x01(\x04R\twriteback\x12\x1c\n" +
	"\tanonpages\x18\x13 \x01(\x04R\tanonpages\x12\x16\n" +
	"\x06mapped\x18\x14 \x01(\x04R\x06mapped\x12\x14\n" +
	"\x05shmem\x18\x15 \x01(\x04R\x05shmem\x12\x12\n" +
	"\x04slab\x18\x16 \x01(\x04R\x04slab\x12\"\n" +
	"\fsreclaimable\x18\x17 \x01(\x04R\fsreclaimable\x12\x1e\n" +
	"\n" +
	"sunreclaim\x18\x18 \x01(\x04R\n" +
	"sunreclaim\x12 \n" +
	"\vkernelstack\x18\x19 \x01(\x04R\vkernelstack\x12\x1e\n" +
	"\n" +
	"pagetables\x18\x1a \x01(\x04R\n" +
	"pagetables\x12 \n" +
	"\vnfsunstable\x18\x1b \x01(\x04R\vnfsunstable\x12\x16\n" +
	"\x06bounce\x18\x1c \x01(\x04R\x06bounce\x12\"\n" +
	"\fwritebacktmp\x18\x1d \x01(\x04R\fwritebacktmp\x12 \n" +
	"\vcommitlimit\x18\x1e \x01(\x04R\vcommitlimit\x12 \n" +
	"\vcommittedas\x18\x1f \x01(\x04R\vcommittedas\x12\"\n" +
	"\fvmalloctotal\x18  \x01(\x04R\fvmalloctotal\x12 \n" +
	"\vvmallocused\x18! \x01(\x04R\vvmallocused\x12\"\n" +
	"\fvmallocchunk\x18\" \x01(\x04R\fvmallocchunk\x12,\n" +
	"\x11hardwarecorrupted\x18# \x01(\x04R\x11hardwarecorrupted\x12$\n" +
	"\ranonhugepages\x18$ \x01(\x04R\ranonhugepages\x12&\n" +
	"\x0eshmemhugepages\x18% \x01(\x04R\x0eshmemhugepages\x12&\n" +
	"\x0eshmempmdmapped\x18& \x01(\x04R\x0eshmempmdmapped\x12\x1a\n" +
	"\bcmatotal\x18' \x01(\x04R\bcmatotal\x12\x18\n" +
	"\acmafree\x18( \x01(\x04R\acmafree\x12&\n" +
	"\x0ehugepagestotal\x18) \x01(\x04R\x0ehugepagestotal\x12$\n" +
	"\rhugepagesfree\x18* \x01(\x04R\rhugepagesfree\x12$\n" +
	"\rhugepagesrsvd\x18+ \x01(\x04R\rhugepagesrsvd\x12$\n" +
	"\rhugepagessurp\x18, \x01(\x04R\rhugepagessurp\x12\"\n" +
	"\fhugepagesize\x18- \x01(\x04R\fhugepagesize\x12 \n" +
	"\vdirectmap4k\x18. \x01(\x04R\vdirectmap4k\x12 \n" +
	"\vdirectmap2m\x18/ \x01(\x04R\vdirectmap2m\x12 \n" +
	"\vdirectmap1g\x180 \x01(\x04R\vdirectmap1g\"A\n" +
	"\x10HostnameResponse\x12-\n" +
	"\bmessages\x18\x01 \x03(\v2\x11.machine.HostnameR\bmessages\"T\n" +
	"\bHostname\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x1a\n" +
	"\bhostname\x18\x02 \x01(\tR\bhostname\"?\n" +
	"\x0fLoadAvgResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.LoadAvgR\bmessages\"{\n" +
	"\aLoadAvg\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x14\n" +
	"\x05load1\x18\x02 \x01(\x01R\x05load1\x12\x14\n" +
	"\x05load5\x18\x03 \x01(\x01R\x05load5\x12\x16\n" +
	"\x06load15\x18\x04 \x01(\x01R\x06load15\"E\n" +
	"\x12SystemStatResponse\x12/\n" +
	"\bmessages\x18\x01 \x03(\v2\x13.machine.SystemStatR\bmessages\"\xd6\x03\n" +
	"\n" +
	"SystemStat\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x1b\n" +
	"\tboot_time\x18\x02 \x01(\x04R\bbootTime\x12-\n" +
	"\tcpu_total\x18\x03 \x01(\v2\x10.machine.CPUStatR\bcpuTotal\x12\"\n" +
	"\x03cpu\x18\x04 \x03(\v2\x10.machine.CPUStatR\x03cpu\x12\x1b\n" +
	"\tirq_total\x18\x05 \x01(\x04R\birqTotal\x12\x10\n" +
	"\x03irq\x18\x06 \x03(\x04R\x03irq\x12)\n" +
	"\x10context_switches\x18\a \x01(\x04R\x0fcontextSwitches\x12'\n" +
	"\x0fprocess_created\x18\b \x01(\x04R\x0eprocessCreated\x12'\n" +
	"\x0fprocess_running\x18\t \x01(\x04R\x0eprocessRunning\x12'\n" +
	"\x0fprocess_blocked\x18\n" +
	" \x01(\x04R\x0eprocessBlocked\x12$\n" +
	"\x0esoft_irq_total\x18\v \x01(\x04R\fsoftIrqTotal\x12/\n" +
	"\bsoft_irq\x18\f \x01(\v2\x14.machine.SoftIRQStatR\asoftIrq\"\xed\x01\n" +
	"\aCPUStat\x12\x12\n" +
	"\x04user\x18\x01 \x01(\x01R\x04user\x12\x12\n" +
	"\x04nice\x18\x02 \x01(\x01R\x04nice\x12\x16\n" +
	"\x06system\x18\x03 \x01(\x01R\x06system\x12\x12\n" +
	"\x04idle\x18\x04 \x01(\x01R\x04idle\x12\x16\n" +
	"\x06iowait\x18\x05 \x01(\x01R\x06iowait\x12\x10\n" +
	"\x03irq\x18\x06 \x01(\x01R\x03irq\x12\x19\n" +
	"\bsoft_irq\x18\a \x01(\x01R\asoftIrq\x12\x14\n" +
	"\x05steal\x18\b \x01(\x01R\x05steal\x12\x14\n" +
	"\x05guest\x18\t \x01(\x01R\x05guest\x12\x1d\n" +
	"\n" +
	"guest_nice\x18\n" +
	" \x01(\x01R\tguestNice\"\xf7\x01\n" +
	"\vSoftIRQStat\x12\x0e\n" +
	"\x02hi\x18\x01 \x01(\x04R\x02hi\x12\x14\n" +
	"\x05timer\x18\x02 \x01(\x04R\x05timer\x12\x15\n" +
	"\x06net_tx\x18\x03 \x01(\x04R\x05netTx\x12\x15\n" +
	"\x06net_rx\x18\x04 \x01(\x04R\x05netRx\x12\x14\n" +
	"\x05block\x18\x05 \x01(\x04R\x05block\x12\"\n" +
	"\rblock_io_poll\x18\x06 \x01(\x04R\vblockIoPoll\x12\x18\n" +
	"\atasklet\x18\a \x01(\x04R\atasklet\x12\x14\n" +
	"\x05sched\x18\b \x01(\x04R\x05sched\x12\x18\n" +
	"\ahrtimer\x18\t \x01(\x04R\ahrtimer\x12\x10\n" +
	"\x03rcu\x18\n" +
	" \x01(\x04R\x03rcu\"J\n" +
	"\x14CPUFreqStatsResponse\x122\n" +
	"\bmessages\x18\x01 \x03(\v2\x16.machine.CPUsFreqStatsR\bmessages\"z\n" +
	"\rCPUsFreqStats\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12;\n" +
	"\x0ecpu_freq_stats\x18\x02 \x03(\v2\x15.machine.CPUFreqStatsR\fcpuFreqStats\"\xb1\x01\n" +
	"\fCPUFreqStats\x12+\n" +
	"\x11current_frequency\x18\x01 \x01(\x04R\x10currentFrequency\x12+\n" +
	"\x11minimum_frequency\x18\x02 \x01(\x04R\x10minimumFrequency\x12+\n" +
	"\x11maximum_frequency\x18\x03 \x01(\x04R\x10maximumFrequency\x12\x1a\n" +
	"\bgovernor\x18\x04 \x01(\tR\bgovernor\"@\n" +
	"\x0fCPUInfoResponse\x12-\n" +
	"\bmessages\x18\x01 \x03(\v2\x11.machine.CPUsInfoR\bmessages\"e\n" +
	"\bCPUsInfo\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12+\n" +
	"\bcpu_info\x18\x02 \x03(\v2\x10.machine.CPUInfoR\acpuInfo\"\x8b\x06\n" +
	"\aCPUInfo\x12\x1c\n" +
	"\tprocessor\x18\x01 \x01(\rR\tprocessor\x12\x1b\n" +
	"\tvendor_id\x18\x02 \x01(\tR\bvendorId\x12\x1d\n" +
	"\n" +
	"cpu_family\x18\x03 \x01(\tR\tcpuFamily\x12\x14\n" +
	"\x05model\x18\x04 \x01(\tR\x05model\x12\x1d\n" +
	"\n" +
	"model_name\x18\x05 \x01(\tR\tmodelName\x12\x1a\n" +
	"\bstepping\x18\x06 \x01(\tR\bstepping\x12\x1c\n" +
	"\tmicrocode\x18\a \x01(\tR\tmicrocode\x12\x17\n" +
	"\acpu_mhz\x18\b \x01(\x01R\x06cpuMhz\x12\x1d\n" +
	"\n" +
	"cache_size\x18\t \x01(\tR\tcacheSize\x12\x1f\n" +
	"\vphysical_id\x18\n" +
	" \x01(\tR\n" +
	"physicalId\x12\x1a\n" +
	"\bsiblings\x18\v \x01(\rR\bsiblings\x12\x17\n" +
	"\acore_id\x18\f \x01(\tR\x06coreId\x12\x1b\n" +
	"\tcpu_cores\x18\r \x01(\rR\bcpuCores\x12\x17\n" +
	"\aapic_id\x18\x0e \x01(\tR\x06apicId\x12&\n" +
	"\x0finitial_apic_id\x18\x0f \x01(\tR\rinitialApicId\x12\x10\n" +
	"\x03fpu\x18\x10 \x01(\tR\x03fpu\x12#\n" +
	"\rfpu_exception\x18\x11 \x01(\tR\ffpuException\x12 \n" +
	"\fcpu_id_level\x18\x12 \x01(\rR\n" +
	"cpuIdLevel\x12\x0e\n" +
	"\x02wp\x18\x13 \x01(\tR\x02wp\x12\x14\n" +
	"\x05flags\x18\x14 \x03(\tR\x05flags\x12\x12\n" +
	"\x04bugs\x18\x15 \x03(\tR\x04bugs\x12\x1b\n" +
	"\tbogo_mips\x18\x16 \x01(\x01R\bbogoMips\x12\"\n" +
	"\rcl_flush_size\x18\x17 \x01(\rR\vclFlushSize\x12'\n" +
	"\x0fcache_alignment\x18\x18 \x01(\rR\x0ecacheAlignment\x12#\n" +
	"\raddress_sizes\x18\x19 \x01(\tR\faddressSizes\x12)\n" +
	"\x10power_management\x18\x1a \x01(\tR\x0fpowerManagement\"U\n" +
	"\x1aNetworkDeviceStatsResponse\x127\n" +
	"\bmessages\x18\x01 \x03(\v2\x1b.machine.NetworkDeviceStatsR\bmessages\"\x94\x01\n" +
	"\x12NetworkDeviceStats\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12%\n" +
	"\x05total\x18\x02 \x01(\v2\x0f.machine.NetDevR\x05total\x12)\n" +
	"\adevices\x18\x03 \x03(\v2\x0f.machine.NetDevR\adevices\"\x86\x04\n" +
	"\x06NetDev\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x19\n" +
	"\brx_bytes\x18\x02 \x01(\x04R\arxBytes\x12\x1d\n" +
	"\n" +
	"rx_packets\x18\x03 \x01(\x04R\trxPackets\x12\x1b\n" +
	"\trx_errors\x18\x04 \x01(\x04R\brxErrors\x12\x1d\n" +
	"\n" +
	"rx_dropped\x18\x05 \x01(\x04R\trxDropped\x12\x17\n" +
	"\arx_fifo\x18\x06 \x01(\x04R\x06rxFifo\x12\x19\n" +
	"\brx_frame\x18\a \x01(\x04R\arxFrame\x12#\n" +
	"\rrx_compressed\x18\b \x01(\x04R\frxCompressed\x12!\n" +
	"\frx_multicast\x18\t \x01(\x04R\vrxMulticast\x12\x19\n" +
	"\btx_bytes\x18\n" +
	" \x01(\x04R\atxBytes\x12\x1d\n" +
	"\n" +
	"tx_packets\x18\v \x01(\x04R\ttxPackets\x12\x1b\n" +
	"\ttx_errors\x18\f \x01(\x04R\btxErrors\x12\x1d\n" +
	"\n" +
	"tx_dropped\x18\r \x01(\x04R\ttxDropped\x12\x17\n" +
	"\atx_fifo\x18\x0e \x01(\x04R\x06txFifo\x12#\n" +
	"\rtx_collisions\x18\x0f \x01(\x04R\ftxCollisions\x12\x1d\n" +
	"\n" +
	"tx_carrier\x18\x10 \x01(\x04R\ttxCarrier\x12#\n" +
	"\rtx_compressed\x18\x11 \x01(\x04R\ftxCompressed\"C\n" +
	"\x11DiskStatsResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.DiskStatsR\bmessages\"\x8f\x01\n" +
	"\tDiskStats\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12'\n" +
	"\x05total\x18\x02 \x01(\v2\x11.machine.DiskStatR\x05total\x12+\n" +
	"\adevices\x18\x03 \x03(\v2\x11.machine.DiskStatR\adevices\"\xd8\x04\n" +
	"\bDiskStat\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12%\n" +
	"\x0eread_completed\x18\x02 \x01(\x04R\rreadCompleted\x12\x1f\n" +
	"\vread_merged\x18\x03 \x01(\x04R\n" +
	"readMerged\x12!\n" +
	"\fread_sectors\x18\x04 \x01(\x04R\vreadSectors\x12 \n" +
	"\fread_time_ms\x18\x05 \x01(\x04R\n" +
	"readTimeMs\x12'\n" +
	"\x0fwrite_completed\x18\x06 \x01(\x04R\x0ewriteCompleted\x12!\n" +
	"\fwrite_merged\x18\a \x01(\x04R\vwriteMerged\x12#\n" +
	"\rwrite_sectors\x18\b \x01(\x04R\fwriteSectors\x12\"\n" +
	"\rwrite_time_ms\x18\t \x01(\x04R\vwriteTimeMs\x12$\n" +
	"\x0eio_in_progress\x18\n" +
	" \x01(\x04R\fioInProgress\x12\x1c\n" +
	"\n" +
	"io_time_ms\x18\v \x01(\x04R\bioTimeMs\x12-\n" +
	"\x13io_time_weighted_ms\x18\f \x01(\x04R\x10ioTimeWeightedMs\x12+\n" +
	"\x11discard_completed\x18\r \x01(\x04R\x10discardCompleted\x12%\n" +
	"\x0ediscard_merged\x18\x0e \x01(\x04R\rdiscardMerged\x12'\n" +
	"\x0fdiscard_sectors\x18\x0f \x01(\x04R\x0ediscardSectors\x12&\n" +
	"\x0fdiscard_time_ms\x18\x10 \x01(\x04R\rdiscardTimeMs\"\x19\n" +
	"\x17EtcdLeaveClusterRequest\"@\n" +
	"\x10EtcdLeaveCluster\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"Q\n" +
	"\x18EtcdLeaveClusterResponse\x125\n" +
	"\bmessages\x18\x01 \x03(\v2\x19.machine.EtcdLeaveClusterR\bmessages\"1\n" +
	"\x17EtcdRemoveMemberRequest\x12\x16\n" +
	"\x06member\x18\x01 \x01(\tR\x06member\"@\n" +
	"\x10EtcdRemoveMember\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"Q\n" +
	"\x18EtcdRemoveMemberResponse\x125\n" +
	"\bmessages\x18\x01 \x03(\v2\x19.machine.EtcdRemoveMemberR\bmessages\":\n" +
	"\x1bEtcdRemoveMemberByIDRequest\x12\x1b\n" +
	"\tmember_id\x18\x01 \x01(\x04R\bmemberId\"D\n" +
	"\x14EtcdRemoveMemberByID\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"Y\n" +
	"\x1cEtcdRemoveMemberByIDResponse\x129\n" +
	"\bmessages\x18\x01 \x03(\v2\x1d.machine.EtcdRemoveMemberByIDR\bmessages\"\x1e\n" +
	"\x1cEtcdForfeitLeadershipRequest\"]\n" +
	"\x15EtcdForfeitLeadership\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x16\n" +
	"\x06member\x18\x02 \x01(\tR\x06member\"[\n" +
	"\x1dEtcdForfeitLeadershipResponse\x12:\n" +
	"\bmessages\x18\x01 \x03(\v2\x1e.machine.EtcdForfeitLeadershipR\bmessages\"8\n" +
	"\x15EtcdMemberListRequest\x12\x1f\n" +
	"\vquery_local\x18\x01 \x01(\bR\n" +
	"queryLocal\"\x95\x01\n" +
	"\n" +
	"EtcdMember\x12\x0e\n" +
	"\x02id\x18\x02 \x01(\x04R\x02id\x12\x1a\n" +
	"\bhostname\x18\x03 \x01(\tR\bhostname\x12\x1b\n" +
	"\tpeer_urls\x18\x04 \x03(\tR\bpeerUrls\x12\x1f\n" +
	"\vclient_urls\x18\x05 \x03(\tR\n" +
	"clientUrls\x12\x1d\n" +
	"\n" +
	"is_learner\x18\x06 \x01(\bR\tisLearner\"\x91\x01\n" +
	"\vEtcdMembers\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12%\n" +
	"\x0elegacy_members\x18\x02 \x03(\tR\rlegacyMembers\x12-\n" +
	"\amembers\x18\x03 \x03(\v2\x13.machine.EtcdMemberR\amembers\"J\n" +
	"\x16EtcdMemberListResponse\x120\n" +
	"\bmessages\x18\x01 \x03(\v2\x14.machine.EtcdMembersR\bmessages\"\x15\n" +
	"\x13EtcdSnapshotRequest\";\n" +
	"\vEtcdRecover\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"G\n" +
	"\x13EtcdRecoverResponse\x120\n" +
	"\bmessages\x18\x01 \x03(\v2\x14.machine.EtcdRecoverR\bmessages\"G\n" +
	"\x15EtcdAlarmListResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.EtcdAlarmR\bmessages\"x\n" +
	"\tEtcdAlarm\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12=\n" +
	"\rmember_alarms\x18\x02 \x03(\v2\x18.machine.EtcdMemberAlarmR\fmemberAlarms\"\x99\x01\n" +
	"\x0fEtcdMemberAlarm\x12\x1b\n" +
	"\tmember_id\x18\x01 \x01(\x04R\bmemberId\x128\n" +
	"\x05alarm\x18\x02 \x01(\x0e2\".machine.EtcdMemberAlarm.AlarmTypeR\x05alarm\"/\n" +
	"\tAlarmType\x12\b\n" +
	"\x04NONE\x10\x00\x12\v\n" +
	"\aNOSPACE\x10\x01\x12\v\n" +
	"\aCORRUPT\x10\x02\"O\n" +
	"\x17EtcdAlarmDisarmResponse\x124\n" +
	"\bmessages\x18\x01 \x03(\v2\x18.machine.EtcdAlarmDisarmR\bmessages\"~\n" +
	"\x0fEtcdAlarmDisarm\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12=\n" +
	"\rmember_alarms\x18\x02 \x03(\v2\x18.machine.EtcdMemberAlarmR\fmemberAlarms\"M\n" +
	"\x16EtcdDefragmentResponse\x123\n" +
	"\bmessages\x18\x01 \x03(\v2\x17.machine.EtcdDefragmentR\bmessages\">\n" +
	"\x0eEtcdDefragment\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"E\n" +
	"\x12EtcdStatusResponse\x12/\n" +
	"\bmessages\x18\x01 \x03(\v2\x13.machine.EtcdStatusR\bmessages\"z\n" +
	"\n" +
	"EtcdStatus\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12>\n" +
	"\rmember_status\x18\x02 \x01(\v2\x19.machine.EtcdMemberStatusR\fmemberStatus\"\xfa\x02\n" +
	"\x10EtcdMemberStatus\x12'\n" +
	"\x0fstorage_version\x18\v \x01(\tR\x0estorageVersion\x12\x1b\n" +
	"\tmember_id\x18\n" +
	" \x01(\x04R\bmemberId\x12)\n" +
	"\x10protocol_version\x18\x01 \x01(\tR\x0fprotocolVersion\x12\x17\n" +
	"\adb_size\x18\x02 \x01(\x03R\x06dbSize\x12#\n" +
	"\x0edb_size_in_use\x18\x03 \x01(\x03R\vdbSizeInUse\x12\x16\n" +
	"\x06leader\x18\x04 \x01(\x04R\x06leader\x12\x1d\n" +
	"\n" +
	"raft_index\x18\x05 \x01(\x04R\traftIndex\x12\x1b\n" +
	"\traft_term\x18\x06 \x01(\x04R\braftTerm\x12,\n" +
	"\x12raft_applied_index\x18\a \x01(\x04R\x10raftAppliedIndex\x12\x16\n" +
	"\x06errors\x18\b \x03(\tR\x06errors\x12\x1d\n" +
	"\n" +
	"is_learner\x18\t \x01(\bR\tisLearner\"8\n" +
	"\x1cEtcdDowngradeValidateRequest\x12\x18\n" +
	"\aversion\x18\x01 \x01(\tR\aversion\"[\n" +
	"\x1dEtcdDowngradeValidateResponse\x12:\n" +
	"\bmessages\x18\x01 \x03(\v2\x1e.machine.EtcdDowngradeValidateR\bmessages\"\x91\x01\n" +
	"\x15EtcdDowngradeValidate\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12J\n" +
	"\x11cluster_downgrade\x18\x02 \x01(\v2\x1d.machine.EtcdClusterDowngradeR\x10clusterDowngrade\"6\n" +
	"\x1aEtcdDowngradeEnableRequest\x12\x18\n" +
	"\aversion\x18\x01 \x01(\tR\aversion\"W\n" +
	"\x1bEtcdDowngradeEnableResponse\x128\n" +
	"\bmessages\x18\x01 \x03(\v2\x1c.machine.EtcdDowngradeEnableR\bmessages\"\x8f\x01\n" +
	"\x13EtcdDowngradeEnable\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12J\n" +
	"\x11cluster_downgrade\x18\x02 \x01(\v2\x1d.machine.EtcdClusterDowngradeR\x10clusterDowngrade\"W\n" +
	"\x1bEtcdDowngradeCancelResponse\x128\n" +
	"\bmessages\x18\x01 \x03(\v2\x1c.machine.EtcdDowngradeCancelR\bmessages\"\x8f\x01\n" +
	"\x13EtcdDowngradeCancel\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12J\n" +
	"\x11cluster_downgrade\x18\x02 \x01(\v2\x1d.machine.EtcdClusterDowngradeR\x10clusterDowngrade\"?\n" +
	"\x14EtcdClusterDowngrade\x12'\n" +
	"\x0fcluster_version\x18\x01 \x01(\tR\x0eclusterVersion\"Y\n" +
	"\vRouteConfig\x12\x18\n" +
	"\anetwork\x18\x01 \x01(\tR\anetwork\x12\x18\n" +
	"\agateway\x18\x02 \x01(\tR\agateway\x12\x16\n" +
	"\x06metric\x18\x03 \x01(\rR\x06metric\"6\n" +
	"\x11DHCPOptionsConfig\x12!\n" +
	"\froute_metric\x18\x01 \x01(\rR\vrouteMetric\"\xf2\x01\n" +
	"\x13NetworkDeviceConfig\x12\x1c\n" +
	"\tinterface\x18\x01 \x01(\tR\tinterface\x12\x12\n" +
	"\x04cidr\x18\x02 \x01(\tR\x04cidr\x12\x10\n" +
	"\x03mtu\x18\x03 \x01(\x05R\x03mtu\x12\x12\n" +
	"\x04dhcp\x18\x04 \x01(\bR\x04dhcp\x12\x16\n" +
	"\x06ignore\x18\x05 \x01(\bR\x06ignore\x12=\n" +
	"\fdhcp_options\x18\x06 \x01(\v2\x1a.machine.DHCPOptionsConfigR\vdhcpOptions\x12,\n" +
	"\x06routes\x18\a \x03(\v2\x14.machine.RouteConfigR\x06routes\"i\n" +
	"\rNetworkConfig\x12\x1a\n" +
	"\bhostname\x18\x01 \x01(\tR\bhostname\x12<\n" +
	"\n" +
	"interfaces\x18\x02 \x03(\v2\x1c.machine.NetworkDeviceConfigR\n" +
	"interfaces\"W\n" +
	"\rInstallConfig\x12!\n" +
	"\finstall_disk\x18\x01 \x01(\tR\vinstallDisk\x12#\n" +
	"\rinstall_image\x18\x02 \x01(\tR\finstallImage\"\xcd\x02\n" +
	"\rMachineConfig\x126\n" +
	"\x04type\x18\x01 \x01(\x0e2\".machine.MachineConfig.MachineTypeR\x04type\x12=\n" +
	"\x0einstall_config\x18\x02 \x01(\v2\x16.machine.InstallConfigR\rinstallConfig\x12=\n" +
	"\x0enetwork_config\x18\x03 \x01(\v2\x16.machine.NetworkConfigR\rnetworkConfig\x12-\n" +
	"\x12kubernetes_version\x18\x04 \x01(\tR\x11kubernetesVersion\"W\n" +
	"\vMachineType\x12\x10\n" +
	"\fTYPE_UNKNOWN\x10\x00\x12\r\n" +
	"\tTYPE_INIT\x10\x01\x12\x16\n" +
	"\x12TYPE_CONTROL_PLANE\x10\x02\x12\x0f\n" +
	"\vTYPE_WORKER\x10\x03\"0\n" +
	"\x12ControlPlaneConfig\x12\x1a\n" +
	"\bendpoint\x18\x01 \x01(\tR\bendpoint\"3\n" +
	"\tCNIConfig\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12\x12\n" +
	"\x04urls\x18\x02 \x03(\tR\x04urls\"h\n" +
	"\x14ClusterNetworkConfig\x12\x1d\n" +
	"\n" +
	"dns_domain\x18\x01 \x01(\tR\tdnsDomain\x121\n" +
	"\n" +
	"cni_config\x18\x02 \x01(\v2\x12.machine.CNIConfigR\tcniConfig\"\xf9\x01\n" +
	"\rClusterConfig\x12\x12\n" +
	"\x04name\x18\x01 \x01(\tR\x04name\x12@\n" +
	"\rcontrol_plane\x18\x02 \x01(\v2\x1b.machine.ControlPlaneConfigR\fcontrolPlane\x12F\n" +
	"\x0fcluster_network\x18\x03 \x01(\v2\x1d.machine.ClusterNetworkConfigR\x0eclusterNetwork\x12J\n" +
	"\"allow_scheduling_on_control_planes\x18\x04 \x01(\bR\x1eallowSchedulingOnControlPlanes\"n\n" +
	"\"GenerateClientConfigurationRequest\x12\x14\n" +
	"\x05roles\x18\x01 \x03(\tR\x05roles\x122\n" +
	"\acrt_ttl\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\x06crtTtl\"\xa1\x01\n" +
	"\x1bGenerateClientConfiguration\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x0e\n" +
	"\x02ca\x18\x02 \x01(\fR\x02ca\x12\x10\n" +
	"\x03crt\x18\x03 \x01(\fR\x03crt\x12\x10\n" +
	"\x03key\x18\x04 \x01(\fR\x03key\x12 \n" +
	"\vtalosconfig\x18\x05 \x01(\fR\vtalosconfig\"g\n" +
	"#GenerateClientConfigurationResponse\x12@\n" +
	"\bmessages\x18\x01 \x03(\v2$.machine.GenerateClientConfigurationR\bmessages\"\xa9\x01\n" +
	"\x14PacketCaptureRequest\x12\x1c\n" +
	"\tinterface\x18\x01 \x01(\tR\tinterface\x12 \n" +
	"\vpromiscuous\x18\x02 \x01(\bR\vpromiscuous\x12\x19\n" +
	"\bsnap_len\x18\x03 \x01(\rR\asnapLen\x126\n" +
	"\n" +
	"bpf_filter\x18\x04 \x03(\v2\x17.machine.BPFInstructionR\tbpfFilter\"N\n" +
	"\x0eBPFInstruction\x12\x0e\n" +
	"\x02op\x18\x01 \x01(\rR\x02op\x12\x0e\n" +
	"\x02jt\x18\x02 \x01(\rR\x02jt\x12\x0e\n" +
	"\x02jf\x18\x03 \x01(\rR\x02jf\x12\f\n" +
	"\x01k\x18\x04 \x01(\rR\x01k\"\xd2\x04\n" +
	"\x0eNetstatRequest\x126\n" +
	"\x06filter\x18\x01 \x01(\x0e2\x1e.machine.NetstatRequest.FilterR\x06filter\x129\n" +
	"\afeature\x18\x02 \x01(\v2\x1f.machine.NetstatRequest.FeatureR\afeature\x129\n" +
	"\al4proto\x18\x03 \x01(\v2\x1f.machine.NetstatRequest.L4protoR\al4proto\x123\n" +
	"\x05netns\x18\x04 \x01(\v2\x1d.machine.NetstatRequest.NetNSR\x05netns\x1a\x1b\n" +
	"\aFeature\x12\x10\n" +
	"\x03pid\x18\x01 \x01(\bR\x03pid\x1a\xb1\x01\n" +
	"\aL4proto\x12\x10\n" +
	"\x03tcp\x18\x01 \x01(\bR\x03tcp\x12\x12\n" +
	"\x04tcp6\x18\x02 \x01(\bR\x04tcp6\x12\x10\n" +
	"\x03udp\x18\x03 \x01(\bR\x03udp\x12\x12\n" +
	"\x04udp6\x18\x04 \x01(\bR\x04udp6\x12\x18\n" +
	"\audplite\x18\x05 \x01(\bR\audplite\x12\x1a\n" +
	"\budplite6\x18\x06 \x01(\bR\budplite6\x12\x10\n" +
	"\x03raw\x18\a \x01(\bR\x03raw\x12\x12\n" +
	"\x04raw6\x18\b \x01(\bR\x04raw6\x1a[\n" +
	"\x05NetNS\x12 \n" +
	"\vhostnetwork\x18\x01 \x01(\bR\vhostnetwork\x12\x14\n" +
	"\x05netns\x18\x02 \x03(\tR\x05netns\x12\x1a\n" +
	"\ballnetns\x18\x03 \x01(\bR\ballnetns\"/\n" +
	"\x06Filter\x12\a\n" +
	"\x03ALL\x10\x00\x12\r\n" +
	"\tCONNECTED\x10\x01\x12\r\n" +
	"\tLISTENING\x10\x02\"\xdc\x06\n" +
	"\rConnectRecord\x12\x18\n" +
	"\al4proto\x18\x01 \x01(\tR\al4proto\x12\x18\n" +
	"\alocalip\x18\x02 \x01(\tR\alocalip\x12\x1c\n" +
	"\tlocalport\x18\x03 \x01(\rR\tlocalport\x12\x1a\n" +
	"\bremoteip\x18\x04 \x01(\tR\bremoteip\x12\x1e\n" +
	"\n" +
	"remoteport\x18\x05 \x01(\rR\n" +
	"remoteport\x122\n" +
	"\x05state\x18\x06 \x01(\x0e2\x1c.machine.ConnectRecord.StateR\x05state\x12\x18\n" +
	"\atxqueue\x18\a \x01(\x04R\atxqueue\x12\x18\n" +
	"\arxqueue\x18\b \x01(\x04R\arxqueue\x122\n" +
	"\x02tr\x18\t \x01(\x0e2\".machine.ConnectRecord.TimerActiveR\x02tr\x12\x1c\n" +
	"\ttimerwhen\x18\n" +
	" \x01(\x04R\ttimerwhen\x12\x1a\n" +
	"\bretrnsmt\x18\v \x01(\x04R\bretrnsmt\x12\x10\n" +
	"\x03uid\x18\f \x01(\rR\x03uid\x12\x18\n" +
	"\atimeout\x18\r \x01(\x04R\atimeout\x12\x14\n" +
	"\x05inode\x18\x0e \x01(\x04R\x05inode\x12\x10\n" +
	"\x03ref\x18\x0f \x01(\x04R\x03ref\x12\x18\n" +
	"\apointer\x18\x10 \x01(\x04R\apointer\x128\n" +
	"\aprocess\x18\x11 \x01(\v2\x1e.machine.ConnectRecord.ProcessR\aprocess\x12\x14\n" +
	"\x05netns\x18\x12 \x01(\tR\x05netns\x1a/\n" +
	"\aProcess\x12\x10\n" +
	"\x03pid\x18\x01 \x01(\rR\x03pid\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\"\xaf\x01\n" +
	"\x05State\x12\f\n" +
	"\bRESERVED\x10\x00\x12\x0f\n" +
	"\vESTABLISHED\x10\x01\x12\f\n" +
	"\bSYN_SENT\x10\x02\x12\f\n" +
	"\bSYN_RECV\x10\x03\x12\r\n" +
	"\tFIN_WAIT1\x10\x04\x12\r\n" +
	"\tFIN_WAIT2\x10\x05\x12\r\n" +
	"\tTIME_WAIT\x10\x06\x12\t\n" +
	"\x05CLOSE\x10\a\x12\r\n" +
	"\tCLOSEWAIT\x10\b\x12\v\n" +
	"\aLASTACK\x10\t\x12\n" +
	"\n" +
	"\x06LISTEN\x10\n" +
	"\x12\v\n" +
	"\aCLOSING\x10\v\"F\n" +
	"\vTimerActive\x12\a\n" +
	"\x03OFF\x10\x00\x12\x06\n" +
	"\x02ON\x10\x01\x12\r\n" +
	"\tKEEPALIVE\x10\x02\x12\f\n" +
	"\bTIMEWAIT\x10\x03\x12\t\n" +
	"\x05PROBE\x10\x04\"u\n" +
	"\aNetstat\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12<\n" +
	"\rconnectrecord\x18\x02 \x03(\v2\x16.machine.ConnectRecordR\rconnectrecord\"?\n" +
	"\x0fNetstatResponse\x12,\n" +
	"\bmessages\x18\x01 \x03(\v2\x10.machine.NetstatR\bmessages\":\n" +
	"\x10MetaWriteRequest\x12\x10\n" +
	"\x03key\x18\x01 \x01(\rR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\fR\x05value\"9\n" +
	"\tMetaWrite\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"C\n" +
	"\x11MetaWriteResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.MetaWriteR\bmessages\"%\n" +
	"\x11MetaDeleteRequest\x12\x10\n" +
	"\x03key\x18\x01 \x01(\rR\x03key\":\n" +
	"\n" +
	"MetaDelete\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"E\n" +
	"\x12MetaDeleteResponse\x12/\n" +
	"\bmessages\x18\x01 \x03(\v2\x13.machine.MetaDeleteR\bmessages\"M\n" +
	"\x10ImageListRequest\x129\n" +
	"\tnamespace\x18\x01 \x01(\x0e2\x1b.common.ContainerdNamespaceR\tnamespace\"\xbc\x01\n" +
	"\x11ImageListResponse\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\x12\x16\n" +
	"\x06digest\x18\x03 \x01(\tR\x06digest\x12\x12\n" +
	"\x04size\x18\x04 \x01(\x03R\x04size\x129\n" +
	"\n" +
	"created_at\x18\x05 \x01(\v2\x1a.google.protobuf.TimestampR\tcreatedAt\"k\n" +
	"\x10ImagePullRequest\x129\n" +
	"\tnamespace\x18\x01 \x01(\x0e2\x1b.common.ContainerdNamespaceR\tnamespace\x12\x1c\n" +
	"\treference\x18\x02 \x01(\tR\treference\"9\n" +
	"\tImagePull\x12,\n" +
	"\bmetadata\x18\x01 \x01(\v2\x10.common.MetadataR\bmetadata\"C\n" +
	"\x11ImagePullResponse\x12.\n" +
	"\bmessages\x18\x01 \x03(\v2\x12.machine.ImagePullR\bmessages2\xef\x1d\n" +
	"\x0eMachineService\x12]\n" +
	"\x12ApplyConfiguration\x12\".machine.ApplyConfigurationRequest\x1a#.machine.ApplyConfigurationResponse\x12B\n" +
	"\tBootstrap\x12\x19.machine.BootstrapRequest\x1a\x1a.machine.BootstrapResponse\x12E\n" +
	"\n" +
	"Containers\x12\x1a.machine.ContainersRequest\x1a\x1b.machine.ContainersResponse\x12,\n" +
	"\x04Copy\x12\x14.machine.CopyRequest\x1a\f.common.Data0\x01\x12E\n" +
	"\fCPUFreqStats\x12\x16.google.protobuf.Empty\x1a\x1d.machine.CPUFreqStatsResponse\x12;\n" +
	"\aCPUInfo\x12\x16.google.protobuf.Empty\x1a\x18.machine.CPUInfoResponse\x12?\n" +
	"\tDiskStats\x12\x16.google.protobuf.Empty\x1a\x1a.machine.DiskStatsResponse\x12.\n" +
	"\x05Dmesg\x12\x15.machine.DmesgRequest\x1a\f.common.Data0\x01\x122\n" +
	"\x06Events\x12\x16.machine.EventsRequest\x1a\x0e.machine.Event0\x01\x12Q\n" +
	"\x0eEtcdMemberList\x12\x1e.machine.EtcdMemberListRequest\x1a\x1f.machine.EtcdMemberListResponse\x12c\n" +
	"\x14EtcdRemoveMemberByID\x12$.machine.EtcdRemoveMemberByIDRequest\x1a%.machine.EtcdRemoveMemberByIDResponse\x12W\n" +
	"\x10EtcdLeaveCluster\x12 .machine.EtcdLeaveClusterRequest\x1a!.machine.EtcdLeaveClusterResponse\x12f\n" +
	"\x15EtcdForfeitLeadership\x12%.machine.EtcdForfeitLeadershipRequest\x1a&.machine.EtcdForfeitLeadershipResponse\x12;\n" +
	"\vEtcdRecover\x12\f.common.Data\x1a\x1c.machine.EtcdRecoverResponse(\x01\x12<\n" +
	"\fEtcdSnapshot\x12\x1c.machine.EtcdSnapshotRequest\x1a\f.common.Data0\x01\x12G\n" +
	"\rEtcdAlarmList\x12\x16.google.protobuf.Empty\x1a\x1e.machine.EtcdAlarmListResponse\x12K\n" +
	"\x0fEtcdAlarmDisarm\x12\x16.google.protobuf.Empty\x1a .machine.EtcdAlarmDisarmResponse\x12I\n" +
	"\x0eEtcdDefragment\x12\x16.google.protobuf.Empty\x1a\x1f.machine.EtcdDefragmentResponse\x12A\n" +
	"\n" +
	"EtcdStatus\x12\x16.google.protobuf.Empty\x1a\x1b.machine.EtcdStatusResponse\x12f\n" +
	"\x15EtcdDowngradeValidate\x12%.machine.EtcdDowngradeValidateRequest\x1a&.machine.EtcdDowngradeValidateResponse\x12`\n" +
	"\x13EtcdDowngradeEnable\x12#.machine.EtcdDowngradeEnableRequest\x1a$.machine.EtcdDowngradeEnableResponse\x12S\n" +
	"\x13EtcdDowngradeCancel\x12\x16.google.protobuf.Empty\x1a$.machine.EtcdDowngradeCancelResponse\x12=\n" +
	"\bHostname\x12\x16.google.protobuf.Empty\x1a\x19.machine.HostnameResponse\x124\n" +
	"\n" +
	"Kubeconfig\x12\x16.google.protobuf.Empty\x1a\f.common.Data0\x01\x121\n" +
	"\x04List\x12\x14.machine.ListRequest\x1a\x11.machine.FileInfo0\x01\x12@\n" +
	"\tDiskUsage\x12\x19.machine.DiskUsageRequest\x1a\x16.machine.DiskUsageInfo0\x01\x12;\n" +
	"\aLoadAvg\x12\x16.google.protobuf.Empty\x1a\x18.machine.LoadAvgResponse\x12,\n" +
	"\x04Logs\x12\x14.machine.LogsRequest\x1a\f.common.Data0\x01\x12I\n" +
	"\x0eLogsContainers\x12\x16.google.protobuf.Empty\x1a\x1f.machine.LogsContainersResponse\x129\n" +
	"\x06Memory\x12\x16.google.protobuf.Empty\x1a\x17.machine.MemoryResponse\x129\n" +
	"\x06Mounts\x12\x16.google.protobuf.Empty\x1a\x17.machine.MountsResponse\x12Q\n" +
	"\x12NetworkDeviceStats\x12\x16.google.protobuf.Empty\x1a#.machine.NetworkDeviceStatsResponse\x12?\n" +
	"\tProcesses\x12\x16.google.protobuf.Empty\x1a\x1a.machine.ProcessesResponse\x12,\n" +
	"\x04Read\x12\x14.machine.ReadRequest\x1a\f.common.Data0\x01\x129\n" +
	"\x06Reboot\x12\x16.machine.RebootRequest\x1a\x17.machine.RebootResponse\x12<\n" +
	"\aRestart\x12\x17.machine.RestartRequest\x1a\x18.machine.RestartResponse\x12?\n" +
	"\bRollback\x12\x18.machine.RollbackRequest\x1a\x19.machine.RollbackResponse\x126\n" +
	"\x05Reset\x12\x15.machine.ResetRequest\x1a\x16.machine.ResetResponse\x12C\n" +
	"\vServiceList\x12\x16.google.protobuf.Empty\x1a\x1c.machine.ServiceListResponse\x12Q\n" +
	"\x0eServiceRestart\x12\x1e.machine.ServiceRestartRequest\x1a\x1f.machine.ServiceRestartResponse\x12K\n" +
	"\fServiceStart\x12\x1c.machine.ServiceStartRequest\x1a\x1d.machine.ServiceStartResponse\x12H\n" +
	"\vServiceStop\x12\x1b.machine.ServiceStopRequest\x1a\x1c.machine.ServiceStopResponse\x12?\n" +
	"\bShutdown\x12\x18.machine.ShutdownRequest\x1a\x19.machine.ShutdownResponse\x126\n" +
	"\x05Stats\x12\x15.machine.StatsRequest\x1a\x16.machine.StatsResponse\x12A\n" +
	"\n" +
	"SystemStat\x12\x16.google.protobuf.Empty\x1a\x1b.machine.SystemStatResponse\x12J\n" +
	"\aUpgrade\x12\x17.machine.UpgradeRequest\x1a\x18.machine.UpgradeResponse\"\f\xea\xbb-\x05v1.18\x88\x02\x01\x12;\n" +
	"\aVersion\x12\x16.google.protobuf.Empty\x1a\x18.machine.VersionResponse\x12x\n" +
	"\x1bGenerateClientConfiguration\x12+.machine.GenerateClientConfigurationRequest\x1a,.machine.GenerateClientConfigurationResponse\x12>\n" +
	"\rPacketCapture\x12\x1d.machine.PacketCaptureRequest\x1a\f.common.Data0\x01\x12<\n" +
	"\aNetstat\x12\x17.machine.NetstatRequest\x1a\x18.machine.NetstatResponse\x12B\n" +
	"\tMetaWrite\x12\x19.machine.MetaWriteRequest\x1a\x1a.machine.MetaWriteResponse\x12E\n" +
	"\n" +
	"MetaDelete\x12\x1a.machine.MetaDeleteRequest\x1a\x1b.machine.MetaDeleteResponse\x12R\n" +
	"\tImageList\x12\x19.machine.ImageListRequest\x1a\x1a.machine.ImageListResponse\"\f\xea\xbb-\x05v1.18\x88\x02\x010\x01\x12P\n" +
	"\tImagePull\x12\x19.machine.ImagePullRequest\x1a\x1a.machine.ImagePullResponse\"\f\xea\xbb-\x05v1.18\x88\x02\x01BN\n" +
	"\x15dev.talos.api.machineZ5github.com/siderolabs/talos/pkg/machinery/api/machineb\x06proto3"

var (
	file_machine_machine_proto_rawDescOnce sync.Once
	file_machine_machine_proto_rawDescData []byte
)

func file_machine_machine_proto_rawDescGZIP() []byte {
	file_machine_machine_proto_rawDescOnce.Do(func() {
		file_machine_machine_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_machine_machine_proto_rawDesc), len(file_machine_machine_proto_rawDesc)))
	})
	return file_machine_machine_proto_rawDescData
}

var file_machine_machine_proto_enumTypes = make([]protoimpl.EnumInfo, 15)
var file_machine_machine_proto_msgTypes = make([]protoimpl.MessageInfo, 178)
var file_machine_machine_proto_goTypes = []any{
	(ApplyConfigurationRequest_Mode)(0),                     // 0: machine.ApplyConfigurationRequest.Mode
	(RebootRequest_Mode)(0),                                 // 1: machine.RebootRequest.Mode
	(SequenceEvent_Action)(0),                               // 2: machine.SequenceEvent.Action
	(PhaseEvent_Action)(0),                                  // 3: machine.PhaseEvent.Action
	(TaskEvent_Action)(0),                                   // 4: machine.TaskEvent.Action
	(ServiceStateEvent_Action)(0),                           // 5: machine.ServiceStateEvent.Action
	(MachineStatusEvent_MachineStage)(0),                    // 6: machine.MachineStatusEvent.MachineStage
	(ResetRequest_WipeMode)(0),                              // 7: machine.ResetRequest.WipeMode
	(UpgradeRequest_RebootMode)(0),                          // 8: machine.UpgradeRequest.RebootMode
	(ListRequest_Type)(0),                                   // 9: machine.ListRequest.Type
	(EtcdMemberAlarm_AlarmType)(0),                          // 10: machine.EtcdMemberAlarm.AlarmType
	(MachineConfig_MachineType)(0),                          // 11: machine.MachineConfig.MachineType
	(NetstatRequest_Filter)(0),                              // 12: machine.NetstatRequest.Filter
	(ConnectRecord_State)(0),                                // 13: machine.ConnectRecord.State
	(ConnectRecord_TimerActive)(0),                          // 14: machine.ConnectRecord.TimerActive
	(*ApplyConfigurationRequest)(nil),                       // 15: machine.ApplyConfigurationRequest
	(*ApplyConfiguration)(nil),                              // 16: machine.ApplyConfiguration
	(*ApplyConfigurationResponse)(nil),                      // 17: machine.ApplyConfigurationResponse
	(*RebootRequest)(nil),                                   // 18: machine.RebootRequest
	(*Reboot)(nil),                                          // 19: machine.Reboot
	(*RebootResponse)(nil),                                  // 20: machine.RebootResponse
	(*BootstrapRequest)(nil),                                // 21: machine.BootstrapRequest
	(*Bootstrap)(nil),                                       // 22: machine.Bootstrap
	(*BootstrapResponse)(nil),                               // 23: machine.BootstrapResponse
	(*SequenceEvent)(nil),                                   // 24: machine.SequenceEvent
	(*PhaseEvent)(nil),                                      // 25: machine.PhaseEvent
	(*TaskEvent)(nil),                                       // 26: machine.TaskEvent
	(*ServiceStateEvent)(nil),                               // 27: machine.ServiceStateEvent
	(*RestartEvent)(nil),                                    // 28: machine.RestartEvent
	(*ConfigLoadErrorEvent)(nil),                            // 29: machine.ConfigLoadErrorEvent
	(*ConfigValidationErrorEvent)(nil),                      // 30: machine.ConfigValidationErrorEvent
	(*AddressEvent)(nil),                                    // 31: machine.AddressEvent
	(*MachineStatusEvent)(nil),                              // 32: machine.MachineStatusEvent
	(*EventsRequest)(nil),                                   // 33: machine.EventsRequest
	(*Event)(nil),                                           // 34: machine.Event
	(*ResetPartitionSpec)(nil),                              // 35: machine.ResetPartitionSpec
	(*ResetRequest)(nil),                                    // 36: machine.ResetRequest
	(*Reset)(nil),                                           // 37: machine.Reset
	(*ResetResponse)(nil),                                   // 38: machine.ResetResponse
	(*Shutdown)(nil),                                        // 39: machine.Shutdown
	(*ShutdownRequest)(nil),                                 // 40: machine.ShutdownRequest
	(*ShutdownResponse)(nil),                                // 41: machine.ShutdownResponse
	(*UpgradeRequest)(nil),                                  // 42: machine.UpgradeRequest
	(*Upgrade)(nil),                                         // 43: machine.Upgrade
	(*UpgradeResponse)(nil),                                 // 44: machine.UpgradeResponse
	(*ServiceList)(nil),                                     // 45: machine.ServiceList
	(*ServiceListResponse)(nil),                             // 46: machine.ServiceListResponse
	(*ServiceInfo)(nil),                                     // 47: machine.ServiceInfo
	(*ServiceEvents)(nil),                                   // 48: machine.ServiceEvents
	(*ServiceEvent)(nil),                                    // 49: machine.ServiceEvent
	(*ServiceHealth)(nil),                                   // 50: machine.ServiceHealth
	(*ServiceStartRequest)(nil),                             // 51: machine.ServiceStartRequest
	(*ServiceStart)(nil),                                    // 52: machine.ServiceStart
	(*ServiceStartResponse)(nil),                            // 53: machine.ServiceStartResponse
	(*ServiceStopRequest)(nil),                              // 54: machine.ServiceStopRequest
	(*ServiceStop)(nil),                                     // 55: machine.ServiceStop
	(*ServiceStopResponse)(nil),                             // 56: machine.ServiceStopResponse
	(*ServiceRestartRequest)(nil),                           // 57: machine.ServiceRestartRequest
	(*ServiceRestart)(nil),                                  // 58: machine.ServiceRestart
	(*ServiceRestartResponse)(nil),                          // 59: machine.ServiceRestartResponse
	(*CopyRequest)(nil),                                     // 60: machine.CopyRequest
	(*ListRequest)(nil),                                     // 61: machine.ListRequest
	(*DiskUsageRequest)(nil),                                // 62: machine.DiskUsageRequest
	(*FileInfo)(nil),                                        // 63: machine.FileInfo
	(*Xattr)(nil),                                           // 64: machine.Xattr
	(*DiskUsageInfo)(nil),                                   // 65: machine.DiskUsageInfo
	(*Mounts)(nil),                                          // 66: machine.Mounts
	(*MountsResponse)(nil),                                  // 67: machine.MountsResponse
	(*MountStat)(nil),                                       // 68: machine.MountStat
	(*Version)(nil),                                         // 69: machine.Version
	(*VersionResponse)(nil),                                 // 70: machine.VersionResponse
	(*VersionInfo)(nil),                                     // 71: machine.VersionInfo
	(*PlatformInfo)(nil),                                    // 72: machine.PlatformInfo
	(*FeaturesInfo)(nil),                                    // 73: machine.FeaturesInfo
	(*LogsRequest)(nil),                                     // 74: machine.LogsRequest
	(*ReadRequest)(nil),                                     // 75: machine.ReadRequest
	(*LogsContainer)(nil),                                   // 76: machine.LogsContainer
	(*LogsContainersResponse)(nil),                          // 77: machine.LogsContainersResponse
	(*RollbackRequest)(nil),                                 // 78: machine.RollbackRequest
	(*Rollback)(nil),                                        // 79: machine.Rollback
	(*RollbackResponse)(nil),                                // 80: machine.RollbackResponse
	(*ContainersRequest)(nil),                               // 81: machine.ContainersRequest
	(*ContainerInfo)(nil),                                   // 82: machine.ContainerInfo
	(*Container)(nil),                                       // 83: machine.Container
	(*ContainersResponse)(nil),                              // 84: machine.ContainersResponse
	(*DmesgRequest)(nil),                                    // 85: machine.DmesgRequest
	(*ProcessesResponse)(nil),                               // 86: machine.ProcessesResponse
	(*Process)(nil),                                         // 87: machine.Process
	(*ProcessInfo)(nil),                                     // 88: machine.ProcessInfo
	(*RestartRequest)(nil),                                  // 89: machine.RestartRequest
	(*Restart)(nil),                                         // 90: machine.Restart
	(*RestartResponse)(nil),                                 // 91: machine.RestartResponse
	(*StatsRequest)(nil),                                    // 92: machine.StatsRequest
	(*Stats)(nil),                                           // 93: machine.Stats
	(*StatsResponse)(nil),                                   // 94: machine.StatsResponse
	(*Stat)(nil),                                            // 95: machine.Stat
	(*Memory)(nil),                                          // 96: machine.Memory
	(*MemoryResponse)(nil),                                  // 97: machine.MemoryResponse
	(*MemInfo)(nil),                                         // 98: machine.MemInfo
	(*HostnameResponse)(nil),                                // 99: machine.HostnameResponse
	(*Hostname)(nil),                                        // 100: machine.Hostname
	(*LoadAvgResponse)(nil),                                 // 101: machine.LoadAvgResponse
	(*LoadAvg)(nil),                                         // 102: machine.LoadAvg
	(*SystemStatResponse)(nil),                              // 103: machine.SystemStatResponse
	(*SystemStat)(nil),                                      // 104: machine.SystemStat
	(*CPUStat)(nil),                                         // 105: machine.CPUStat
	(*SoftIRQStat)(nil),                                     // 106: machine.SoftIRQStat
	(*CPUFreqStatsResponse)(nil),                            // 107: machine.CPUFreqStatsResponse
	(*CPUsFreqStats)(nil),                                   // 108: machine.CPUsFreqStats
	(*CPUFreqStats)(nil),                                    // 109: machine.CPUFreqStats
	(*CPUInfoResponse)(nil),                                 // 110: machine.CPUInfoResponse
	(*CPUsInfo)(nil),                                        // 111: machine.CPUsInfo
	(*CPUInfo)(nil),                                         // 112: machine.CPUInfo
	(*NetworkDeviceStatsResponse)(nil),                      // 113: machine.NetworkDeviceStatsResponse
	(*NetworkDeviceStats)(nil),                              // 114: machine.NetworkDeviceStats
	(*NetDev)(nil),                                          // 115: machine.NetDev
	(*DiskStatsResponse)(nil),                               // 116: machine.DiskStatsResponse
	(*DiskStats)(nil),                                       // 117: machine.DiskStats
	(*DiskStat)(nil),                                        // 118: machine.DiskStat
	(*EtcdLeaveClusterRequest)(nil),                         // 119: machine.EtcdLeaveClusterRequest
	(*EtcdLeaveCluster)(nil),                                // 120: machine.EtcdLeaveCluster
	(*EtcdLeaveClusterResponse)(nil),                        // 121: machine.EtcdLeaveClusterResponse
	(*EtcdRemoveMemberRequest)(nil),                         // 122: machine.EtcdRemoveMemberRequest
	(*EtcdRemoveMember)(nil),                                // 123: machine.EtcdRemoveMember
	(*EtcdRemoveMemberResponse)(nil),                        // 124: machine.EtcdRemoveMemberResponse
	(*EtcdRemoveMemberByIDRequest)(nil),                     // 125: machine.EtcdRemoveMemberByIDRequest
	(*EtcdRemoveMemberByID)(nil),                            // 126: machine.EtcdRemoveMemberByID
	(*EtcdRemoveMemberByIDResponse)(nil),                    // 127: machine.EtcdRemoveMemberByIDResponse
	(*EtcdForfeitLeadershipRequest)(nil),                    // 128: machine.EtcdForfeitLeadershipRequest
	(*EtcdForfeitLeadership)(nil),                           // 129: machine.EtcdForfeitLeadership
	(*EtcdForfeitLeadershipResponse)(nil),                   // 130: machine.EtcdForfeitLeadershipResponse
	(*EtcdMemberListRequest)(nil),                           // 131: machine.EtcdMemberListRequest
	(*EtcdMember)(nil),                                      // 132: machine.EtcdMember
	(*EtcdMembers)(nil),                                     // 133: machine.EtcdMembers
	(*EtcdMemberListResponse)(nil),                          // 134: machine.EtcdMemberListResponse
	(*EtcdSnapshotRequest)(nil),                             // 135: machine.EtcdSnapshotRequest
	(*EtcdRecover)(nil),                                     // 136: machine.EtcdRecover
	(*EtcdRecoverResponse)(nil),                             // 137: machine.EtcdRecoverResponse
	(*EtcdAlarmListResponse)(nil),                           // 138: machine.EtcdAlarmListResponse
	(*EtcdAlarm)(nil),                                       // 139: machine.EtcdAlarm
	(*EtcdMemberAlarm)(nil),                                 // 140: machine.EtcdMemberAlarm
	(*EtcdAlarmDisarmResponse)(nil),                         // 141: machine.EtcdAlarmDisarmResponse
	(*EtcdAlarmDisarm)(nil),                                 // 142: machine.EtcdAlarmDisarm
	(*EtcdDefragmentResponse)(nil),                          // 143: machine.EtcdDefragmentResponse
	(*EtcdDefragment)(nil),                                  // 144: machine.EtcdDefragment
	(*EtcdStatusResponse)(nil),                              // 145: machine.EtcdStatusResponse
	(*EtcdStatus)(nil),                                      // 146: machine.EtcdStatus
	(*EtcdMemberStatus)(nil),                                // 147: machine.EtcdMemberStatus
	(*EtcdDowngradeValidateRequest)(nil),                    // 148: machine.EtcdDowngradeValidateRequest
	(*EtcdDowngradeValidateResponse)(nil),                   // 149: machine.EtcdDowngradeValidateResponse
	(*EtcdDowngradeValidate)(nil),                           // 150: machine.EtcdDowngradeValidate
	(*EtcdDowngradeEnableRequest)(nil),                      // 151: machine.EtcdDowngradeEnableRequest
	(*EtcdDowngradeEnableResponse)(nil),                     // 152: machine.EtcdDowngradeEnableResponse
	(*EtcdDowngradeEnable)(nil),                             // 153: machine.EtcdDowngradeEnable
	(*EtcdDowngradeCancelResponse)(nil),                     // 154: machine.EtcdDowngradeCancelResponse
	(*EtcdDowngradeCancel)(nil),                             // 155: machine.EtcdDowngradeCancel
	(*EtcdClusterDowngrade)(nil),                            // 156: machine.EtcdClusterDowngrade
	(*RouteConfig)(nil),                                     // 157: machine.RouteConfig
	(*DHCPOptionsConfig)(nil),                               // 158: machine.DHCPOptionsConfig
	(*NetworkDeviceConfig)(nil),                             // 159: machine.NetworkDeviceConfig
	(*NetworkConfig)(nil),                                   // 160: machine.NetworkConfig
	(*InstallConfig)(nil),                                   // 161: machine.InstallConfig
	(*MachineConfig)(nil),                                   // 162: machine.MachineConfig
	(*ControlPlaneConfig)(nil),                              // 163: machine.ControlPlaneConfig
	(*CNIConfig)(nil),                                       // 164: machine.CNIConfig
	(*ClusterNetworkConfig)(nil),                            // 165: machine.ClusterNetworkConfig
	(*ClusterConfig)(nil),                                   // 166: machine.ClusterConfig
	(*GenerateClientConfigurationRequest)(nil),              // 167: machine.GenerateClientConfigurationRequest
	(*GenerateClientConfiguration)(nil),                     // 168: machine.GenerateClientConfiguration
	(*GenerateClientConfigurationResponse)(nil),             // 169: machine.GenerateClientConfigurationResponse
	(*PacketCaptureRequest)(nil),                            // 170: machine.PacketCaptureRequest
	(*BPFInstruction)(nil),                                  // 171: machine.BPFInstruction
	(*NetstatRequest)(nil),                                  // 172: machine.NetstatRequest
	(*ConnectRecord)(nil),                                   // 173: machine.ConnectRecord
	(*Netstat)(nil),                                         // 174: machine.Netstat
	(*NetstatResponse)(nil),                                 // 175: machine.NetstatResponse
	(*MetaWriteRequest)(nil),                                // 176: machine.MetaWriteRequest
	(*MetaWrite)(nil),                                       // 177: machine.MetaWrite
	(*MetaWriteResponse)(nil),                               // 178: machine.MetaWriteResponse
	(*MetaDeleteRequest)(nil),                               // 179: machine.MetaDeleteRequest
	(*MetaDelete)(nil),                                      // 180: machine.MetaDelete
	(*MetaDeleteResponse)(nil),                              // 181: machine.MetaDeleteResponse
	(*ImageListRequest)(nil),                                // 182: machine.ImageListRequest
	(*ImageListResponse)(nil),                               // 183: machine.ImageListResponse
	(*ImagePullRequest)(nil),                                // 184: machine.ImagePullRequest
	(*ImagePull)(nil),                                       // 185: machine.ImagePull
	(*ImagePullResponse)(nil),                               // 186: machine.ImagePullResponse
	(*MachineStatusEvent_MachineStatus)(nil),                // 187: machine.MachineStatusEvent.MachineStatus
	(*MachineStatusEvent_MachineStatus_UnmetCondition)(nil), // 188: machine.MachineStatusEvent.MachineStatus.UnmetCondition
	(*NetstatRequest_Feature)(nil),                          // 189: machine.NetstatRequest.Feature
	(*NetstatRequest_L4Proto)(nil),                          // 190: machine.NetstatRequest.L4proto
	(*NetstatRequest_NetNS)(nil),                            // 191: machine.NetstatRequest.NetNS
	(*ConnectRecord_Process)(nil),                           // 192: machine.ConnectRecord.Process
	(*durationpb.Duration)(nil),                             // 193: google.protobuf.Duration
	(*common.Metadata)(nil),                                 // 194: common.Metadata
	(*common.Error)(nil),                                    // 195: common.Error
	(*anypb.Any)(nil),                                       // 196: google.protobuf.Any
	(*timestamppb.Timestamp)(nil),                           // 197: google.protobuf.Timestamp
	(common.ContainerDriver)(0),                             // 198: common.ContainerDriver
	(common.ContainerdNamespace)(0),                         // 199: common.ContainerdNamespace
	(*emptypb.Empty)(nil),                                   // 200: google.protobuf.Empty
	(*common.Data)(nil),                                     // 201: common.Data
}
var file_machine_machine_proto_depIdxs = []int32{
	0,   // 0: machine.ApplyConfigurationRequest.mode:type_name -> machine.ApplyConfigurationRequest.Mode
	193, // 1: machine.ApplyConfigurationRequest.try_mode_timeout:type_name -> google.protobuf.Duration
	194, // 2: machine.ApplyConfiguration.metadata:type_name -> common.Metadata
	0,   // 3: machine.ApplyConfiguration.mode:type_name -> machine.ApplyConfigurationRequest.Mode
	16,  // 4: machine.ApplyConfigurationResponse.messages:type_name -> machine.ApplyConfiguration
	1,   // 5: machine.RebootRequest.mode:type_name -> machine.RebootRequest.Mode
	194, // 6: machine.Reboot.metadata:type_name -> common.Metadata
	19,  // 7: machine.RebootResponse.messages:type_name -> machine.Reboot
	194, // 8: machine.Bootstrap.metadata:type_name -> common.Metadata
	22,  // 9: machine.BootstrapResponse.messages:type_name -> machine.Bootstrap
	2,   // 10: machine.SequenceEvent.action:type_name -> machine.SequenceEvent.Action
	195, // 11: machine.SequenceEvent.error:type_name -> common.Error
	3,   // 12: machine.PhaseEvent.action:type_name -> machine.PhaseEvent.Action
	4,   // 13: machine.TaskEvent.action:type_name -> machine.TaskEvent.Action
	5,   // 14: machine.ServiceStateEvent.action:type_name -> machine.ServiceStateEvent.Action
	50,  // 15: machine.ServiceStateEvent.health:type_name -> machine.ServiceHealth
	6,   // 16: machine.MachineStatusEvent.stage:type_name -> machine.MachineStatusEvent.MachineStage
	187, // 17: machine.MachineStatusEvent.status:type_name -> machine.MachineStatusEvent.MachineStatus
	194, // 18: machine.Event.metadata:type_name -> common.Metadata
	196, // 19: machine.Event.data:type_name -> google.protobuf.Any
	35,  // 20: machine.ResetRequest.system_partitions_to_wipe:type_name -> machine.ResetPartitionSpec
	7,   // 21: machine.ResetRequest.mode:type_name -> machine.ResetRequest.WipeMode
	194, // 22: machine.Reset.metadata:type_name -> common.Metadata
	37,  // 23: machine.ResetResponse.messages:type_name -> machine.Reset
	194, // 24: machine.Shutdown.metadata:type_name -> common.Metadata
	39,  // 25: machine.ShutdownResponse.messages:type_name -> machine.Shutdown
	8,   // 26: machine.UpgradeRequest.reboot_mode:type_name -> machine.UpgradeRequest.RebootMode
	194, // 27: machine.Upgrade.metadata:type_name -> common.Metadata
	43,  // 28: machine.UpgradeResponse.messages:type_name -> machine.Upgrade
	194, // 29: machine.ServiceList.metadata:type_name -> common.Metadata
	47,  // 30: machine.ServiceList.services:type_name -> machine.ServiceInfo
	45,  // 31: machine.ServiceListResponse.messages:type_name -> machine.ServiceList
	48,  // 32: machine.ServiceInfo.events:type_name -> machine.ServiceEvents
	50,  // 33: machine.ServiceInfo.health:type_name -> machine.ServiceHealth
	49,  // 34: machine.ServiceEvents.events:type_name -> machine.ServiceEvent
	197, // 35: machine.ServiceEvent.ts:type_name -> google.protobuf.Timestamp
	197, // 36: machine.ServiceHealth.last_change:type_name -> google.protobuf.Timestamp
	194, // 37: machine.ServiceStart.metadata:type_name -> common.Metadata
	52,  // 38: machine.ServiceStartResponse.messages:type_name -> machine.ServiceStart
	194, // 39: machine.ServiceStop.metadata:type_name -> common.Metadata
	55,  // 40: machine.ServiceStopResponse.messages:type_name -> machine.ServiceStop
	194, // 41: machine.ServiceRestart.metadata:type_name -> common.Metadata
	58,  // 42: machine.ServiceRestartResponse.messages:type_name -> machine.ServiceRestart
	9,   // 43: machine.ListRequest.types:type_name -> machine.ListRequest.Type
	194, // 44: machine.FileInfo.metadata:type_name -> common.Metadata
	64,  // 45: machine.FileInfo.xattrs:type_name -> machine.Xattr
	194, // 46: machine.DiskUsageInfo.metadata:type_name -> common.Metadata
	194, // 47: machine.Mounts.metadata:type_name -> common.Metadata
	68,  // 48: machine.Mounts.stats:type_name -> machine.MountStat
	66,  // 49: machine.MountsResponse.messages:type_name -> machine.Mounts
	194, // 50: machine.Version.metadata:type_name -> common.Metadata
	71,  // 51: machine.Version.version:type_name -> machine.VersionInfo
	72,  // 52: machine.Version.platform:type_name -> machine.PlatformInfo
	73,  // 53: machine.Version.features:type_name -> machine.FeaturesInfo
	69,  // 54: machine.VersionResponse.messages:type_name -> machine.Version
	198, // 55: machine.LogsRequest.driver:type_name -> common.ContainerDriver
	194, // 56: machine.LogsContainer.metadata:type_name -> common.Metadata
	76,  // 57: machine.LogsContainersResponse.messages:type_name -> machine.LogsContainer
	194, // 58: machine.Rollback.metadata:type_name -> common.Metadata
	79,  // 59: machine.RollbackResponse.messages:type_name -> machine.Rollback
	198, // 60: machine.ContainersRequest.driver:type_name -> common.ContainerDriver
	194, // 61: machine.Container.metadata:type_name -> common.Metadata
	82,  // 62: machine.Container.containers:type_name -> machine.ContainerInfo
	83,  // 63: machine.ContainersResponse.messages:type_name -> machine.Container
	87,  // 64: machine.ProcessesResponse.messages:type_name -> machine.Process
	194, // 65: machine.Process.metadata:type_name -> common.Metadata
	88,  // 66: machine.Process.processes:type_name -> machine.ProcessInfo
	198, // 67: machine.RestartRequest.driver:type_name -> common.ContainerDriver
	194, // 68: machine.Restart.metadata:type_name -> common.Metadata
	90,  // 69: machine.RestartResponse.messages:type_name -> machine.Restart
	198, // 70: machine.StatsRequest.driver:type_name -> common.ContainerDriver
	194, // 71: machine.Stats.metadata:type_name -> common.Metadata
	95,  // 72: machine.Stats.stats:type_name -> machine.Stat
	93,  // 73: machine.StatsResponse.messages:type_name -> machine.Stats
	194, // 74: machine.Memory.metadata:type_name -> common.Metadata
	98,  // 75: machine.Memory.meminfo:type_name -> machine.MemInfo
	96,  // 76: machine.MemoryResponse.messages:type_name -> machine.Memory
	100, // 77: machine.HostnameResponse.messages:type_name -> machine.Hostname
	194, // 78: machine.Hostname.metadata:type_name -> common.Metadata
	102, // 79: machine.LoadAvgResponse.messages:type_name -> machine.LoadAvg
	194, // 80: machine.LoadAvg.metadata:type_name -> common.Metadata
	104, // 81: machine.SystemStatResponse.messages:type_name -> machine.SystemStat
	194, // 82: machine.SystemStat.metadata:type_name -> common.Metadata
	105, // 83: machine.SystemStat.cpu_total:type_name -> machine.CPUStat
	105, // 84: machine.SystemStat.cpu:type_name -> machine.CPUStat
	106, // 85: machine.SystemStat.soft_irq:type_name -> machine.SoftIRQStat
	108, // 86: machine.CPUFreqStatsResponse.messages:type_name -> machine.CPUsFreqStats
	194, // 87: machine.CPUsFreqStats.metadata:type_name -> common.Metadata
	109, // 88: machine.CPUsFreqStats.cpu_freq_stats:type_name -> machine.CPUFreqStats
	111, // 89: machine.CPUInfoResponse.messages:type_name -> machine.CPUsInfo
	194, // 90: machine.CPUsInfo.metadata:type_name -> common.Metadata
	112, // 91: machine.CPUsInfo.cpu_info:type_name -> machine.CPUInfo
	114, // 92: machine.NetworkDeviceStatsResponse.messages:type_name -> machine.NetworkDeviceStats
	194, // 93: machine.NetworkDeviceStats.metadata:type_name -> common.Metadata
	115, // 94: machine.NetworkDeviceStats.total:type_name -> machine.NetDev
	115, // 95: machine.NetworkDeviceStats.devices:type_name -> machine.NetDev
	117, // 96: machine.DiskStatsResponse.messages:type_name -> machine.DiskStats
	194, // 97: machine.DiskStats.metadata:type_name -> common.Metadata
	118, // 98: machine.DiskStats.total:type_name -> machine.DiskStat
	118, // 99: machine.DiskStats.devices:type_name -> machine.DiskStat
	194, // 100: machine.EtcdLeaveCluster.metadata:type_name -> common.Metadata
	120, // 101: machine.EtcdLeaveClusterResponse.messages:type_name -> machine.EtcdLeaveCluster
	194, // 102: machine.EtcdRemoveMember.metadata:type_name -> common.Metadata
	123, // 103: machine.EtcdRemoveMemberResponse.messages:type_name -> machine.EtcdRemoveMember
	194, // 104: machine.EtcdRemoveMemberByID.metadata:type_name -> common.Metadata
	126, // 105: machine.EtcdRemoveMemberByIDResponse.messages:type_name -> machine.EtcdRemoveMemberByID
	194, // 106: machine.EtcdForfeitLeadership.metadata:type_name -> common.Metadata
	129, // 107: machine.EtcdForfeitLeadershipResponse.messages:type_name -> machine.EtcdForfeitLeadership
	194, // 108: machine.EtcdMembers.metadata:type_name -> common.Metadata
	132, // 109: machine.EtcdMembers.members:type_name -> machine.EtcdMember
	133, // 110: machine.EtcdMemberListResponse.messages:type_name -> machine.EtcdMembers
	194, // 111: machine.EtcdRecover.metadata:type_name -> common.Metadata
	136, // 112: machine.EtcdRecoverResponse.messages:type_name -> machine.EtcdRecover
	139, // 113: machine.EtcdAlarmListResponse.messages:type_name -> machine.EtcdAlarm
	194, // 114: machine.EtcdAlarm.metadata:type_name -> common.Metadata
	140, // 115: machine.EtcdAlarm.member_alarms:type_name -> machine.EtcdMemberAlarm
	10,  // 116: machine.EtcdMemberAlarm.alarm:type_name -> machine.EtcdMemberAlarm.AlarmType
	142, // 117: machine.EtcdAlarmDisarmResponse.messages:type_name -> machine.EtcdAlarmDisarm
	194, // 118: machine.EtcdAlarmDisarm.metadata:type_name -> common.Metadata
	140, // 119: machine.EtcdAlarmDisarm.member_alarms:type_name -> machine.EtcdMemberAlarm
	144, // 120: machine.EtcdDefragmentResponse.messages:type_name -> machine.EtcdDefragment
	194, // 121: machine.EtcdDefragment.metadata:type_name -> common.Metadata
	146, // 122: machine.EtcdStatusResponse.messages:type_name -> machine.EtcdStatus
	194, // 123: machine.EtcdStatus.metadata:type_name -> common.Metadata
	147, // 124: machine.EtcdStatus.member_status:type_name -> machine.EtcdMemberStatus
	150, // 125: machine.EtcdDowngradeValidateResponse.messages:type_name -> machine.EtcdDowngradeValidate
	194, // 126: machine.EtcdDowngradeValidate.metadata:type_name -> common.Metadata
	156, // 127: machine.EtcdDowngradeValidate.cluster_downgrade:type_name -> machine.EtcdClusterDowngrade
	153, // 128: machine.EtcdDowngradeEnableResponse.messages:type_name -> machine.EtcdDowngradeEnable
	194, // 129: machine.EtcdDowngradeEnable.metadata:type_name -> common.Metadata
	156, // 130: machine.EtcdDowngradeEnable.cluster_downgrade:type_name -> machine.EtcdClusterDowngrade
	155, // 131: machine.EtcdDowngradeCancelResponse.messages:type_name -> machine.EtcdDowngradeCancel
	194, // 132: machine.EtcdDowngradeCancel.metadata:type_name -> common.Metadata
	156, // 133: machine.EtcdDowngradeCancel.cluster_downgrade:type_name -> machine.EtcdClusterDowngrade
	158, // 134: machine.NetworkDeviceConfig.dhcp_options:type_name -> machine.DHCPOptionsConfig
	157, // 135: machine.NetworkDeviceConfig.routes:type_name -> machine.RouteConfig
	159, // 136: machine.NetworkConfig.interfaces:type_name -> machine.NetworkDeviceConfig
	11,  // 137: machine.MachineConfig.type:type_name -> machine.MachineConfig.MachineType
	161, // 138: machine.MachineConfig.install_config:type_name -> machine.InstallConfig
	160, // 139: machine.MachineConfig.network_config:type_name -> machine.NetworkConfig
	164, // 140: machine.ClusterNetworkConfig.cni_config:type_name -> machine.CNIConfig
	163, // 141: machine.ClusterConfig.control_plane:type_name -> machine.ControlPlaneConfig
	165, // 142: machine.ClusterConfig.cluster_network:type_name -> machine.ClusterNetworkConfig
	193, // 143: machine.GenerateClientConfigurationRequest.crt_ttl:type_name -> google.protobuf.Duration
	194, // 144: machine.GenerateClientConfiguration.metadata:type_name -> common.Metadata
	168, // 145: machine.GenerateClientConfigurationResponse.messages:type_name -> machine.GenerateClientConfiguration
	171, // 146: machine.PacketCaptureRequest.bpf_filter:type_name -> machine.BPFInstruction
	12,  // 147: machine.NetstatRequest.filter:type_name -> machine.NetstatRequest.Filter
	189, // 148: machine.NetstatRequest.feature:type_name -> machine.NetstatRequest.Feature
	190, // 149: machine.NetstatRequest.l4proto:type_name -> machine.NetstatRequest.L4proto
	191, // 150: machine.NetstatRequest.netns:type_name -> machine.NetstatRequest.NetNS
	13,  // 151: machine.ConnectRecord.state:type_name -> machine.ConnectRecord.State
	14,  // 152: machine.ConnectRecord.tr:type_name -> machine.ConnectRecord.TimerActive
	192, // 153: machine.ConnectRecord.process:type_name -> machine.ConnectRecord.Process
	194, // 154: machine.Netstat.metadata:type_name -> common.Metadata
	173, // 155: machine.Netstat.connectrecord:type_name -> machine.ConnectRecord
	174, // 156: machine.NetstatResponse.messages:type_name -> machine.Netstat
	194, // 157: machine.MetaWrite.metadata:type_name -> common.Metadata
	177, // 158: machine.MetaWriteResponse.messages:type_name -> machine.MetaWrite
	194, // 159: machine.MetaDelete.metadata:type_name -> common.Metadata
	180, // 160: machine.MetaDeleteResponse.messages:type_name -> machine.MetaDelete
	199, // 161: machine.ImageListRequest.namespace:type_name -> common.ContainerdNamespace
	194, // 162: machine.ImageListResponse.metadata:type_name -> common.Metadata
	197, // 163: machine.ImageListResponse.created_at:type_name -> google.protobuf.Timestamp
	199, // 164: machine.ImagePullRequest.namespace:type_name -> common.ContainerdNamespace
	194, // 165: machine.ImagePull.metadata:type_name -> common.Metadata
	185, // 166: machine.ImagePullResponse.messages:type_name -> machine.ImagePull
	188, // 167: machine.MachineStatusEvent.MachineStatus.unmet_conditions:type_name -> machine.MachineStatusEvent.MachineStatus.UnmetCondition
	15,  // 168: machine.MachineService.ApplyConfiguration:input_type -> machine.ApplyConfigurationRequest
	21,  // 169: machine.MachineService.Bootstrap:input_type -> machine.BootstrapRequest
	81,  // 170: machine.MachineService.Containers:input_type -> machine.ContainersRequest
	60,  // 171: machine.MachineService.Copy:input_type -> machine.CopyRequest
	200, // 172: machine.MachineService.CPUFreqStats:input_type -> google.protobuf.Empty
	200, // 173: machine.MachineService.CPUInfo:input_type -> google.protobuf.Empty
	200, // 174: machine.MachineService.DiskStats:input_type -> google.protobuf.Empty
	85,  // 175: machine.MachineService.Dmesg:input_type -> machine.DmesgRequest
	33,  // 176: machine.MachineService.Events:input_type -> machine.EventsRequest
	131, // 177: machine.MachineService.EtcdMemberList:input_type -> machine.EtcdMemberListRequest
	125, // 178: machine.MachineService.EtcdRemoveMemberByID:input_type -> machine.EtcdRemoveMemberByIDRequest
	119, // 179: machine.MachineService.EtcdLeaveCluster:input_type -> machine.EtcdLeaveClusterRequest
	128, // 180: machine.MachineService.EtcdForfeitLeadership:input_type -> machine.EtcdForfeitLeadershipRequest
	201, // 181: machine.MachineService.EtcdRecover:input_type -> common.Data
	135, // 182: machine.MachineService.EtcdSnapshot:input_type -> machine.EtcdSnapshotRequest
	200, // 183: machine.MachineService.EtcdAlarmList:input_type -> google.protobuf.Empty
	200, // 184: machine.MachineService.EtcdAlarmDisarm:input_type -> google.protobuf.Empty
	200, // 185: machine.MachineService.EtcdDefragment:input_type -> google.protobuf.Empty
	200, // 186: machine.MachineService.EtcdStatus:input_type -> google.protobuf.Empty
	148, // 187: machine.MachineService.EtcdDowngradeValidate:input_type -> machine.EtcdDowngradeValidateRequest
	151, // 188: machine.MachineService.EtcdDowngradeEnable:input_type -> machine.EtcdDowngradeEnableRequest
	200, // 189: machine.MachineService.EtcdDowngradeCancel:input_type -> google.protobuf.Empty
	200, // 190: machine.MachineService.Hostname:input_type -> google.protobuf.Empty
	200, // 191: machine.MachineService.Kubeconfig:input_type -> google.protobuf.Empty
	61,  // 192: machine.MachineService.List:input_type -> machine.ListRequest
	62,  // 193: machine.MachineService.DiskUsage:input_type -> machine.DiskUsageRequest
	200, // 194: machine.MachineService.LoadAvg:input_type -> google.protobuf.Empty
	74,  // 195: machine.MachineService.Logs:input_type -> machine.LogsRequest
	200, // 196: machine.MachineService.LogsContainers:input_type -> google.protobuf.Empty
	200, // 197: machine.MachineService.Memory:input_type -> google.protobuf.Empty
	200, // 198: machine.MachineService.Mounts:input_type -> google.protobuf.Empty
	200, // 199: machine.MachineService.NetworkDeviceStats:input_type -> google.protobuf.Empty
	200, // 200: machine.MachineService.Processes:input_type -> google.protobuf.Empty
	75,  // 201: machine.MachineService.Read:input_type -> machine.ReadRequest
	18,  // 202: machine.MachineService.Reboot:input_type -> machine.RebootRequest
	89,  // 203: machine.MachineService.Restart:input_type -> machine.RestartRequest
	78,  // 204: machine.MachineService.Rollback:input_type -> machine.RollbackRequest
	36,  // 205: machine.MachineService.Reset:input_type -> machine.ResetRequest
	200, // 206: machine.MachineService.ServiceList:input_type -> google.protobuf.Empty
	57,  // 207: machine.MachineService.ServiceRestart:input_type -> machine.ServiceRestartRequest
	51,  // 208: machine.MachineService.ServiceStart:input_type -> machine.ServiceStartRequest
	54,  // 209: machine.MachineService.ServiceStop:input_type -> machine.ServiceStopRequest
	40,  // 210: machine.MachineService.Shutdown:input_type -> machine.ShutdownRequest
	92,  // 211: machine.MachineService.Stats:input_type -> machine.StatsRequest
	200, // 212: machine.MachineService.SystemStat:input_type -> google.protobuf.Empty
	42,  // 213: machine.MachineService.Upgrade:input_type -> machine.UpgradeRequest
	200, // 214: machine.MachineService.Version:input_type -> google.protobuf.Empty
	167, // 215: machine.MachineService.GenerateClientConfiguration:input_type -> machine.GenerateClientConfigurationRequest
	170, // 216: machine.MachineService.PacketCapture:input_type -> machine.PacketCaptureRequest
	172, // 217: machine.MachineService.Netstat:input_type -> machine.NetstatRequest
	176, // 218: machine.MachineService.MetaWrite:input_type -> machine.MetaWriteRequest
	179, // 219: machine.MachineService.MetaDelete:input_type -> machine.MetaDeleteRequest
	182, // 220: machine.MachineService.ImageList:input_type -> machine.ImageListRequest
	184, // 221: machine.MachineService.ImagePull:input_type -> machine.ImagePullRequest
	17,  // 222: machine.MachineService.ApplyConfiguration:output_type -> machine.ApplyConfigurationResponse
	23,  // 223: machine.MachineService.Bootstrap:output_type -> machine.BootstrapResponse
	84,  // 224: machine.MachineService.Containers:output_type -> machine.ContainersResponse
	201, // 225: machine.MachineService.Copy:output_type -> common.Data
	107, // 226: machine.MachineService.CPUFreqStats:output_type -> machine.CPUFreqStatsResponse
	110, // 227: machine.MachineService.CPUInfo:output_type -> machine.CPUInfoResponse
	116, // 228: machine.MachineService.DiskStats:output_type -> machine.DiskStatsResponse
	201, // 229: machine.MachineService.Dmesg:output_type -> common.Data
	34,  // 230: machine.MachineService.Events:output_type -> machine.Event
	134, // 231: machine.MachineService.EtcdMemberList:output_type -> machine.EtcdMemberListResponse
	127, // 232: machine.MachineService.EtcdRemoveMemberByID:output_type -> machine.EtcdRemoveMemberByIDResponse
	121, // 233: machine.MachineService.EtcdLeaveCluster:output_type -> machine.EtcdLeaveClusterResponse
	130, // 234: machine.MachineService.EtcdForfeitLeadership:output_type -> machine.EtcdForfeitLeadershipResponse
	137, // 235: machine.MachineService.EtcdRecover:output_type -> machine.EtcdRecoverResponse
	201, // 236: machine.MachineService.EtcdSnapshot:output_type -> common.Data
	138, // 237: machine.MachineService.EtcdAlarmList:output_type -> machine.EtcdAlarmListResponse
	141, // 238: machine.MachineService.EtcdAlarmDisarm:output_type -> machine.EtcdAlarmDisarmResponse
	143, // 239: machine.MachineService.EtcdDefragment:output_type -> machine.EtcdDefragmentResponse
	145, // 240: machine.MachineService.EtcdStatus:output_type -> machine.EtcdStatusResponse
	149, // 241: machine.MachineService.EtcdDowngradeValidate:output_type -> machine.EtcdDowngradeValidateResponse
	152, // 242: machine.MachineService.EtcdDowngradeEnable:output_type -> machine.EtcdDowngradeEnableResponse
	154, // 243: machine.MachineService.EtcdDowngradeCancel:output_type -> machine.EtcdDowngradeCancelResponse
	99,  // 244: machine.MachineService.Hostname:output_type -> machine.HostnameResponse
	201, // 245: machine.MachineService.Kubeconfig:output_type -> common.Data
	63,  // 246: machine.MachineService.List:output_type -> machine.FileInfo
	65,  // 247: machine.MachineService.DiskUsage:output_type -> machine.DiskUsageInfo
	101, // 248: machine.MachineService.LoadAvg:output_type -> machine.LoadAvgResponse
	201, // 249: machine.MachineService.Logs:output_type -> common.Data
	77,  // 250: machine.MachineService.LogsContainers:output_type -> machine.LogsContainersResponse
	97,  // 251: machine.MachineService.Memory:output_type -> machine.MemoryResponse
	67,  // 252: machine.MachineService.Mounts:output_type -> machine.MountsResponse
	113, // 253: machine.MachineService.NetworkDeviceStats:output_type -> machine.NetworkDeviceStatsResponse
	86,  // 254: machine.MachineService.Processes:output_type -> machine.ProcessesResponse
	201, // 255: machine.MachineService.Read:output_type -> common.Data
	20,  // 256: machine.MachineService.Reboot:output_type -> machine.RebootResponse
	91,  // 257: machine.MachineService.Restart:output_type -> machine.RestartResponse
	80,  // 258: machine.MachineService.Rollback:output_type -> machine.RollbackResponse
	38,  // 259: machine.MachineService.Reset:output_type -> machine.ResetResponse
	46,  // 260: machine.MachineService.ServiceList:output_type -> machine.ServiceListResponse
	59,  // 261: machine.MachineService.ServiceRestart:output_type -> machine.ServiceRestartResponse
	53,  // 262: machine.MachineService.ServiceStart:output_type -> machine.ServiceStartResponse
	56,  // 263: machine.MachineService.ServiceStop:output_type -> machine.ServiceStopResponse
	41,  // 264: machine.MachineService.Shutdown:output_type -> machine.ShutdownResponse
	94,  // 265: machine.MachineService.Stats:output_type -> machine.StatsResponse
	103, // 266: machine.MachineService.SystemStat:output_type -> machine.SystemStatResponse
	44,  // 267: machine.MachineService.Upgrade:output_type -> machine.UpgradeResponse
	70,  // 268: machine.MachineService.Version:output_type -> machine.VersionResponse
	169, // 269: machine.MachineService.GenerateClientConfiguration:output_type -> machine.GenerateClientConfigurationResponse
	201, // 270: machine.MachineService.PacketCapture:output_type -> common.Data
	175, // 271: machine.MachineService.Netstat:output_type -> machine.NetstatResponse
	178, // 272: machine.MachineService.MetaWrite:output_type -> machine.MetaWriteResponse
	181, // 273: machine.MachineService.MetaDelete:output_type -> machine.MetaDeleteResponse
	183, // 274: machine.MachineService.ImageList:output_type -> machine.ImageListResponse
	186, // 275: machine.MachineService.ImagePull:output_type -> machine.ImagePullResponse
	222, // [222:276] is the sub-list for method output_type
	168, // [168:222] is the sub-list for method input_type
	168, // [168:168] is the sub-list for extension type_name
	168, // [168:168] is the sub-list for extension extendee
	0,   // [0:168] is the sub-list for field type_name
}

func init() { file_machine_machine_proto_init() }
func file_machine_machine_proto_init() {
	if File_machine_machine_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_machine_machine_proto_rawDesc), len(file_machine_machine_proto_rawDesc)),
			NumEnums:      15,
			NumMessages:   178,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_machine_machine_proto_goTypes,
		DependencyIndexes: file_machine_machine_proto_depIdxs,
		EnumInfos:         file_machine_machine_proto_enumTypes,
		MessageInfos:      file_machine_machine_proto_msgTypes,
	}.Build()
	File_machine_machine_proto = out.File
	file_machine_machine_proto_goTypes = nil
	file_machine_machine_proto_depIdxs = nil
}
